/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.apportal ;

import java.lang.reflect.Method;

import javax.servlet.http.*;

import org.azuki.bean.apportal.server.PortalAdvice;
import org.azuki.bean.apportal.server.console.AppPortalConsole;
import org.azuki.bean.apportal.server.imp.PService;
import org.azuki.bean.apportal.shared.ConstAppPortal;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;
import org.jdom.Element;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class AppPortalBean implements IAspect<PortalAdvice>
{	
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}

	@DependencyInjectionInfo(desc="The method used to get the binding URIs of the specified bean and method",
			                 param={Integer.class, String.class})
	public IMethodProxy<String[]> getBindingURI;
	
	// URI of console (only end)
	private String myURI = null ;
	final public String getURI()
	{
		// myURI must be found here, otherwise is too soon.
		if(myURI==null)
		{
			try
			{
				if(getBindingURI==null) 
					throw new Exception("Method getBindingURI has not been imported") ;		
				
				myURI = StringUtil.slashStart(getBindingURI.call(getMetadata().getID(), "service")[0]) ;
				
				Log.finest("My URI : ("+ myURI +")") ;
			}
			catch(Exception e)
			{
				Log.severe(e.getMessage()) ;
				myURI = "" ; 
			}
		}
		
		if("".equals(myURI))
			throw new NullPointerException("This aspect is not bound to any URI") ;

		return myURI;
	}
	
	private String myName = null ;
	final public String getName()   { return myName ; }
	
	private String myTitle = null ;
	final public String getTitle()	{ return myTitle ; }
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element lmen)
	throws Exception
	{
		Element para = JDOMUtil.getElement(lmen, ConstAppPortal.APPL) ;
		myName = para.getAttributeValue(ConstAppPortal.APPL_NAME, ConstAppPortal.UAN) ;
		myTitle = para.getAttributeValue(ConstAppPortal.APPL_TITLE, ConstAppPortal.UAT) ;	
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public PortalAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		getMetadata().getContract().checkBindable(tmeta.getBeanClass()) ;
		
		Method[] methods = ipc.getSelectedMethods() ;
		
		if(methods.length<1)
			throw new Exception("No method selected") ;

		if(methods.length>1)
			throw new Exception("This aspect can be applied to one method only") ;
	
		Integer bid = tmeta.getID() ;

		// uri cannot contains # define in ID
		String meth = methods[0].getName() ;

		String entry = null ;
		
		Element lmen = ipc.getParameter() ;
		if(lmen!=null)
		{
			lmen = lmen.getChild(ConstAppPortal.ENTY) ;
			if(lmen!=null) entry = lmen.getAttributeValue(ConstAppPortal.ENTY_NAME) ;
		}
		
		return new PortalAdvice(bid, ipc.getPointcutID(), meth, entry) ;
	}	

	@MethodInfo(desc="The contract implementation of the request dispatcher aspect (RequestDispatcher)")
	final public void service(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		try
		{ PService.p(this, srv, req, res) ; }
		catch(Exception ex)
		{ ex.printStackTrace() ; throw ex ; }
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{		
		AppPortalConsole.p(this, srv, req, res) ;
	}
}