/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.apportal.server.imp ;

import java.util.*;

import javax.servlet.http.HttpServletRequest;

import org.azuki.bean.apportal.AppPortalBean;
import org.azuki.bean.apportal.server.PortalAdvice;
import org.azuki.bean.apportal.server.imp.resource.RsrcPortal;
import org.azuki.common.console.html.HtmlBuilder;
import org.azuki.common.icon.CommonIcons;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class PPortalMenu
{
	final static public String p(AppPortalBean pa, HttpServletRequest req)
	throws Exception
	{
		try
		{
			return pp(pa, req) ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
			throw e ;
		}
	}
	
	final static public String pp(AppPortalBean pa, HttpServletRequest req)
	throws Exception
	{
		Collection<IPointcut> ipcs = Bean.getPointcuts(pa.getMetadata().getID()) ;
		
		GroupNode root = new GroupNode("ROOT") ;
		
		for(IPointcut ipc : ipcs)
		{
			PortalAdvice padv = (PortalAdvice)ipc.getAdvice() ; 
			if(padv==null) continue ;
			
			Integer bid = padv.getBeanID() ;
			
			IMetadata meta = null ;
			try
			{ meta = Bean.getMetadata(bid) ; }
			catch(Exception ex)
			{ continue ; }
			
			String beanGrup = meta.getGroup() ;
			if(beanGrup==null)
			{	root.addBean(meta, padv.getPointcutID(), padv.getEntry()) ;
				continue ; }
			
			GroupNode beanGrupNode = root ;
			
			StringTokenizer st = new StringTokenizer(beanGrup, ".") ;
			while(st.hasMoreTokens())
				beanGrupNode = beanGrupNode.addGroup(st.nextToken()) ;
			
			beanGrupNode.addBean(meta, padv.getPointcutID(), padv.getEntry()) ;
		}
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		bdr.println("<table>") ;
		
		for(GroupNode grup : root.getGroups())
		{
			bdr.println("<tr><td class=\"menu\">") ;
			
			bdr.println("<table class=\"menu\">") ;
			bdr.print("<tr><th class=\"menu\">").print(grup.name).println("</th></tr>") ;
			bdr.println("<tr><td>") ;
			bdr.println(grup.contentAsString(pa, req)) ;
			bdr.println("</td></tr>") ;
			bdr.println("</table>") ;
			
			bdr.println("</td></tr>") ;
		}
		
		for(BeanNode bean : root.getBeans())
		{
			bdr.println("<tr><td class=\"menu\">") ;
			bdr.println(bean.asString(pa, req)) ;
			bdr.println("</td></tr>") ;
		}
		
		bdr.println("</table>") ;
	
		return bdr.toString() ; //root.asString(ca, req) ;
	} 
	
	final static class GroupNode implements Comparable<GroupNode>
	{
		final private SortedSet<GroupNode> tasks = new TreeSet<GroupNode>() ;
		final private SortedSet<BeanNode> beans = new TreeSet<BeanNode>() ;
		
		final public String name ;
		
		public GroupNode(String name)
		{
			this.name = name ;
		}
		
		final public Collection<GroupNode> getGroups()
		{
			return Collections.unmodifiableCollection(tasks) ;
		}
		
		final public Collection<BeanNode> getBeans()
		{
			return Collections.unmodifiableCollection(beans) ;
		}
	
		final private GroupNode getGroup(String name)
		{
			for(GroupNode task : tasks)
				if(task.name.equals(name)) return task ;
			return null ;
		}
		
		final public GroupNode addGroup(String name)
		{
			GroupNode task = getGroup(name) ;
			if(task==null)
			{	task = new GroupNode(name) ;
				tasks.add(task) ; }
			
			return task ;
		}
		
		final public void addBean(IMetadata meta, Integer pcid, String entry)
		{
			beans.add(new BeanNode(meta, pcid, entry)) ;
		}
		
		@Override
		public boolean equals(Object obj)
		{
			return obj instanceof GroupNode
					? name.equals(((GroupNode)obj).name) 
					: super.equals(obj) ;
		}
		
		public String asString(AppPortalBean ca, HttpServletRequest req)
		{
			HtmlBuilder bdr = new HtmlBuilder() ;
			bdr.println("<table>") ;
			bdr.println("<tr>") ;
			bdr.print("<td><img src=\"").print(PortalUtil.mkRsrcURL(ca, req, RsrcPortal.FOLDER_PNG)).println("\"/></td>") ;
			bdr.print("<td>").print(name).println("</td>") ;
			bdr.println("</tr>") ;
			bdr.println("<tr>") ;
			bdr.println("<td/>") ;
			bdr.println("<td>") ;
			bdr.println(contentAsString(ca, req)) ;
			bdr.println("</td>") ;
			bdr.println("</tr>") ;
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}

		final public String contentAsString(AppPortalBean ca, HttpServletRequest req)
		{
			HtmlBuilder bdr = new HtmlBuilder() ;
			
			bdr.println("<table>") ;
			
			for(GroupNode task : tasks)
			{
				bdr.println("<tr>") ;
				bdr.println("<td>") ;
				bdr.println(task.asString(ca, req)) ;
				bdr.println("</td>") ;
				bdr.println("</tr>") ;
			}
			
			for(BeanNode bean : beans)
			{
				bdr.println("<tr>") ;
				bdr.println("<td>") ;
				bdr.println(bean.asString(ca, req)) ;
				bdr.println("</td>") ;
				bdr.println("</tr>") ;
			}
			
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}
		
		/*
		 * Comparable
		 */
		final public int compareTo(GroupNode tn)
		{
			return name.compareTo(tn.name) ;
		}
	}
	
	final static class BeanNode implements Comparable<BeanNode>
	{
		final public IMetadata 	meta ;	// bean metadata
		final public Integer 	pcid ;	// pointcut id
		final public String  	enty ;	// menu entry label
		
		public BeanNode(IMetadata meta, Integer pcid, String enty)
		{
			this.meta  = meta ;
			this.pcid  = pcid ;
			this.enty = enty!=null && !enty.trim().isEmpty() ? enty : Util.oneString(meta.getName(), meta.getLabel()) ; //TODO: +"."+pid ???
		}
		
		final public String asString(AppPortalBean ca, HttpServletRequest req)
		{
			String sico = meta.getIcon() ;
			if(StringUtil.isEmpty(sico)) sico = CommonIcons.BEAN_22 ;
			
			HtmlBuilder bdr = new HtmlBuilder() ;
			
			bdr.println("<table cellspacing=\"2\" cellpadding=\"0\">") ;
			bdr.println("<tr>") ;
			
			bdr.print("<td>") ;
			try
			{ bdr.print("<img src=\"").print(PortalUtil.mkRsrcURL(ca, req, sico)).print("\" width=\"22\" height=\"22\" />") ; }
			catch(Exception ex){}			
			bdr.print("</td>") ;
			bdr.println() ;
			
			bdr.print("<td>") ;
			bdr.print("<a href=\"").print(PortalUtil.mkBeanURL(ca, req, meta.getID(), pcid.toString())).print("\">") ;
			bdr.print(enty) ;
			bdr.print("</a>") ;
			bdr.print("</td>") ;
			bdr.println() ;
			
			bdr.println("</tr>") ;
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}

		/*
		 * Comparable
		 */
		final public int compareTo(BeanNode bn)
		{
			String s1 = Util.oneString(meta.getName(), meta.getLabel()) ;
			String s2 = Util.oneString(bn.meta.getName(), bn.meta.getLabel()) ;
			
			int comp = s1.compareTo(s2) ;
			return comp!=0 ? comp : pcid.compareTo(bn.pcid) ;
		}
	}
}