/**
 *   Created the 5 Decemberl 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu JUCKER (mathieu.jucker@azuki-framework.org)
 */
package org.azuki.bean.authenticator ;

import java.util.Hashtable;
import java.util.List;

import javax.naming.*;
//import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.azuki.bean.authenticator.shared.ConstLDAP;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.annotation.Export;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.Log;
import org.jdom.Element;
 
/**
 * <pre>
 *  LDAP authenticator is based on user LDAP connection. We just try to get an initialDirContext on LDAP to
 *  check user / password.
 *	 The Provider URL context
 *   
 *   LDAP user atttribute represente attributes that should be used to connect to the application. it's defined like this
 *      uid=? or
 *      uid=?,application=app1
 *      
 *   java.naming.provider.url is directly build with paramters serverurl and basedn. you don't need to add it as a
 *   parameter.
 *    
 * 	<para>
 * 		<ldap serverurl="ldap://localhost:389" basedn="o=societedemo,c=fr" user="uid=?"/>
 * 		<context type="java.naming.factory.initial" value="com.sun.jndi.ldap.LdapCtxFactory"/>
 * 		<context type="java.naming.security.authentication" value="simple"/>
 *         ...
 *     </para>
 *  </pre>    
 */
public class LDAPAuthenticatorBean
{

	private String LDAP_BASE_DN;
	private String LDAP_USER;
	private Hashtable<String,String> env ;
	
	@SuppressWarnings("unchecked")
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		env = new Hashtable<String,String>();
		
		Element ldap = para.getChild(ConstLDAP.LDAP) ;
		if(ldap==null) throw new Exception("Missing LDAP configuration") ;
		
		LDAP_BASE_DN = ldap.getAttributeValue(ConstLDAP.BASE_DN) ;
		if(StringUtil.isEmpty(LDAP_BASE_DN)) throw new Exception("Missing LDAP base dn attribute") ;
		
		String LDAP_SERVER_URL = ldap.getAttributeValue(ConstLDAP.SERVER_URL) ;
		if(StringUtil.isEmpty(LDAP_SERVER_URL)) throw new Exception("Missing LDAP provider URL attribute") ;
		// build server provider url
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(LDAP_SERVER_URL);
		if (!LDAP_SERVER_URL.endsWith("/")) bdr.append("/");
		bdr.append(LDAP_BASE_DN);
		env.put(Context.PROVIDER_URL,bdr.toString());
		
		LDAP_USER = ldap.getAttributeValue(ConstLDAP.LDAP_LOGIN) ;
		if(StringUtil.isEmpty(LDAP_USER)) throw new Exception("Missing LDAP user authentifier attribute") ;
		
		List<Element> list = para.getChildren(ConstLDAP.CONTEXT) ;
		for(Element temp : list)
		{
			String type = temp.getAttributeValue(ConstLDAP.TYPE) ;
			String valu = temp.getAttributeValue(ConstLDAP.VALUE) ;
			env.put(type, valu) ;
		}
	}	
	
	@Export
	@MethodInfo(desc="Check if the user's password against configured entries.", retu="true if the password is correct")
	final public boolean check(String user, String pass)  
	throws Exception
	{
		try
		{
			Log.info("LDAPAuthenticator:: check user acces");
			LDAP_USER = LDAP_USER.replace("?",user)+","+LDAP_BASE_DN;
			
			env.put( Context.SECURITY_PRINCIPAL, LDAP_USER);
			env.put( Context.SECURITY_CREDENTIALS, pass );
			
			Log.finest("LDAPAuthenticator:: Context = "+env.toString());


			// connexion au LDAP
			/*DirContext ctxtDir = */new InitialDirContext( env );
			Log.finest("LDAPAuthenticator::connection successfull");
			return true;
		}
		catch(AuthenticationException ae)
		{
			Log.info("LDAPAuthenticator:: Invalid user / password");
			return false;
		}
		catch(NamingException ne)
		{
			ne.printStackTrace();
			throw ne;
		}
	}
}