/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.authenticator ;

import java.util.*;

import org.azuki.bean.authenticator.shared.ConstSimple;
import org.azuki.framework.annotation.*;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
public class SimpleAuthenticatorBean
{
	final private Map<String,String> ups = new HashMap<String, String>() ;
	
	@SuppressWarnings("unchecked")
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		List<Element> users = para.getChildren(ConstSimple.USER) ;
		for(Element user : users)
		{
			String name = user.getAttributeValue(ConstSimple.USER_NAME) ;
			if(name==null) continue ;
			
			ups.put(name, user.getAttributeValue(ConstSimple.USER_PASS, "")) ;
		}
	}
	
	@Export
	@MethodInfo(desc="Check if the user's password against configured entries.", retu="true if the password is correct")
	final public boolean check(String user, String pass)
	{
		String check = ups.get(user) ;
		return check!=null && check.equals(pass) ;
	}
}