/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.basicrpc.shared.contract ;

import java.lang.reflect.*;
import java.util.*;

import org.azuki.bean.basicrpc.shared.UtilBasicRpc;
import org.azuki.common.util.ListUtil;
import org.azuki.framework.aop.IContract;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class BasicRpcContract implements IContract
{
	final public void checkBindable(Class beanclass)
	throws Exception
	{
		Method[] candidates = getMethodCandidates(beanclass) ;
		if(candidates==null || candidates.length==0)
			throw new Exception("At least one method should contains no arguments or with java.lang types or array") ;
	}
	
	// Fields
	final public Selection getFieldSelection()				{ return Selection.NONE ; }
	final public Field[] getFieldCandidates(Class beanclass){ return null ; }

	// Methods
	final public Selection getMethodSelection()				{ return Selection.AT_LEAST_ONE ; }
	final public Method[] getMethodCandidates(Class beanclass)
	{
		List<Method> ms = ListUtil.asList(beanclass.getMethods()) ;
		List<Method> todel = new ArrayList<Method>() ;
		
		for(Method m : beanclass.getMethods())
		{
			// check arguments types
			Class<?>[] typs = m.getParameterTypes() ;
			if(typs.length==0) continue ;
			
			for(Class<?> type : typs)
			{
				if(!UtilBasicRpc.isValidType(type))
				{	todel.add(m) ;
					break ; }
			}
		}
		
		ms.removeAll(todel) ;
		
		return ms.toArray(new Method[ms.size()]) ;
	}
}