/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.chrono;

import javax.servlet.http.*;

import org.azuki.bean.chrono.server.ChronoAdvice;
import org.azuki.bean.chrono.server.console.ChronoConsole;
import org.azuki.bean.chrono.server.shell.ChronoShell;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;

final public class ChronoBean implements IAspect<ChronoAdvice>
{
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public ChronoAdvice createAdvice(IPointcut ipc) 
	throws Exception
	{
		return new ChronoAdvice(this, ipc.getSelectedMethods()) ;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		ChronoConsole.serviceHtml(this, servlet, req, res) ;
	}

	@MethodInfo(desc="Build the list of all ChronoBean advice",
				retu="The advices listing")
	final public String listChronometers()
	throws Exception
	{
		return ChronoShell.listPointcuts(this) ;
	}
	
	// to start and stop the chronometer anyway
	// auto started
	private boolean isRunning = true;
	
	@Export
	@MethodInfo(desc="Starts the chronometer")
	final public void start()
	{
		isRunning = true ;
	}
	
	@Export
	@MethodInfo(desc="Stops the chronometer")
	final public void stop()
	{
		isRunning = false ;
	}
	
	@Export
	@MethodInfo(desc="Return the current chronometer status",
				retu="TRUE if chronometer is running, FALSE otherwise")
	final public boolean isRunning() 
	{ 
		return isRunning ; 
	}
	
	@Export
	@MethodInfo(desc="Reset the chronometers values")
	final public void reset()
	throws Exception
	{
		IMetadata meta = Bean.getMetadata(this) ;
		
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			ChronoAdvice adv = (ChronoAdvice)pc.getAdvice() ;
			if(adv==null) continue ;
			
			adv.getLines().clear() ;
		}
	}
}