/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.chrono.server.console;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.chrono.ChronoBean;
import org.azuki.bean.chrono.server.ChronoAdvice;
import org.azuki.bean.chrono.server.imp.ChronoLine;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.html.renderer.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

public class ChronoConsole
{
	final static public String[] tableHeaders = {"Bean", "Method", "Calls", "Last Call", "Min.", "Max.", "Total", "Average"} ;
	
	final public static void serviceHtml(ChronoBean aspect, HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{		
		List<List> data = new ArrayList<List>() ;
		
		IMetadata meta = Bean.getMetadata(aspect) ;
		
		// aspect properties
		Map<String,String> pmap = Collections.singletonMap("Is running", String.valueOf(aspect.isRunning())) ;
		Table atab = HtmlFactory.propertyList("Properties", pmap) ;
		
		// aspect pointcuts
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			ChronoAdvice advc = (ChronoAdvice)pc.getAdvice() ;
			if(advc==null) continue ;
			
			Map<String,ChronoLine> chronoMethod = advc.getLines() ;
			for(Map.Entry<String,ChronoLine> ety : chronoMethod.entrySet())
			{
				String meth = ety.getKey() ;
				ChronoLine chrl = ety.getValue() ;
			
				IMetadata tmeta = pc.getTargetMetadata() ;
				
				List row = ListUtil.toList(	Util.oneString(tmeta.getName(), tmeta.getLabel()),
											meth,
											Integer.valueOf(chrl.getCalls()),
											new Date(chrl.getLastCall()),
											Long.valueOf(chrl.getMin()),
											Long.valueOf(chrl.getMax()),
											Long.valueOf(chrl.getTotal()),
											Long.valueOf(chrl.getAverage())) ;
				
				data.add(row) ;
			}
		}
		
		Table ptab = HtmlFactory.sortedTable(req, data, tableHeaders) ;
		ptab.setColumnRenderer(4, new MinuteCellRenderer()) ;
		ptab.setColumnRenderer(5, new MinuteCellRenderer()) ;
		ptab.setColumnRenderer(6, new HourCellRenderer()) ;
		ptab.setColumnRenderer(7, new MinuteCellRenderer()) ;
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		bdr.print("<h2>").print(Util.oneString(meta.getName(), meta.getLabel())).print("</h2>") ;
		bdr.println(atab) ;
		bdr.println("<br/>") ;
		bdr.println(ptab) ;
		
		HttpResponse.print(res, bdr) ;
	}
}
