/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.chrono.server.shell ;

import java.text.DateFormat;
import java.util.*;

import org.azuki.bean.chrono.ChronoBean;
import org.azuki.bean.chrono.server.ChronoAdvice;
import org.azuki.bean.chrono.server.imp.ChronoLine;
import org.azuki.bean.chrono.shared.ConstChrono;
import org.azuki.common.console.renderer.*;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

final public class ChronoShell
{
	final static public String listPointcuts(ChronoBean aspt)
	throws Exception
	{
		StringBuilder bdr = new StringBuilder() ;
		
		// state of Chronometer
		if(aspt.isRunning()) 	bdr.append(ConstChrono.MSG_RUNNING).append('\n') ;
		else 					bdr.append(ConstChrono.MSG_STOPPED).append('\n') ;
		
		// headers
		StringBuilder tmp = new StringBuilder() ;
		tmp.append(ShellUtil.rpad("Bean", ConstChrono.SZ_BEAN)) ;
		tmp.append(ShellUtil.rpad("Method", ConstChrono.SZ_METHOD)) ;
		tmp.append(ShellUtil.rpad("Calls", ConstChrono.SZ_CALL)) ;
		tmp.append(ShellUtil.rpad("Last", ConstChrono.SZ_LAST)) ;
		tmp.append(ShellUtil.rpad("Min", ConstChrono.SZ_MIN)) ;
		tmp.append(ShellUtil.rpad("Max", ConstChrono.SZ_MAX)) ;
		tmp.append(ShellUtil.rpad("Total", ConstChrono.SZ_TOTAL)) ;
		tmp.append(ShellUtil.rpad("Average", ConstChrono.SZ_AVG)) ;
		tmp.append("\n") ;
		tmp.append(ShellUtil.rpad("", tmp.length(), '-')) ;
		tmp.append("\n") ;
		
		bdr.append(tmp) ;
		
		// lines
		IMetadata meta = Bean.getMetadata(aspt) ;
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			ChronoAdvice adv = (ChronoAdvice)pc.getAdvice() ;
			if(adv==null) continue ;
			
			IMetadata tmeta = pc.getTargetMetadata() ;
			
			String bkey = Util.oneString(tmeta.getName(), tmeta.getLabel()) ;
				
			for(Map.Entry<String,ChronoLine> ety : adv.getLines().entrySet())
			{
				String     meth = ety.getKey() ;
				ChronoLine chrl = ety.getValue() ;
				
				bdr.append(ShellUtil.rpad(bkey, ConstChrono.SZ_BEAN)) ;
				bdr.append(ShellUtil.rpad(meth, ConstChrono.SZ_METHOD)) ;
				bdr.append(ShellUtil.rpad(String.valueOf(chrl.getCalls()), ConstChrono.SZ_CALL)) ;
				bdr.append(ShellUtil.rpad(DateFormat.getInstance().format(new Date(chrl.getLastCall())), ConstChrono.SZ_LAST)) ;
				bdr.append(ShellUtil.rpad(MinuteRenderer.get(chrl.getMin()), ConstChrono.SZ_MIN)) ;
				bdr.append(ShellUtil.rpad(MinuteRenderer.get(chrl.getMax()), ConstChrono.SZ_MAX)) ;
				bdr.append(ShellUtil.rpad(HourRenderer.get(chrl.getTotal()), ConstChrono.SZ_TOTAL)) ;
				bdr.append(ShellUtil.rpad(MinuteRenderer.get(chrl.getAverage()), ConstChrono.SZ_AVG)) ;
				bdr.append("\n") ;
			}
		}
		
		return bdr.toString() ;
	}	
}