/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.command ;

import org.azuki.framework.annotation.*;
import org.azuki.framework.command.ICommand;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;


/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class CommandExecutionBean 
{
	// output to notify the begining of the process
	final static private String outputStart = "trace_start";
	
	// output to notify the end of the process
	final static private String outputEnd   = "trace_end";
	
	// output to notify the success execution of the process
	final static private String outputSuccess = "success";
	
	// output to notify that the process failed
	final static private String outputErr     = "err";
	
	// EventPublisher injection
	@DependencyInjectionInfo(desc="The method used to publish a message to an output connector",
							 param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish;
	
	// Event Handler Contract
	@Export
	@MethodInfo(desc="Execute a Job: a Job is an instance of a class that must implement the ICommand interface")
	final public void processCommand(
			@ArgInfo(name="command",desc="a Job") ICommand<?> cmd)
	throws Exception
	{		
		
		Log.info("receive event to execute a command:"+cmd.toString());
		
		// publish start execution
		publish.call(this, outputStart, "start command "+cmd.toString());
		
		try
		{
			// process
			cmd.process() ;
			
			// send event to success output
			publish.call(this,outputSuccess,cmd.toString());
		}
		catch(Exception e)
		{
			// send event to err output
			publish.call(this,outputErr,"ERROR "+cmd.toString());
			publish.call(this,outputEnd,"end command "+cmd.toString());
			e.printStackTrace();
			throw e;
		}
		// Event Publisher : publish stop execution
		publish.call(this,outputEnd,"end command "+cmd.toString());
	}
}