/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.command.server.imp ;

import org.azuki.framework.command.ICallback;

// This Object is only used as a callback  to synchronize 
// a command execution with a client thread.

final public class SyncBack<T> implements ICallback<T>
{
	private boolean isc = false ;
	final public boolean isComplete() { return isc; }
	
	private T res  = null ; // if OK
	private Exception e = null ; // Error if problem
	
	final synchronized public T waitFor()	
	throws Exception
	{
		if(!isc) wait() ; // waiting for a result or a Exception
		if(e!=null) throw e ;
		return res ;
	}

	final synchronized public T waitFor(long milli)	
	throws Exception
	{
		if(!isc) wait(milli) ; // waiting for a result but return null if task as not complete
		if(e!=null) throw e ;
		return res ;
	}
	
	final synchronized public void setResult(T res)
	{
		isc = true ;
		this.res = res ; // Normally null, else the exception that will be throw by the servlet
		notifyAll() ;
	}

	final synchronized public void setError(Exception e)
	{
		isc = true ;
		this.e = e ; // Normally null, else the exception that will be throw by the servlet	
		notifyAll() ;
	}
}
