/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.command.server.imp.core;

import org.azuki.bean.command.server.imp.queue.OQ;
import org.azuki.framework.context.ThreadContext;

// Thread Pool

final public class TPool
{
	static public int MAX_THREAD = 999 ; // Modfied by KernelBean
	static public int MIN_SPARE  = 4  ; // Idem Tomcat, modified by KernelBean
	static public int MAX_SPARE  = 20 ; // Idem Tomcat, modified by KernelBean
	
	static transient private int TN = 0 ; // Current Thread number
	
	final static public OQ<Thd> sp = new OQ<Thd>(); // Spare Threads
	
	final static public int threads() { return TN ; }  // Total Thread number (running & spare)
	final static public int running() { return TN - sp.size(); } 	

	final static public void init()
	{
		for(int i=0; i<MIN_SPARE; i++)
		{
			sp.put(create()) ;
		}
	}
	
	final static public void setTheadPoolValues(int maxth, int minsp, int maxsp)
	{
		MAX_THREAD = maxth ;
		
		// Security 
		if(maxsp<(minsp+5))
			maxsp = minsp + 5 ;

		MIN_SPARE = minsp ;
		MAX_SPARE = maxsp ;
	}
	
	// Get one thread from pool
	final static public Thd remove()
	throws Exception
	{
		if(TN>=MAX_THREAD)
		    return sp.remove() ; // Block until one thread available 

		Thd thd = sp.remove(0); // Non blocking remove
		
		if(thd!=null)
		{
		    if(sp.size()<MIN_SPARE)
		        TPM.create() ; // Delagate new thread creation
		}
		else
		{
		    // Rare, but possible if thread creation too long.
		    thd = create() ;
		    // A Thread has been created and will be put in the pool.
		}
		    
		return thd ;
	}

	// Call by kernel Manager
	final static public void preCreate()
	{
	    back(create()) ;
	}
	
	// Create and start thread
	final static private Thd create()
	{
		Thd thd = new Thd();
		TN++ ;
		
    	Thread thread = new Thread(thd); 

    	ThreadContext.startThread(thread) ;
    			
		return thd ;
	}
	
	final static public void back(Thd thd)
	{
		if(sp.size()>=MAX_SPARE) 
		{
			thd.end() ; // Thread must stop
			TN-- ; // dec current thread number
		}
		else sp.put(thd) ;
	}
}		