/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.command.server.imp.queue ;

import org.azuki.bean.command.server.imp.core.CPU;
import org.azuki.framework.command.*;
import org.azuki.framework.command.ICommand.Priority;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class CmdQ implements IQueue
{
	final private CPU cpu ;
	
	private String name ;

	final private OQP<ICommand> todo = new OQP<ICommand>(); // Pool of command wrapper
	final private OQ<QRd>  rds  = new OQ<QRd>() ; // Pool of readers 

	private int count   = 0 ; // Compteur de job traites (et non pas recus).
	private int maxread = 1 ; // default
	private int current = 0 ; // Current number of readers

	private boolean active = true; //
	private boolean dead = false; // Destroyed, readers must be discard
	
	public CmdQ(CPU cpu, String name)
	{
		this.cpu = cpu ;
		this.name = name ;
	}

	final public synchronized void start()
	{
		active=true ;
		if(todo.size() > 0) pp() ;
	}

	final public void stop() { active = false ; }

	final public int  getMaxThread() { return maxread; }
	
	public void setMaxThread(int maxread)
	{
		this.maxread = maxread ;
	}

	final public String getName() { return name ; }

	public void setLimit(int max)
	{
		todo.setLimit(max) ; // synchronized 
	}

	final public int getLimit()
	{
		return todo.getLimit();
	}
	
	public void setShift(boolean shift)
	{
		todo.setShift(shift) ; // sync
	}
	
    final public void put(ICommand cmd)
    {
    	put(cmd, Priority.NORMAL) ; // sync
    }
	  
	final public synchronized void put(ICommand cmd, Priority pty)
	{
    	todo.put(cmd, pty); // CmdCBack to preserve the context.
		pp() ;
	}
	
	final private void pp() // already synchronized
	{
		if(!active) return ;
		if(dead) return ;
		
		QRd rd = rds.remove(0) ; // non blocant

		if(rd!=null) // Not all reader in use
		{	
			cpu.put(rd); // request for another thread
			return;
		}
		
		// Lazy, Creating new reader if max is not reached
		if(current < maxread)
		{	
			rd = new QRd(this) ; 
			current++;
			cpu.put(rd); // request for another thread
		}

		// else nothing to do
	}

	final public synchronized ICommand read(QRd rd)
	{
		if(!active)
		{
			rds.put(rd) ; // Keep the reader !
			return null ;
		}

		ICommand ic = todo.remove(0) ; // non blocant

		if(ic == null) // nothing to process, store de reader instance
		{
			if(current>maxread || dead) current--; // ne devrai pas arriver sauf modif maxread
			else // cas normal
			{
				rds.put(rd) ;
			}
		}
		else count++;

		return ic ; // if null, reader must stop
	}

	final public int queuing()      { return todo.size() ; }
	final public int processing()   { return (current-rds.size());  }
	final public int done()         { return count - processing(); }
	final public boolean isActive() { return active; }

	final synchronized public void destroy()
	{
		dead = true;
		active=false ;
		rds.clear() ;
		todo.clear() ;
	}
	
	final public synchronized void reset() { todo.clear(); }
}
