/*
 *   Created the 1 August 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.compiler.server.imp ;

import java.util.*;

import org.azuki.framework.context.Log;


/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class RAMClassLoader extends ClassLoader
{
	// Hashtable for synchronization
	private Map<String, RAMFileObject> jfos = new Hashtable<String, RAMFileObject>() ;
	
	public RAMClassLoader()
	{
		super(Thread.currentThread().getContextClassLoader()) ; 
	}
	
	final public void put(String name, RAMFileObject jfo)
	{
		// Store RAMJF instead of byte[] because RAMJF not yet compiled
		Log.finest("Adding "+name+" object to RAMClassLoader") ;
		jfos.put(name, jfo) ;
	}
	
	@Override
	protected Class<?> findClass(String name)
	throws ClassNotFoundException
	{
		Log.finest("RAMClassLoader looking for : "+name) ;
		RAMFileObject jfo = jfos.get(name) ;
		
		if(jfo==null) return super.findClass(name) ;
		
		String pkag = null ;
		int idx = name.lastIndexOf('.') ;
		if(idx!=-1) pkag = name.substring(0, idx) ;
		
		if(pkag!=null)
		{
			try
			{
				definePackage(pkag, null, null, null, null, null, null, null) ;
			}
			catch(Exception ex) {}
		}

		byte[] bytes = jfo.getBytes() ;
		return defineClass(name, bytes, 0, bytes.length);
	}
}