/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */


package org.azuki.bean.context.server.imp ;

import java.util.Map;
import java.util.Map.Entry;

import org.azuki.common.util.Util;
import org.azuki.framework.context.*;
import org.azuki.framework.context.IContext.IStub;
import org.azuki.framework.object.*;


/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class PC 
{  	
    final static public Object lookup(Map<String, IStub> stubs, String key, Object... args)
    throws Exception
    {
    	IStub stub = stubs.get(key) ;

    	// Already bound
    	if(stub!=null) return stub.getObject();

    	String name = key ;
    	int idx = key.indexOf(Util.beanSeparatorChar) ;
    	if(idx!=-1)
    		name = key.substring(0, idx) ;
    	
    	// Using factory
    	stub = createContextObject(name, args) ;
    	
    	// Factory not found ?
    	// Null value cannot be bound in the stubs Hastable
    	if(stub==null) return null ;
    	
		// Bind in current thread context, using key
		stubs.put(key, stub) ;

		return stub.getObject() ;
    }	
	
	// Finalize, if already bound
	// finalize(objs.get(name)) ;
	    	
    
    // This method create the object using the factory,
	final static private IStub createContextObject(String name, Object... args)
    throws Exception
    {
        // Create resource
        IObjectFactory objfac = Locator.getObjectFactory(name) ;        
        if(objfac==null) return null ; // No resourcefactory -> return null
       
		Object obj = null ;
		try
		{
			obj = objfac.createObject(args) ;
		}
        catch(Throwable t)
        { 
        	if(IContext.OBJECT_LOGGER.equals(name)) System.out.println("> "+t.getMessage()) ;
        	else									Log.severe(t.getMessage());
        	return null; // Could not create object
        }

        ILifecycle life = null ;
        if(objfac instanceof ILifecycleFactory)
        {
			try
			{
				life = ((ILifecycleFactory)objfac).getLifeCycle() ;
			}
	        catch(Throwable t)
	        {
	        	if(IContext.OBJECT_LOGGER.equals(name)) System.out.println("> "+t.getMessage()) ;
	        	else									Log.severe(t.getMessage());
	        } // No lifecycle for this object
        }
        
        return new Stub(obj, life) ;
    }	
    

    
    
	@SuppressWarnings("unchecked")
	final static public void copy(C oldc, C newc) 
	{
		for(Entry<String, IStub> entry : oldc.getStubs().entrySet())
		{
			String name = entry.getKey() ;
			IStub stub = entry.getValue() ;
			
			ILifecycle life = stub.getLifecycle() ;
			if(life==null) continue ; // No lifecycle, object dropped in new context
			
			try
			{
				Object forked = life.fork(stub.getObject()) ;
				
				if (forked==null) continue ; // null value are not bound ?

				newc.bind(name, forked, life) ;
			}
			catch(Throwable t)
			{ Log.severe(t.getMessage()); }
		}
	}
	
    
    
	final static public void end(Map<String, IStub> objs)
    {
		for(Entry<String, IStub> entry : objs.entrySet())
			finalize(entry.getValue()) ;
        
        // Remove all mapping from this map.
        objs.clear() ;
    }	
	
    @SuppressWarnings("unchecked")
	final static public void finalize(IStub stub)
    {
    	if(stub==null) return ; // nothing to do
    	
		ILifecycle life = stub.getLifecycle() ;
		if(life==null) return ; // No lifecycle, object will be simply dropped

		try { life.end(stub.getObject()); }  
        catch(Throwable t) { Log.severe(t.getMessage()); }    	
    }
}