/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.context.server.imp;

import org.azuki.bean.context.server.imp.TCxMngr.THREAD_ORIGIN;
import org.azuki.common.util.Counter;


// Thread's Context 
////////////////////

// Le thread context contient les informations 
// liee au context d'execution Cx
// ces infos sont effacées a la fin d'un traitement
// si pas sauvegarder (cas de la session)

// Il contient en plus des infos concernant le
// thread lui même. (nfo, run)

final public class TCx 
{
	static private Counter count = new Counter(0);

	final static public String IDLE = "idle" ; // Ex thread info
	
	// Thread number act as a second ID for threads 
	// because hashcode is too long for debug purpose.
	public int number  = count.next(); 
	public String nfo  = IDLE ;	// Thread info
 	public boolean run = false ; // false=waiting, true=running	
	
	
	final public Cx get()
	{ 
		if(cx!=null) return cx ;
		cx = new Cx() ;
		return cx ;
	}
	
	private Cx cx = null;
	final public void set(Cx cx) { this.cx = cx; }
	
	final public THREAD_ORIGIN type ; // AZUKI or EXTERN ...
	final public Integer tid ; // id
	
	public TCx(THREAD_ORIGIN type, Integer tid)
	{
		this.type = type ;
		this.tid  = tid  ;
	}
	
	final public Integer getID() { return tid; } 
	
	// use for :
	//  - reusability of default thread context only
	// -  Called by Thd class at the end of Job Execution.
	//////////////////////////////////////////////////
   
	// Called at job's end :
	// - In Bridge servler for HTTP request (J2EE or TOMCAT context)
	// - In Thd for Job process (AZUKI)
	// - In JMX
	// - In PHP bridge
	
	final public void reset() 
    {
    	nfo = IDLE ; // nothing to do
    	run = false ; // waiting (idle)
    	
     	if(cx!=null) cx.end() ;
     	
     	cx  = null ; // Unbind old context for garbage
     	
		// Thread that has not been created by the framework
		// are not kept in the table,
	    // because AZUKI does not manage the lifecycle.
		if(THREAD_ORIGIN.AZUKI!=type) TCxMngr.remove(tid) ;
    }
}
