/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.crypto ;

import org.azuki.bean.crypto.shared.ConstCrypto;
import org.azuki.common.util.Crypto;
import org.azuki.framework.annotation.*;
import org.jdom.Element;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class CryptoBean
{
	private Crypto crypto = null ;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception 
	{
		Element crypLmen = para.getChild(ConstCrypto.CRYPTO) ;
		if(crypLmen==null)
			throw new Exception("Missing configuration informations") ;
		
		String pphras = crypLmen.getAttributeValue(ConstCrypto.CRYPTO_PASSPHRASE) ;
		pphras = Crypto.decypher(pphras) ;
		
		this.crypto = new Crypto(pphras) ;
	}
	
	@Export
	@MethodInfo(desc = "Decypher the 'encoded' text to a readable text using a bean static internal key",
				retu = "A readable text")	
	final public String decypher(@ArgInfo(name="encoded", desc="The encoded text") String encoded)
	throws Exception
	{
		return crypto.decrypt(encoded) ;
	}
	
	@Export
	@MethodInfo(desc = "Cypher the 'readable' text to an encoded text using a bean static internal key",
			retu = "An encoded text")	
	final public String cypher(@ArgInfo(name="readable", desc="The readable text") String readable)
	throws Exception
	{
		return crypto.encrypt(readable) ;
	}
}