/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.cxf.server.console ;

import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.cxf.CXFWSBean;
import org.azuki.bean.cxf.CXFWSBean.Service;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.reflect.*;
import org.azuki.common.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
public class CXFWSConsole
{
	final static public String[] tableHeaders = {"Business service", "Bean", "Method", "Description"} ;
	
	final public static void p(CXFWSBean bean, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		//TODO: recuperer l'url de la generation du WDSL ?
		
		// services with errors
		List<List> errs = new ArrayList<List>() ;
		for(Service serv : bean.getServicesInErrors())
		{
			Method meth = serv.meth ;
			
			String    tnam = Util.oneString(serv.name, serv.labl) ;
			Signature stur = new Signature(meth) ;
			String    desc = Annotations.getMethodDescription(meth) ;
				 
			errs.add(ListUtil.toList(serv.serv, tnam, stur, desc)) ;
		}
		
		// services
		List<List> data = new ArrayList<List>() ;
		
		for(Service serv : bean.getServices())
		{
			Method meth = serv.meth ;
			
			String    tnam = Util.oneString(serv.name, serv.labl) ;
			Signature stur = new Signature(meth) ;
			String    desc = Annotations.getMethodDescription(meth) ;
				 
			data.add(ListUtil.toList(serv.serv, tnam, stur, desc)) ;
		}
		
		Table table = HtmlFactory.sortedTable(req, data, tableHeaders) ;
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		if(!errs.isEmpty())
		{
			bdr.print("<h2>Errors</h2>") ;
			bdr.println(errs) ;
			bdr.println("<br/>") ;	
		}
		
		bdr.print("<h2>Registered services</h2>") ;
		bdr.println(table) ;
		
		HttpResponse.print(res, bdr) ;
	}
}