package org.azuki.bean.documentation ;

import javax.servlet.http.*;

import org.azuki.bean.documentation.server.imp.*;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;

public class DocumentationBean
{
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	@DependencyInjectionInfo(desc="The method used to get the binding URIs of the specified bean and method",
							 param={Integer.class, String.class})
	public IMethodProxy<String[]> getBindingURI ;

	// URI of console (only end)
	private String myURI = null ;

	final public String getURI()
	{
		// myURI must be found here, otherwise is too soon.
		if(myURI==null)
		{
			try
			{
				if(getBindingURI==null)
					throw new Exception("Method getBindingURI has not been imported") ;

				myURI = StringUtil.slashStart(getBindingURI.call(getMetadata().getID(), "service")[0]) ;

				Log.finest("My URI : (" + myURI + ")") ;
			}
			catch(Exception e)
			{
				Log.severe(e.getMessage()) ;
				myURI = "" ;
			}
		}

		if("".equals(myURI))
			throw new NullPointerException("This aspect is not bound to any URI") ;

		return myURI ;
	}
	
	final public void service(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		try
		{
			// .../documentation/something/... --> /something/...
			String ruri = DocumentationUtil.relativeURI(this, req) ;
			
			if(ruri.indexOf(DocumentationUtil.rsrcCtx)!=-1)	PService.doResource(this, srv, req, res) ;
			else					 		  				PService.doMain(this, srv, req, res) ;
		}
		catch(Exception ex)
		{
			ex.printStackTrace() ;
			throw ex ;
		}
	}
}