/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event ;

import java.util.*;

import org.azuki.bean.event.shared.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.Element;

final public class EventSyncBean
{
	private enum AddResult { KEEP, OK, ERR, TIMEOUT}
	
	// Output notifying the synchronization, of all the process, with no error
	final static private String outputOK = "ok" ;
	
	// Output notifying the synchronization, of all the process, with at least one process entries has error
	final static private String outputErr = "err" ;

	// Output notifying that timeout occurs before the synchronization of all process
	final static private String outputTimeOut = "timeout" ;
	
	// EventPublisherBean injection
	@DependencyInjectionInfo(desc="The method used to publish a message to an output connector.",
							param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;	
	
	private int entries = -1 ; // Lazy
	
	final private Map<Integer, Map<String, Boolean>> syncs = new HashMap<Integer, Map<String, Boolean>>();
	
	final public void entryOK(String topic, Object o)
	throws Exception
	{
		if(entries==-1) entries = mkTopicList(this) ;
		
		int pid = ThreadContext.get().getProcessID() ;
		AddResult res = add(pid, topic) ;
		
		if(res==AddResult.ERR)     publish.call(this, outputErr,     Boolean.TRUE);
		if(res==AddResult.TIMEOUT) publish.call(this, outputTimeOut, Boolean.TRUE);		
		if(res==AddResult.OK)      publish.call(this, outputOK,      Boolean.TRUE);
	}

	final public void entryERR(String topic, Object o)
	throws Exception
	{
		if(entries==-1) entries = mkTopicList(this) ;
		
		int pid = ThreadContext.get().getProcessID() ;
		AddResult res = add(pid, topic) ;
		
		if(res==AddResult.ERR)     publish.call(this, outputErr,     Boolean.TRUE);
		if(res==AddResult.TIMEOUT) publish.call(this, outputTimeOut, Boolean.TRUE);		
		if(res==AddResult.OK)      publish.call(this, outputOK,      Boolean.TRUE);
	}
	
	final public AddResult add(int pid, String topic)
	{
		System.out.println("ADD "+pid+" "+topic) ;

		Integer PID = new Integer(pid) ;
		Map<String, Boolean> enty = syncs.get(PID) ;
		if(enty==null)
		{	enty = new HashMap<String, Boolean>() ;
			syncs.put(PID, enty); }
		
		enty.put(topic, Boolean.TRUE) ; // TODO : false means error
		
		Log.info("PID : "+PID+" TOPIC : "+topic) ;
		
		// Check if complete
		// (TODO : pour l'instant simple count, ameliorer avec la teste des cles)
		int nb = enty.keySet().size();
		
		if(entries==nb) return AddResult.OK ;

		return AddResult.KEEP ;
	}
	
	final public void checkTimeOut()
	{
		// TODO : il faut verifier le time out 
	}
	
	@SuppressWarnings("unchecked")
	final static private int mkTopicList(Object bean)
	{
		List<String> ls = new ArrayList<String>() ;

		try
		{	
			IMetadata meta = Bean.getMetadata(bean) ;
			
			List<IPointcut>lpc =  meta.getPointcuts(ConstEvent.EVENT_HANDLER) ;
		
			for(IPointcut pc : lpc)
			{
				Element para = pc.getParameter();
				if(para==null) continue ;
				
				// Topic
				List<Element> list = para.getChildren(TagEvent.TOPC) ;
				for(Element lm : list)
				{
					String topic = lm.getAttributeValue(AttrEvent.NAME) ;
					ls.add(topic) ;
				}
				
				// Link
				list = para.getChildren(TagEvent.LINK) ;
				for(Element lm : list)
				{
					String name = lm.getAttributeValue(AttrEvent.NAME) ;
					String labl = lm.getAttributeValue(AttrEvent.LABL) ;
					String outp = lm.getAttributeValue(AttrEvent.OUTP) ;
					
					Integer bid = Bean.searchFirst(name, labl).getID(); 
					
					String topic = new StringBuilder("link#").append(bid).append("#").append(outp).toString() ;				
					ls.add(topic) ;
				}
			}
		}
		catch(Exception e)
		{
			ls = new ArrayList<String>(0) ; // To prevent other init
			Log.severe(e.getMessage()) ;
		}
		
		return ls.size() ;
	}
}