/**
*   Created the 2 April 2006
*
*   Azuki Framework
*   Copyright 2006 the original author or authors.
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*   
*   Contact : support@azuki-framework.org
*
*   @author Yoann Perrin (yoann.perrin@gmail.com)
*/
package org.azuki.bean.event.editor.imp ;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.Arrays;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;

import org.azuki.common.consts.Attr;
import org.azuki.common.editor.IEditorContext;
import org.azuki.common.editor.jdom.JDOMAttributeListModel;
import org.azuki.common.icon.CommonIcons;
import org.jdom.Element;

@SuppressWarnings("serial")
public class OutputSelectionPane extends JPanel implements ActionListener
{
	final protected IEditorContext ectx ;
	
	protected Element lmen ;
	
	protected JLabel outpLabel ;
	
	protected JTextField outpSelectionField ;
	protected JLabel outpSelectionLabel ;
	protected JPanel outpSelectionMessage ;
	
	protected JDOMAttributeListModel outpListModel ;
	protected JList outpList ;
	protected JScrollPane outpListScroll ;
	
	protected JButton addButton ;
	protected JButton delButton ;
	protected JToolBar actionBar ;
	
	public OutputSelectionPane(IEditorContext ectx, Element lmen)
	{
		this.ectx = ectx ;
		this.lmen = lmen ;
		
		// output selection
		outpSelectionField = new JTextField(20) ;
		
		outpSelectionLabel = new JLabel("Enter output name: ") ;
		outpSelectionLabel.setLabelFor(outpSelectionField) ;
		
		outpSelectionMessage = new JPanel() ;
		outpSelectionMessage.add(outpSelectionLabel) ;
		outpSelectionMessage.add(outpSelectionField) ;
		
		// outputs list
		outpLabel = new JLabel("Outputs:") ;
		outpLabel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2)) ;
		
		outpListModel = new JDOMAttributeListModel(lmen, "output", Attr.NAME) ;
		outpList = new JList(outpListModel) ;
		
		outpListScroll = new JScrollPane(outpList) ;
		
		// actions
		addButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16)));
		addButton.addActionListener(this) ;
		
		delButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16)));
		delButton.addActionListener(this) ;
		
		actionBar = new JToolBar(SwingConstants.VERTICAL) ;
		actionBar.setFloatable(false) ;
		actionBar.add(addButton) ;
		actionBar.add(delButton) ;
		
		setLayout(new BorderLayout()) ;
		add(outpLabel, BorderLayout.NORTH) ;
		add(outpListScroll, BorderLayout.CENTER) ;
		add(actionBar, BorderLayout.EAST) ;
	}
	
	final public void setElement(Element lmen)
	{
		this.lmen = lmen ;
		outpListModel.setElement(lmen) ;
	}
	
	final public Element getSelectedElement()
	{
		return outpListModel.getItem(outpList.getSelectedIndex()) ;
	}
	
	final public boolean isSelectionEmpty()
	{
		return outpList.isSelectionEmpty() ;
	}
	
	final public void setSelectionMode(int selectionMode)
	{
		outpList.setSelectionMode(selectionMode) ;
	}
	
	final public void addListSelectionListener(ListSelectionListener l)
	{
		outpList.addListSelectionListener(l) ;
	}
	
	final public void removeListSelectionListener(ListSelectionListener l)
	{
		outpList.removeListSelectionListener(l) ;
	}
	
	/*
	 * ActionListener
	 */
	final public void actionPerformed(ActionEvent e)
	{
		Object src = e.getSource() ;
		
		if(src==addButton)
		{
			String outputName = JOptionPane.showInputDialog(this, "New output name: ", "Add output", JOptionPane.PLAIN_MESSAGE) ;
			if(outputName!=null) outpListModel.addItem(outputName) ;
		}
		else if(src==delButton)
		{
			int[] indexes = outpList.getSelectedIndices() ;
			
			Arrays.sort(indexes) ;
			
			for(int i=indexes.length-1 ; i>=0 ; i--)
				outpListModel.removeItem(indexes[i]) ;
			
			outpList.clearSelection() ;
		}
	}
}