/**
 *   Created the 5 déc. 07
 *
 *   Azuki Framework
 *   Copyright 2007 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu JUCKER (mathieu.jucker@azuki-framework.org)
 */
package org.azuki.bean.exec;

import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.*;

public class XMLCommandFactoryBean
{
	private Document xmlCommand;
	
	final static private String outputName = "xmlCommand";
	
	// EventPublisher injection
	@DependencyInjectionInfo(desc="Publish a message to an output connector",
							 param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;

	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
    final public void postCreate(Element para)
	throws Exception
	{
		Element p = new Element("para");
		p.addContent(JDOMUtil.getChildren(para, true));
		xmlCommand = new Document();
		xmlCommand.setRootElement(p);
	}
	
	@MethodInfo(desc="This method can receive events to create an XML command document  in the bean output")
	final public void processEvent()
	throws Exception
	{
		if(xmlCommand==null) return;
		
		publish.call(this, outputName, xmlCommand) ;
		Log.info("XMLCommandFactoryBean:processEvent::publish the XmlCommand document  to output ("+outputName+")");	
	}
	
	@Export
	@MethodInfo(desc="This method create an xml command document  with a specific command line. Note that command bean parameter is ignored in this case.")
	final public void processCommand(String command)
	throws Exception
	{
		if(xmlCommand==null) return;
		
		xmlCommand.getRootElement().getChild("exec").getAttribute("command").setValue(command);
		publish.call(this, outputName, xmlCommand) ;
		Log.info("XMLCommandFactoryBean:processCommand:publish the XmlCommand document  to output ("+outputName+")");	
	}

}
