/**
 *   Created the 5 déc. 07
 *
 *   Azuki Framework
 *   Copyright 2007 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu JUCKER (mathieu.jucker@azuki-framework.org)
 */
package org.azuki.bean.exec;

import java.util.*;

import org.azuki.bean.exec.server.imp.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.bean.Bean;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.*;

/**
 * <pre>
 * 	XMLToProcess transform a predefined XML document into an ICommand Object.
 * 	The ICommand is poste on it output.
 * 	Ouput should be connected to a CommandExecutionBean input in order to be processed
 * 
 * 	The bean input must receive an XML Document :
 * 	<?xml version="1.0" encoding="UTF-8"?>
 *		<!DOCTYPE para SYSTEM "xmlToProcess.dtd">
 *		<para>
 *   		<exec command="">
 *      		<row name="" valu=""></row>
 *     		<row name="" valu=""></row>
 *   		</exec>
 * 	</para>
 * 	
 * 	The corresponding DTD
 * 	<?xml version="1.0" encoding="iso-8859-1" ?>
*		<!ELEMENT exec (#PCDATA | row)*>
*		<!ATTLIST exec command CDATA #REQUIRED>
*		<!ELEMENT row (#PCDATA) >
*		<!ATTLIST row name CDATA #REQUIRED>
*		<!ATTLIST row valu CDATA #REQUIRED>
*		<!ELEMENT para (#PCDATA | exec)*>
 * 
 *  </pre>   
 */
final public class XMLToProcessBean
{
	final static private String outputName = "command";
	
	// EventPublisher injection
	@DependencyInjectionInfo(desc="Publish an ICommand Object  to an output connector",
							param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;
	
	//	 EventPublisher injection
	@DependencyInjectionInfo(desc="Gets system env() value and user-defined variable for the external process execution")
	public IMethodProxy<Map<String, String>> getAllInherited ;
		
	final public Map<String, String> getAllInherited()
	{
		if(getAllInherited==null) 
			return new HashMap<String, String>() ;
		
		try
		{
			return getAllInherited.call() ;
		}
		catch(Exception e)
		{
			return new HashMap<String, String>() ;
		}
	}
	
	@MethodInfo(desc="This method can only receive and XML Document that contains a command to execute. it create Jobs in the bean output")
	final public void processEventXml(String topic, Document msg)
	throws Exception
	{
			Element para = msg.getRootElement();
			
			String key = Bean.getMetadata(this).getID().toString();
			PProcess.parseXmlToCommand(para, getAllInherited(), key);

			Execution execution = PProcess.parseXmlToCommand(para, getAllInherited(), key);
			Log.info("Execution Object created : "+ execution);
				
			if(execution==null) return;
			
			publish.call(this, outputName, execution) ;
			Log.info("publish the execution to output ("+outputName+")");	
	}
}
