/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.exec.server.imp ;

import java.io.*;

import org.azuki.framework.command.ICommand;
import org.azuki.framework.context.Log;

final public class GPReader implements ICommand
{
	final static public String OUT  = "OUT" ; // Log stdout of process 
	final static public String ERR  = "ERR" ; // Log stderr of process
	final static public String NONE = "NONE" ; // No Log
	
	private String    type ;
	private InputStream in ;
    private OutputStream out ;
    private GTSync sync ;

	public GPReader(String type, InputStream in, OutputStream out, GTSync sync)
	{
		this.type = type ;
		this.in = in ;
        this.out = out ; // Often null
        this.sync = sync ;
	}

	// IJob
	final public String getName() { return "STREAM READER"; }

	final public Object process()
	{
		BufferedReader br = new BufferedReader(new InputStreamReader(in)) ;
		
		try
		{
			while(true)
			{
				String line = br.readLine() ;
				if(line==null) return null;

				if(type!=NONE) Log.fine(line) ;

                if(out!=null)
                {
                   out.write(line.getBytes());
                   out.write('\n');
                   out.flush();
                }
            }
		}
		catch(Exception e)
		{
			Log.severe(e.getMessage()) ;
		}
        finally
        {
			if(br!=null) try{ br.close() ; } catch(Exception e){}
        	
			if(type==OUT)      sync.setOut();
            else if(type==ERR) sync.setErr();
        	else // NONE
        	{
        		sync.setErr() ;
        		sync.setOut() ;
        	}
        }
        
        return null ;
	}
}