/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.exec.server.imp.mngr ;

import java.io.OutputStream;
import java.util.*;

import org.azuki.bean.exec.ProcessFactoryBean;
import org.azuki.bean.exec.server.imp.Execution;
import org.azuki.framework.command.*;
import org.azuki.framework.command.ICommand.Priority;

public class ProcessManager
{
	final private PRunning prun   = new PRunning() ; 
	final private PError   perror  ; // Error container
	
	public ProcessManager(ProcessFactoryBean es)
	{
		perror = new PError(es) ;
	}
	
	//
	@SuppressWarnings("unchecked")
	final public void invoke(IQueue queue, Priority pty, String name, String[] cmd, Map<String, String> env, Map<String, String> nenv, String path, OutputStream out)
	throws Exception
	{
		Command.invokeQueued(new Execution(this, name, cmd, env, nenv, path, out), queue, pty) ;
	}
	
	@SuppressWarnings("unchecked")
	final public void send(IQueue queue, Priority pty, String name, String[] cmd, Map<String, String> env, Map<String, String> nenv, String path, ICallback cback)
	{
		Command.sendQueuedCallback(new Execution(this, name, cmd, env, nenv, path, null), queue, pty, cback) ;
	}

	//
	

    final public void addRunning(Integer ID, String name, String[] cmd, Process proc, long star)
    {
        prun.add(ID, name, cmd, proc, star) ;
    }
    
    final public void removeRunning(Integer ID)
    {
        prun.remove(ID) ;
    }
    
    final public void addError(Integer ID, String name, String[] cmd,
            String errmess, long start, long end, Map<String, String> env)
    {
        perror.add(ID, name, cmd, errmess, start, end, env) ;
    }

    final  public List getProcessWithError()
    {
    	return perror.list() ;
    }
    
	final public void restartNoWait(Integer ID)
	{
		perror.restartNoWait(ID) ;
	}
	
	final public void restartWait(Integer ID, OutputStream out)
	throws Exception 
	{
		perror.restartWait(ID, out) ;
	}

	final public void kill(Integer ID)
	{
		prun.kill(ID) ;
	}

	final public List getInfo()
	{
		return prun.getInfo() ;
	}	
}