/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.global ;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.global.server.*;
import org.azuki.bean.global.server.console.GlobalConsole;
import org.azuki.bean.global.server.imp.*;
import org.azuki.bean.global.shared.ConstGlobal;
import org.azuki.common.editor.*;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.context.*;
import org.jdom.Element;

/**
 * This convenient class stores a set of 'global' values inherited from the system env() parameter.
 * Global values can be added but will only be available for the current process execution.
 * 
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class GlobalBean extends GlobalFactory implements IAspect<GlobalAdvice>
{
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		GlobalMap envp = getenv() ;
		
		// get and add new variables declared in the bean Global.
		List<Map<String,String>> param = EditorUtil.tableToList(para, TagCommon.ROW);
		
		for(Map<String,String> datas : param)
		{
			String  name = datas.get(AttrCommon.NAME) ;
			String  valu = datas.get(AttrCommon.VALU) ;
			boolean exec = Boolean.parseBoolean(datas.get(ConstGlobal.EXEC)) ;
			
			envp.putVar(name, valu, exec) ;
		}
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public GlobalAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		List<Map<String,String>> param = EditorUtil.tableToList(ipc.getParameter(), TagCommon.ROW) ;
		
		GlobalMap envspec = new GlobalMap() ;
		
		for (Map<String,String> datas : param)
		{
			String  name = datas.get(AttrCommon.NAME) ;
			String  valu = datas.get(AttrCommon.VALU) ;
			boolean exec = Boolean.parseBoolean(datas.get(ConstGlobal.EXEC)) ;
			
			envspec.putVar(name, valu, exec) ;
		}
		
		return new GlobalAdvice(envspec) ;
	}

	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		GlobalConsole.service(servlet, req, res) ;
	}
	
////////////////////////////////////////////////////////////////////////////
	
	@Export
	final static public Map<String,String> getAll()
	throws Exception
	{
		GlobalMap env = (GlobalMap)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ;
		
		Map<String,String> map = new HashMap<String,String>(env.size()) ;
		
		for(Map.Entry<String,String> ety : env.entrySet())
			map.put(ety.getKey(), ety.getValue()) ;
		
		return map ; 
	}

	@Export
	final static public Map<String,String> getAllInherited()
	throws Exception
	{
		GlobalMap env = (GlobalMap)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ;
		
		Map<String,String> map = new HashMap<String,String>() ;
		
		for(String key : env.keySet())
		{
			GlobalVar var = env.getVar(key) ;
			if(var.isInherit())
				map.put(key, var.getValue()) ;
		}
		
		return map ;
	}

	// Return null if not found
	@Export
	final static public String get(String key)
	throws Exception
	{
		GlobalMap env = (GlobalMap)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ;
		GlobalVar var = env.getVar(key) ;
		return var==null ? null : var.getValue() ;
	}

	@Export
	final static public boolean isInherited(String key)
	throws Exception
	{
		GlobalMap env = (GlobalMap)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ;
		GlobalVar var = env.getVar(key) ;
		return var==null ? false : var.isInherit() ;
	}

	@Export
	final static public void put(String key, String value, boolean inherit)
	throws Exception
	{
		GlobalMap env = (GlobalMap)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ;		
		env.putVar(key, value, inherit) ;
	}
}