/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.global.server.imp ;

import java.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class GlobalMap implements Map<String,String>, Cloneable
{
	final private Map<String,GlobalVar> vars ;
	
	public GlobalMap()
	{
		this.vars = new HashMap<String,GlobalVar>() ;
	}
	public GlobalMap(GlobalMap map)
	{
		this() ;
		putAllVar(map) ;
	}

	final public GlobalVar getVar(String key)
	{
		return vars.get(key) ;
	}
	
	final public GlobalVar putVar(String key, String value, boolean inherit)
	{
		GlobalVar var = new GlobalVar(value, inherit) ;
		GlobalVar old = vars.put(key, var) ;
		return old ;
	}
	
	final public void putAllVar(GlobalMap map)
	{
		vars.putAll(map.vars) ;
	}
	
	final public Set<Map.Entry<String,GlobalVar>> varSet()
	{
		return vars.entrySet() ;
	}
	
	@Override
	final public GlobalMap clone()
	{
		GlobalMap clon = new GlobalMap() ;
		clon.putAllVar(this) ;
		return clon ;
	}
	
	//
	// Map
	//
	
	final public int size()
	{
		return vars.size() ;
	}
	
	final public boolean isEmpty()
	{
		return vars.isEmpty() ;
	}
	
	final public boolean containsKey(Object key)
	{
		return vars.containsKey(key) ;
	}

	final public boolean containsValue(Object value)
	{
		for(Map.Entry<String,GlobalVar> ety : vars.entrySet())
		{
			String gval = ety.getValue().getValue() ;
			if(gval==value) return true ;
			if(gval!=null && gval.equals(value)) return true ;
		}
		
		return false ;
	}

	final public Set<String> keySet()
	{
		return vars.keySet() ;
	}
	
	final public Set<Map.Entry<String,String>> entrySet()
	{
		HashSet<Map.Entry<String,String>> set = new HashSet<Map.Entry<String,String>>() ;
		
		for(Map.Entry<String,GlobalVar> ety : vars.entrySet())
		{
			String key = ety.getKey() ;
			String val = ety.getValue().getValue() ;
			
			Entry<String,String> e = new Entry<String,String>(key, val) ;
			
			set.add(e) ;
		}
		
		return set ;
	}
	
	final public Collection<String> values()
	{
		List<String> vals = new ArrayList<String>() ;
		
		for(Map.Entry<String,GlobalVar> ety : vars.entrySet())
		{
			String val = ety.getValue().getValue() ;
			vals.add(val) ;
		}

		return vals ;
	}

	final public String get(Object key)
	{
		GlobalVar var = vars.get(key) ;
		return var==null ? null : var.getValue() ;
	}

	final public String put(String key, String value)
	{
		GlobalVar var = new GlobalVar(value, true) ;
		GlobalVar old = vars.put(key, var) ;
		return old==null ? null :old.getValue() ;
	}

	final public void putAll(Map<? extends String,? extends String> map)
	{
		if(map==null) return ;
		
		for(Map.Entry<? extends String,? extends String> ety : map.entrySet())
		{
			GlobalVar var = new GlobalVar(ety.getValue(), true) ;
			vars.put(ety.getKey(), var) ;
		}
	}

	final public String remove(Object key)
	{
		GlobalVar var = vars.remove(key) ;
		return var==null ? null : var.getValue() ;
	}

	final public void clear()
	{
		vars.clear() ;
	}
	
	///////////////////////////////////////////////////////////////////////////
	
	final static private class Entry<K, V> implements Map.Entry<K,V>
	{
		final private K key ;
		
		private V val ;

		Entry(K key, V val)
		{
			this.key = key ;
			this.val = val ;
		}

		final public K getKey()
		{
			return key ;
		}
		
		final public V getValue()
		{
			return val ;
		}

		final public V setValue(V val)
		{
			V old = val ;
			
			this.val = val ;
			
			return old ;
		}
		
		@Override
		final public boolean equals(Object o)
		{
			if(!(o instanceof Map.Entry)) return false ;
			
			Map.Entry e = (Map.Entry)o ;
			
			Object k1 = getKey() ;
			Object k2 = e.getKey() ;
			if(k1==k2 || (k1!=null && k1.equals(k2)))
			{
				Object v1 = getValue() ;
				Object v2 = e.getValue() ;
				if(v1==v2 || (v1!=null && v1.equals(v2))) return true ;
			}
			
			return false ;
		}

		@Override
		final public int hashCode()
		{
			return (key==null ? 0 : key.hashCode()) ^ (val==null ? 0 : val.hashCode()) ;
		}

		@Override
		final public String toString()
		{
			return getKey() + "=" + getValue() ;
		}
	}
}