/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.initializer ;

import java.lang.reflect.Method;

import org.azuki.bean.initializer.server.InitializerAdvice;
import org.azuki.common.consts.Tag;
import org.azuki.common.util.Util;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class InitializerBean implements IAspect<InitializerAdvice>
{
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public InitializerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		String st = Util.oneString(tmeta.getName(), tmeta.getLabel()) ;
		
		Object bean = tmeta.getBean() ;
		
		if(bean==null)
			throw new Exception(new StringBuilder("Bean (").append(st).append(") has not been created.").toString()) ;
	
		getMetadata().getContract().checkBindable(bean.getClass()) ;
		
		Element para = ipc.getParameter() ;
		if(para!=null) para = para.getChild(Tag.PARA) ;
		
		Method[] ms = ipc.getSelectedMethods() ;
		if(ms==null || ms.length==0)
			throw new Exception("no methods selected") ;
		
		Method m = ms[0] ;
		
		Object[] args = null ;
		if(m.getParameterTypes().length==1)
			args = new Object[]{para} ;
		
		StringBuilder sb = new StringBuilder("(").append(st).append(")");
		
		/* No more
		if(99!=tmeta.getRunLevel())
				sb.append("@").append(tmeta.getRunLevel()) ;
		*/
		
		Log.info(sb.toString()) ;
		
		m.invoke(bean, args) ;
				
		return new InitializerAdvice() ;
	}	
}