/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.initializer.editor;

import java.awt.Component;
import java.net.URL;
import java.util.*;

import javax.swing.JOptionPane;

import org.azuki.bean.initializer.shared.ConstInitializer;
import org.azuki.common.editor.*;
import org.azuki.common.reflect.BeanInfo;
import org.azuki.common.util.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

/**
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */
final public class InitializerEditor implements IPointcutEditor
{
	private Element para ;
	
	private Collection<Element> flds = Collections.emptyList() ;
	private Collection<Element> mths = Collections.emptyList() ;
	
	/*
	 * IPointcutEditor
	 */
	public Element getParameters()
	{
		return para ;
	}
	
	final public Collection<Element> getFields()
	{
		return flds ;
	}

	final public Collection<Element> getMethods()
	{
		return mths ;
	}

	@SuppressWarnings("unchecked")
	public boolean edit(Component parent, IPointcutEditorContext context)
	{
		para = context.getParameters() ;
		
		flds = context.getSelectedFields() ;
		mths = context.getSelectedMethods() ;
		
		// retrieve editor class from 'beans.xml' 
		Element editLmen = context.getDefaultParameters().getChild(ConstInitializer.EDIT) ;
		if(editLmen==null)
		{	JOptionPane.showMessageDialog(parent, "Missing editor configuration", "Initializer editor error", JOptionPane.ERROR_MESSAGE) ;
			return false ; }
		
		String editName = editLmen.getAttributeValue(ConstInitializer.EDIT_NAME) ;
		if(StringUtil.isEmpty(editName))
		{	JOptionPane.showMessageDialog(parent, "Editor class is not set", "Initializer editor error", JOptionPane.ERROR_MESSAGE) ;
			return false ; }
		
		// update editor class attribute
		editLmen = JDOMUtil.getElement(para, ConstInitializer.EDIT) ;
		editLmen.setAttribute(ConstInitializer.EDIT_NAME, editName) ;
		
		// search editor class
		Class<?> ctmp = null ;
		try
		{ ctmp = Class.forName(editName) ; }
		catch(Exception ex)
		{ JOptionPane.showMessageDialog(parent, "Editor ("+editName+") not found", "Initializer editor error", JOptionPane.ERROR_MESSAGE) ; }
		
		if(ctmp==null) return false ;
		
		Class<? extends IEditor> editClas = null ;
		if(IEditor.class.isAssignableFrom(ctmp))
			editClas = ctmp.asSubclass(IEditor.class) ;
		else
			JOptionPane.showMessageDialog(parent, "Editor ("+editName+") does not implement "+IEditor.class.getName(), "Initializer editor error", JOptionPane.ERROR_MESSAGE) ;
			
		if(editClas==null) return false ;
		
		// build editor parameters
		Element editPara = (Element)JDOMUtil.getElement(para, ConstInitializer.PARA).clone() ;
		Element editDefp = (Element)JDOMUtil.getElement(context.getDefaultParameters(), ConstInitializer.PARA).clone() ;
		
		// starting editor
		boolean validated = false ;
		
		try
		{
			IEditorContext iedtCtxt = new SyntheticContext(context, editPara, editDefp) ;
			
			IEditor iedt = editClas.newInstance() ;
			validated = iedt.edit(parent, iedtCtxt) ;
			
			if(validated)
			{	para.removeChild(ConstInitializer.PARA) ;
				para.addContent(iedt.getParameters()) ; }
		}
		catch(Exception ex)
		{
			JOptionPane.showMessageDialog(parent, Log.getStack(ex), "Initializer editor error", JOptionPane.ERROR_MESSAGE) ;
		}
				
		return validated ;
	}
	
	final private class SyntheticContext implements IEditorContext
	{
		final private IPointcutEditorContext delegate ;
		
		final private Element syntPara ;
		final private Element syntDefp ;
		
		public SyntheticContext(IPointcutEditorContext delegate, Element para, Element defp)
		{
			this.delegate = delegate ;
			
			this.syntPara = para ;
			this.syntDefp = defp ;
		}
		
		/*
		 * IEditorContext
		 */
		final public BeanInfo getInfo()
		{
			return delegate.getInfo() ;
		}
		
		final public String getLabel()
		{
			return delegate.getLabel() ;
		}
		
		final public Element getParameters()
		{
			return syntPara;
		}
		
		final public Element getDefaultParameters()
		{
			return syntDefp ;
		}

		final public URL getResource(String name)
		{
			return delegate.getResource(name) ;
		}

		final public byte[] getResourceBytes(String name)
		{
			return delegate.getResourceBytes(name) ;
		}
		
		final public Collection<BeanInfo> getBeanInfos()
		{
			return delegate.getBeanInfos() ;
		}
		
		final public BeanInfo getBeanInfo(String name)
		{
			return delegate.getBeanInfo(name) ;
		}
		
		final public Collection<String> getDeclaredInstances(String beanName)
		{
			return delegate.getDeclaredInstances(beanName) ;
		}

		final public Collection<String> getDeclaredTopics()
		{
			return delegate.getDeclaredTopics() ;
		}
	}
}