/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.injection.server.imp ;

import org.azuki.common.reflect.ReflectUtil;
import org.azuki.common.util.Util;
import org.azuki.framework.bean.*;
import org.azuki.framework.command.*;
import org.azuki.framework.command.ICommand.Priority;
import org.azuki.framework.injection.IMethodProxy;

public class MethodProxyImp<T> implements IMethodProxy<T>
{
	// Target
	final private String name ;
	final private String labl ;
	final private String meth ;
	final private Class[]  cs  = null ; // Not used = Method parameter must match the argument type. 
	
	final private boolean asyn; // true : send
	final private boolean fork; // true : invoke

	private IMetadata meta ; // Not resolve, until first call for initialisation raison
	
	public MethodProxyImp(String name, String labl, String meth, /*Class[] cs,*/ boolean asyn, boolean fork)
	{
		this.name = name ;
		this.labl = labl ;
		this.meth = meth ;
		// this.cs   = cs ; // null
		this.asyn = asyn ;
		this.fork = fork ;
	}
	
	//---------------------------------------------
	final private IMetadata getMetadata()
	throws Exception
	{
		if(meta==null)
		{
			meta = Bean.searchFirst(name, labl) ;
			
			if(meta==null)
				throw new Exception("Bean not found ("+Util.oneString(name, labl)+")") ;
		}
			
		return meta ;		
	}
	
	final public Integer getBeanID()
	throws Exception
	{
		return getMetadata().getID() ;
	}
	
	final public boolean exists()
	{
		IMetadata meta = null ;
		try{ meta = getMetadata() ; }
		catch(Exception ex) {}
		
		if(meta==null) return false ;
		
		Class clas = meta.getBeanClass() ;
		return ReflectUtil.bestMatch(clas, meth, null)!=null ;
	}
	
	final public String    getMethod() {	return meth ; }		

	final public Class[]   getParameterType() {	return cs ; }	
	
	final public boolean isAsynchronous() { return asyn ; } 

	final public boolean isForked() { return (asyn || fork) ; } 
	
	//---------------------------------------------

	// Rem : Cette methode utilise les parametres du constructor
	// pour savoir si l'appel est sync/async & si prreservation du 
	// context ou non
	final public T service(Object... args)
	throws Exception
	{
		if(asyn)
		{	send(args) ;
			return null ; }
		
		if(fork) invoke(args) ;
		
		// default	
		return call(args) ;
	}	
	
	
	@SuppressWarnings("unchecked")
	final public T call(Object... args)
	throws Exception
	{
		IMetadata meta = getMetadata() ;

		/* BOB : Process login will be added in next version 
 		IContext cx = ThreadContext.get() ;
		
		IMILogger mil = null ;
		
		try
		{
			mil = (IMILogger)cx.lookup(IMILogger.RESOURCE_NAME) ; 		
		}
		catch(Exception e) { Log.severe(e.getMessage());}
			
		if(mil==null) // Not found
		*/
			return (T)Command.call(new MethodInvocation(meta, meth, cs, args));			
/*
			
		int pid  = cx.getProcessID() ;
		int tcid = cx.getID() ;
		
		try
		{
			mil.start(pid, tcid, meta.getID(), meth) ;
			T t = (T)Command.call(new MethodInvocation(meta, meth, cs, args));			
			mil.end(pid, tcid, meta.getID(), meth) ;
			return t ;
		}
		catch(Exception e)
		{
			if(mil!=null) mil.exception(pid, tcid, meta.getID(), meth, e) ;
			throw e ;
		}		
		*/
	}	

	@SuppressWarnings("unchecked")
	final public T invoke(Object... args)
	throws Exception
	{
		IMetadata meta = getMetadata() ;
		return (T)Command.invokeQueued(new MethodInvocation(meta, meth, cs, args), meta.getQueue(), Priority.NORMAL) ;
	}	

	final public void send(Object... args)
	throws Exception
	{
		send((ICallback)null, args) ;
	}	

	final public void send(ICallback cback, Object... args)
	throws Exception
	{
		IMetadata meta = getMetadata() ;
		Command.sendQueuedCallback(new MethodInvocation(meta, meth, cs, args), meta.getQueue(), Priority.NORMAL, cback) ;
	}
	
	private String st = null ;
	@Override
	public String toString()
	{
		if(st!=null) return st ;
		st = new StringBuilder("[").append(Util.oneString(name, labl)).append("].").append(meth).toString() ;
		return st ;
	}
	
//	 Method invocation logger
	public interface IMILogger
	{
		final static public String RESOURCE_NAME = "procanalyzer" ;
		
		public void start    (int pid, int tcid, Integer bid, String meth) ;
		public void end      (int pid, int tcid, Integer bid, String meth) ;
		public void exception(int pid, int tcid, Integer bid, String meth, Exception e) ;
	}	
}