/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.javarpc ;

import javax.servlet.http.*;

import org.azuki.bean.javarpc.server.imp.*;
import org.azuki.client.javarpc.imp.Sender;
import org.azuki.common.bean.rpc.*;
import org.azuki.common.util.ProcessLifecycle;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class JavaRpcBean extends AbstractRpcBean
{
	@DependencyInjectionInfo(desc="This method can be used for the authentication",
							param={String.class, Object.class},
							mandatory=false)
	public IMethodProxy<Boolean> check;
	
	final public boolean isSecured()	{ return check!=null ; }
	
	@MethodInfo(desc="The contract implementation of the request dispatcher aspect (RequestDispatcher)")
	final public void service(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
    throws Exception
	{
		try
		{		
	    	String uri = req.getRequestURI() ;
		    
	    	// Connection
	    	if(uri.endsWith(Sender.AZUKIX_A)) // connect method
	    	{
	    		connect(req, res) ;
	    		return ;
	    	}
	
	    	// Authentication check
	    	try
			{			
				IContext cx = ThreadContext.get() ;
				if(isSecured())
				{
					HttpSession sess = req.getSession(false) ;
					if(sess==null) throw new Exception("No valid session.") ;
					
					Object user = sess.getAttribute(IContext.ATTR_USER) ;
					if(user==null) throw new Exception("No user defined.") ;
	
					cx.bind(IContext.ATTR_USER, user, new ProcessLifecycle()) ;
				}
	
				cx.bind(IContext.OBJECT_HTTPREQUEST, req, null) ; // If you need information from the original servlet request
	
			}
			catch(Throwable t) // Error are serialized to the client
			{
				Log.severe(t.getMessage());
				Response.write(req, res, t) ;
				return ;
			}		
	
			// Method invocation
			call(this, req, res) ;
		}
		catch(Exception ee)
		{
			Log.severe(ee) ;
			throw ee ; // to erreur 500
		}
	}
	
	final private void connect(HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		if(check==null)
			throw new Exception("Server error : missing authentication method configuration") ;

		Object ores ;

		try
		{
			// Read of java serialize parameters
			Object[] param = (Object[])Http.readObject(Http.getContent(req));

			String user = (String)param[0]; 
			Object args = param[1]; 
		
			ores = check.call(user, args) ;
		
			if(Boolean.TRUE.equals(ores))
			{
				HttpSession sess = req.getSession(true) ; // Session creation
				sess.setAttribute(IContext.ATTR_USER, user) ;
			}
				
		}
		catch(Throwable t) // Error are serialized to the client
		{
			String msg = t.getMessage() ;
			// Il faut créer une nouvelle instance de l'exception 
			// afin de s'assurer qu'elle soit serializable (connue du client)
			ores = new Exception(msg)  ;
		}
		
		Response.write(req, res, ores);
	}
	
	
	final static public void call(AbstractRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Object ores ;

		try
		{
			// Read of java serialize parameters
			Object[] param = (Object[])Http.readObject(Http.getContent(req));

			String service = (String)  param[0]; 
			Object[] args  = (Object[])param[1] ; 
			
	    	Log.info("Invoking : "+service) ;
	    	
	    	Service serv = bean.getService(service);
			if (serv == null)
				throw new Exception("Service not found: " + service);
	    	 	 
			ores = serv.call(args);    
		}
		catch(Throwable t) // Error are serialized to the client
		{
			Log.severe(t) ;
			// Il faut créer une nouvelle instance de l'exception 
			// afin de s'assurer qu'elle soit serializable (connue du client)
			String msg = t.getMessage() ;
			ores = new Exception(msg)  ;
		}
		
		Response.write(req, res, ores);    	
	}
}