/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.javarpc.server.imp ;

import java.io.*;

import javax.servlet.http.*;

import org.azuki.client.javarpc.imp.PZip;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class Http
{
	final static public byte[] getContent(HttpServletRequest req)
	throws IOException
	{
        int clength = req.getContentLength() ;
        
        byte[] content = null ;
        if(clength > 0)
        {
            // Reading cotent
            BufferedInputStream bis = new BufferedInputStream(req.getInputStream()) ;
            content = new byte[clength] ;
            
            int rd = 0 ;
            while(true)
            {
                int read = bis.read(content, rd, clength-rd) ;
                if(read==-1) break; 
                rd+=read ;
                if(rd>=clength) break;
            }
        }
        else clength = 0 ;          	
    	
        if("gzip".equals(req.getHeader("Content-Encoding")))
            content = PZip.unCompress(content) ;
	
        return content ;
	}
	
	final static public Object readObject(byte[] content)
	throws IOException
	{
		if(content==null) return null ;

		try
		{
			return PZip.deserialize(content) ;
		}
		catch(Exception e) {
			e.printStackTrace();
			throw new IOException(e.getMessage()); }
	}	
	
	final static public String getCookie(HttpServletRequest req, String name)
	{
    	Cookie[] cooks = req.getCookies() ;
    	if(cooks==null) return null ;
    	
    	for(int i=0; i<cooks.length; i++)
    	{
    		Cookie cook = cooks[i] ;
    		if(cook.getName().equals(name)) return cook.getValue() ;
		}
    	return null ;
	}

	final static public boolean allowGzip(HttpServletRequest req)
	{
		String ae = req.getHeader("accept-encoding"); // Case insensitive
		if(ae==null) return false;
		if(ae.indexOf("gzip") != -1) return true ;
		return false ;
	}
}