/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.jmx.server;

import java.lang.reflect.Method;

import javax.management.*;

import org.azuki.bean.jmx.server.imp.ObjectNameFactory;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

/**
 * TODO : use table to register interest more than 1 bean (?)
 * parameter : Form Parametre
 * <para>
 * 	<row name="name" desc="Bean Name" valu=""/>
 *  <row name="labl" desc="Bean Label" valu=""/>
 * </para>
 * @author mjucker
 *
 */
public class JMXListenerAdvice implements NotificationListener
{
	private String methName ;
	private boolean callback ;
	
	private IMethodProxy methProxy ;
	
	private ObjectName targetName ;
	
	public JMXListenerAdvice(String domain, Integer beanID, Method method, String tName, String tLabl, Integer targetID)
	throws Exception
	{
		methName = method.getName() ;
		callback = method.getParameterTypes().length==2 ; //TODO: maybe sharpest ?
				
		// create method proxy
		methProxy = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, tName, tLabl, method.getName()) ; 
	
		// define MBean name to listen to
		targetName = ObjectNameFactory.create(domain, tName, tLabl, targetID) ;
	}
	
	final public ObjectName getTargetObjectName()
	{
		return targetName ;
	}
	
	/*
	 * NotificationListener
	 */
	final public void handleNotification(Notification notification, Object handback)
	{
		// call bean method defined in <method> tag : PointCut properties
		try
		{
			if(callback) methProxy.call(notification, handback) ;
			else		 methProxy.call() ;	
		}
		catch(Exception e)
		{
			Log.severe("Cannot call method ("+methName+")") ;
		}
		finally
		{
			ThreadContext.reset() ; // Clean thread context
		}
	}
}
