/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.jndi ;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.jndi.server.JNDIResourceAdvice;
import org.azuki.bean.jndi.server.console.JNDIResourceConsole;
import org.azuki.bean.jndi.shared.ConstJNDI;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.jdom.Element;

final public class JNDIResourceBean implements IAspect<JNDIResourceAdvice>
{
	/*
	  <para>
	    <namespace name="java:/comp/env/jdbc" />
		<reference 
		   className="com.mysql.jdbc.jdbc2.optional.MysqlDataSource" 
		   factory="com.mysql.jdbc.jdbc2.optional.MysqlDataSourceFactory" 
		   factoryLocation="" >
		   
		   <address type="driverClassName" value="com.mysql.jdbc.Driver" />
		   <address type="url"      value="jdbc:mysql://localhost:3306/test" />
		   <address type="user"     value="root" />
		   <address type="password" value="hephaistos" />
		</reference>
		<context bind="datasource" />
	  </para>
	 */
	
	private IMetadata meta = null ;
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta!=null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
		
	private String bind = null ;
	final public String getContextBinding() 	{ return bind ; }
	
	private String nspc = null ;
	final public String getNamespace()			{ return nspc ; }
	
	private String name = null ;
	final public String getName() 				{ return name ; }
	
	private String clas = null ;
	final public String getClassName() 			{ return clas ; }
	
	private String fact = null ;
	final public String getFactoryClassName()	{ return fact ; }
	
	private String loca = null ;
	final public String getFactoryLocation()	{ return loca ; }
	
	private Map<String,String> addrs = new HashMap<String,String>() ;
	final public Map<String,String> getAddresses() { return Collections.unmodifiableMap(addrs) ; }
	
	// IAspect
	final public JNDIResourceAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		return new JNDIResourceAdvice(this) ;
	}
	
	@SuppressWarnings("unchecked")
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")	
    final public void postCreate(Element para)
	throws Exception
	{
		IMetadata meta = Bean.getMetadata(this) ;
		
		String labl = meta.getLabel() ;	
		
		if(para==null)
			throw new Exception("Parameters must be defined") ;
		
		// Namspace
		Element nspLmen = para.getChild(ConstJNDI.NAMESPACE) ;
		if(nspLmen==null)
			throw new Exception("No namespace defined") ;
		
		// ex : java:/comp/env/jdbc
		nspc = nspLmen.getAttributeValue(ConstJNDI.NAME) ;
		if(nspc==null || nspc.trim().isEmpty())
			throw new Exception("Not a valid namespace ("+nspc+")") ;
		
		// Reference
		Element refLmen = para.getChild(ConstJNDI.REFERENCE) ;
		if(refLmen==null)
			throw new Exception("No reference defined") ;
		
		this.clas = refLmen.getAttributeValue(ConstJNDI.CLASSNAME) ;
		this.fact = refLmen.getAttributeValue(ConstJNDI.FACTORY) ;
		this.loca = refLmen.getAttributeValue(ConstJNDI.FACTORY_LOCATION) ;
		
		List<Element> list = refLmen.getChildren(ConstJNDI.ADDRESS) ;
		for(Element temp : list)
		{
			String type = temp.getAttributeValue(ConstJNDI.TYPE) ;
			String valu = temp.getAttributeValue(ConstJNDI.VALUE) ;
			
			addrs.put(type, valu) ;
		}
		
		// Resource
		Element ctxLmen = para.getChild(ConstJNDI.CONTEXT) ;
		if(ctxLmen!=null) bind = ctxLmen.getAttributeValue(ConstJNDI.BINDING) ;
		
		// build full bind name for further lookups (cf. createAdvice(..))
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(nspc) ;
		if(!nspc.endsWith("/")) bdr.append("/") ;
		bdr.append(labl) ;
		
		this.name = bdr.toString() ;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		JNDIResourceConsole.service(this, srv, req, res) ;
	}
}