/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.jndi.editor;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;

import org.azuki.bean.jndi.shared.ConstJNDI;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.*;
import org.azuki.common.widget.JFormPanel;
import org.jdom.*;

@SuppressWarnings("serial")
public class JNDIResourceEditor extends AbstractEditor
{
	final static private JDOMElementTableBinding tableBinding
	= JDOMElementTableBinding.create(	ConstJNDI.ADDRESS,
										ListUtil.asList(ConstJNDI.TYPE, ConstJNDI.VALUE),
										ListUtil.asList("Type", "Value")) ;
	
	private JDOMAttributeField namespace ;
	private JDOMAttributeField className ;
	private JDOMAttributeField factory ;
	private JDOMAttributeField factoryLocation ;
	
	private JDOMComboField bind ;
	
	private JDOMElementTablePanel paramsPane ;
	
	private JFormPanel form ;
	
	/*
	 * AbstractEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element nspLmen = JDOMUtil.getElement(para, ConstJNDI.NAMESPACE) ;
		Attribute nspAttr = JDOMUtil.getAttribute(nspLmen, ConstJNDI.NAME) ;
		
		Element refLmen = JDOMUtil.getElement(para, ConstJNDI.REFERENCE) ;
		Attribute refClasAttr = JDOMUtil.getAttribute(refLmen, ConstJNDI.CLASSNAME) ;
		Attribute refFactAttr = JDOMUtil.getAttribute(refLmen, ConstJNDI.FACTORY) ;
		Attribute refLocaAttr = JDOMUtil.getAttribute(refLmen, ConstJNDI.FACTORY_LOCATION) ;
		
		Element ctxLmen = JDOMUtil.getElement(para, ConstJNDI.CONTEXT) ;
		Attribute bindAttr = JDOMUtil.getAttribute(ctxLmen, ConstJNDI.BINDING) ;
		
		namespace = new JDOMAttributeField(nspAttr) ;
		className = new JDOMAttributeField(refClasAttr) ;
		factory   = new JDOMAttributeField(refFactAttr) ;
		factoryLocation = new JDOMAttributeField(refLocaAttr) ;
		
		bind = new JDOMComboField(ConstJNDI.CONTEXT_BINDING_KEYS) ;
		bind.setAttribute(bindAttr) ;
		bind.setEditable(true) ;
		
		paramsPane = new JDOMElementTablePanel(refLmen, tableBinding) ;
		
		form = new JFormPanel() ;
		form.addField(namespace, "Namespace") ;
		form.addField(className, "Class name") ;
		form.addField(factory, "Factory") ;
		form.addField(factoryLocation, "Factory location") ;
		form.addField(bind, "Context binding") ;
		form.addField(paramsPane, "Parameters") ;
		
		setDialogTitle("JNDI Editor") ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(form, BorderLayout.NORTH) ;
	}
}