/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger;

import java.util.*;
import java.util.logging.Level;

import org.azuki.bean.logger.server.imp.*;
import org.azuki.bean.logger.shared.*;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.Log;
import org.jdom.Element;

final public class DBHandlerBean extends AbstractHandlerBean<DBHandler>
{
	private int retention ;
	
	final public String getDataSource()
	{ return getHandler().getDataSource() ; }
	
	final public int getRetentionDay()
	{ return retention ; }
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")	
	final public void postCreate(Element para)
	throws Exception
	{
		Element logLmen = JDOMUtil.getElement(para, ConstHandler.LOG) ;
		
		String logDatasource = logLmen.getAttributeValue(ConstDBHandler.LOG_DATASOURCE) ;
		if(logDatasource==null || logDatasource.isEmpty()) 
			throw new Exception("Invalid datasource ("+logDatasource+")") ;

		String logLevel = logLmen.getAttributeValue(ConstHandler.LOG_LEVEL) ;
		if(logLevel==null || logLevel.isEmpty()) 
		{
			Log.warning("Invalid level("+logLevel+"), use default value ("+ConstDBHandler.defaultLevel+")") ;
			logLevel = ConstDBHandler.defaultLevel ;
		}
		
		String daysStr = logLmen.getAttributeValue(ConstDBHandler.LOG_RETENTION) ;
		if(daysStr==null || daysStr.isEmpty())
		{
			Log.warning("Invalid days("+daysStr+"), use default value ("+ConstDBHandler.defaultRetention+")") ;
			daysStr = ConstDBHandler.defaultRetention.toString() ;
		}
		retention = Integer.parseInt(daysStr) ;

		// create the DB Handler
		DBHandler dbHandler = new DBHandler(logDatasource) ;
		dbHandler.setLevel(Level.parse(logLevel)) ;
		
		Log.info("\t (*) Create DBHandler database '"+logDatasource+"'") ;
		Log.info("\t     Level="+logLevel) ;
		
		// allocate handler
		setHandler(dbHandler);
	}
	
	@Override
	final public void purge()
	throws Exception
	{
  		UtilDBHandler.deleteLogs(getHandler().getDataSource(), retention) ;
	}
	
	@Override
	final public String consoleInfo()
	{
		Map<String,Object> map = new HashMap<String,Object>() ;
		map.put("Level", getHandler().getLevel().getName()) ;
		map.put("JNDI DataSource", getHandler().getDataSource()) ;
		
		return HtmlFactory.propertyList("Properties", map).toString() ; 
	}

	@Override
	final public String shellInfo()
	{
		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append("Level : ").append(getHandler().getLevel().getName());
		bdr.append('\n') ;
		bdr.append("JNDI DataSource : ").append(getHandler().getDataSource()) ;
		bdr.append('\n') ;
		
		return bdr.toString() ;
	}
}