/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger ;

import java.util.*;
import java.util.logging.*;

import org.azuki.bean.logger.server.LoggerLifecycle;
import org.azuki.bean.logger.server.imp.*;
import org.azuki.framework.context.IContext;
import org.azuki.framework.object.*;

/**
 * @author Mathieu Jucker (mathieu.jucker@azuki-framework.org)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class LoggerFactoryBean implements IObjectFactory<Logger>, ILifecycleFactory<Logger>, ILogger
{
	final public String getObjectName()
	{
		return IContext.OBJECT_LOGGER ;
	}
	
//	 LoggerFactoryBean returns always the same instance : The default Logger
	private Logger lgr = null ;  
	
	final public Logger createObject(Object... args)
	{
		// create new logger instance
		if(lgr!=null) return lgr ;

		synchronized(this)
		{
			if(lgr!=null) return lgr ;
						
			Logger localgr = Logger.getAnonymousLogger() ;
			
			localgr.setLevel(Level.ALL) ;
			localgr.setUseParentHandlers(false) ;
	
			// add handlers
			for(Handler hdlr : hdlrs)
				localgr.addHandler(hdlr) ;
			
			lgr = localgr ;
		}
		
		return lgr ;
	}

	final public ILifecycle<Logger> getLifeCycle()
	{
		return new LoggerLifecycle() ;
	}
	
	// ILogger
	private Collection<Handler> hdlrs = new HashSet<Handler>() ;
	final public void addHandler(Handler hdlr) { hdlrs.add(hdlr) ; }
}