/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger.server;

import org.azuki.bean.logger.LoggerBean;
import org.azuki.framework.context.*;
import org.azuki.framework.context.IContext.IStub;
import org.azuki.framework.injection.*;

/**
 * @author Mathieu Jucker (mathieu.jucker@azuki-framework.org)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class LoggerAdvice implements IController
{
	final private LoggerBean logb ;
	
	private boolean enab = true ;
	final public void setEnabled(boolean enab) { this.enab = enab ; }
	final public boolean isEnabled() { return enab ; }
	
	public LoggerAdvice(LoggerBean logb)
	{
		this.logb = logb ;
	}
	
	// IController
	final public Object intercept(InvocationTarget it)
	throws Exception
	{
		if(!enab) return it.proceed() ;
		
		IContext ictx = ThreadContext.get() ;
		
		IStub stub = ictx.bind(IContext.OBJECT_LOGGER, logb.getLogger(), new LoggerLifecycle()) ;
		
		try
		{ return it.proceed() ; }
		finally
		{ ictx.unbind(IContext.OBJECT_LOGGER, stub) ; }
	}
}
