/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger.server.imp;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.* ;

import org.azuki.framework.context.Log;

public class FileLogFormatter extends Formatter
{
	final static String NL = System.getProperty("line.separator");
	final static private SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yy HH:mm:ss") ;
    
    @Override
	final public String format(LogRecord lr)
    {
        String lvl = lr.getLevel().getName() ;
        String msg = lr.getMessage() ;
        long tim = lr.getMillis() ;
        int tdn = lr.getThreadID() ;

        String src = lr.getSourceClassName() ; // Can be null
		Throwable det = lr.getThrown() ;

		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append(sdf.format(new Date(tim))).append('.').append(lvl) ;

		if(tdn>=0) // tdn == -1 si LogFile
		{
			bdr.append('.') ;
			if(tdn<10) bdr.append("0") ;
			bdr.append(tdn) ;
		}

		if(src!=null) 	bdr.append("(").append(src).append(") ") ;
		else			bdr.append(' ') ;

		bdr.append(msg) ;
		bdr.append("\n") ;

		if(det != null)
		{
			bdr.append(NL) ;
			bdr.append("\t-- ") ;
			bdr.append(Log.getStack(det)) ;
		}

		return bdr.toString() ;	    
    }
}