/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger.server.shell ;

import org.azuki.bean.logger.LoggerBean;
import org.azuki.bean.logger.server.LoggerAdvice;
import org.azuki.bean.logger.shared.ConstLogger;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class LoggerShell
{
	final static public String listPointcuts(LoggerBean aspt)
	throws Exception
	{
		IMetadata meta = aspt.getMetadata() ;
		
		StringBuilder bdr = new StringBuilder() ;

		bdr.append("Logger [").append(Util.oneString(meta.getName(), meta.getLabel())).append("]").append('\n') ;
		bdr.append("Level : ").append(aspt.getLogger().getLevel()).append('\n') ;
		bdr.append('\n') ;

		// headers
		StringBuilder tmp = new StringBuilder() ;
		tmp.append(ShellUtil.rpad("Bean", ConstLogger.SZ_BEAN)) ;
		tmp.append(ShellUtil.rpad("Enabled", ConstLogger.SZ_ENABLED)) ;
		tmp.append("\n") ;
		tmp.append(ShellUtil.rpad("", tmp.length(), '-')) ;
		tmp.append("\n") ;
		
		bdr.append(tmp) ;
		
		// lines
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			IMetadata tmeta = pc.getTargetMetadata() ;
			LoggerAdvice advc =(LoggerAdvice)pc.getAdvice() ;
			if(advc==null) continue ;
			
			bdr.append(ShellUtil.rpad(Util.oneString(tmeta.getName(), tmeta.getLabel()), ConstLogger.SZ_BEAN)) ;
			bdr.append(ShellUtil.rpad(String.valueOf(advc.isEnabled()), ConstLogger.SZ_ENABLED)) ;
			
			bdr.append("\n") ;
		}
		
		return bdr.toString() ;
	}
	
	final static public String setEnabled(LoggerBean aspt, String bkey, boolean enab)
	throws Exception
	{
		IMetadata meta = aspt.getMetadata() ;
		String akey = Util.oneString(meta.getName(), meta.getLabel()) ;
		
		StringBuilder bdr = new StringBuilder() ;
		
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			IMetadata tmeta = pc.getTargetMetadata() ;
			String tkey = Util.oneString(tmeta.getName(), tmeta.getLabel()) ;
			
			LoggerAdvice advc =(LoggerAdvice)pc.getAdvice() ;
			
			if(bkey==null || bkey.equals(tkey))
			{
				advc.setEnabled(enab) ;
				bdr.append("Logger (").append(akey).append(")") ;
				bdr.append(" advice on bean (").append(tkey).append(')') ;
				bdr.append(" ").append(enab?"enabled":"disabled").append('\n') ;
			}
		}
		
		return bdr.toString() ;
	}
}