/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.mail;

import java.util.Map;

import javax.mail.*;
import javax.mail.internet.MimeMessage;
import javax.naming.*;

import org.azuki.bean.mail.shared.ConstMail;
import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

public class MailPOP3Bean
{
	private String user ;
	private String pswd ;
	
	private String jndi;
	
	private boolean isValid = false;
	
	@Export
	public javax.mail.Message[] recv()
	throws Exception
	{ 
		if (!isValid) throw new Exception("cant receive mail via pop3 : "+getClass().getName() +" parameters are invalid");
		Context ctx = new InitialContext();
		Session ses = (Session) ctx.lookup("java:/comp/env/"+jndi);
		return rcv(ses, ConstMail.POP3_PROTOCOL, user, pswd); 
	}

	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		Map<String,String> param = EditorUtil.formToMap(para);
		
		if (param==null)
		{
			Log.severe("ERROR : "+ getClass().getName() +" parameter are empty !");
			throw new Exception("ERROR : "+ getClass().getName() +" parameter are empty !"); 
		}
		
		jndi = param.get(ConstMail.KEY_JNDI_MAIL_RESOURCE);
		user = param.get(ConstMail.KEY_USER);
		pswd = param.get(ConstMail.KEY_PASSWORD);
		
		if (jndi==null||jndi.isEmpty())
		{
			Log.severe("ERROR : "+getClass().getName()+" JNDI_MAIL_RESOURCE parameter is null");
			throw new Exception("ERROR : "+getClass().getName()+" JNDI_MAIL_RESOURCE parameter is null");
		}
		
		if (user==null||user.isEmpty())
		{
			Log.severe("ERROR : "+getClass().getName()+" USER parameter is null");
			throw new Exception("ERROR : "+getClass().getName()+" USER parameter is null");
		}

		if (pswd==null||pswd.isEmpty())
		{
			Log.severe("ERROR : "+getClass().getName()+" passwork parameter is null");
			throw new Exception("ERROR : "+getClass().getName()+" PASSWORD parameter is null");
		}
		
		isValid = true;
	} 
	
	final private javax.mail.Message[] rcv(Session session, String protocol, String user, String password)
	throws Exception
	{
		Log.info("Receive Mails...");
		// Get the store and connect
		Store store = session.getStore(protocol) ;
		store.connect(null,user, password) ;
		
		// Open the default folder
		// 'INBOX'  is reserved to mean the "primary folder for this user on this server"
		Folder folder = store.getFolder("INBOX") ;
		folder.open(Folder.READ_WRITE) ;
		
		javax.mail.Message[] msg = folder.getMessages() ;
		javax.mail.Message[] ret = clone(msg) ;
		
		// Consume messages
		for(int i=0 ; i<msg.length ; i++)
			msg[i].setFlag(Flags.Flag.DELETED, true) ;
		
		// Close connection
		folder.close(true) ;	// true: delete messages marked as deleted
		store.close() ;
		
		return ret ;
	}
	
	final static private Message[] clone(Message[] m)
	throws Exception
	{
		Message[] clon = new Message[m.length] ;
		for(int i=0 ; i<m.length ; i++)
			clon[i] = new MimeMessage((MimeMessage)m[i]) ; //FIXME: not really nice
		return clon ;
	}
}
