package org.azuki.bean.ooo.server.imp ;

import java.io.*;
import java.util.*;
import java.util.jar.*;

import org.azuki.bean.ooo.shared.ConstOOO;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.context.Log;
import org.jdom.*;

/*
 * <draw:frame draw:style-name="fr3" draw:name="tata" text:anchor-type="paragraph" svg:width="1.685cm" svg:height="2.332cm" draw:z-index="23">
 *   <draw:image xlink:href="Pictures/100002010000000100000001285BC994.png" xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad"/> 
 * </draw:frame>
 * 
 * xmlns:ooow="http://openoffice.org/2004/writer"
 * xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
 * xmlns:dc="http://purl.org/dc/elements/1.1/"
 * xmlns:ooo="http://openoffice.org/2004/office"
 * xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
 * xmlns:xforms="http://www.w3.org/2002/xforms"
 * xmlns:form="urn:oasis:names:tc:opendocument:xmlns:form:1.0"
 * xmlns:svg="urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"
 * xmlns:dom="http://www.w3.org/2001/xml-events"
 * xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
 * xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
 * xmlns:script="urn:oasis:names:tc:opendocument:xmlns:script:1.0"
 * xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0"
 * xmlns:math="http://www.w3.org/1998/Math/MathML"
 * xmlns:draw="urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"
 * xmlns:dr3d="urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"
 * xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0"
 * xmlns:xsd="http://www.w3.org/2001/XMLSchema"
 * xmlns:xlink="http://www.w3.org/1999/xlink"
 * xmlns:chart="urn:oasis:names:tc:opendocument:xmlns:chart:1.0"
 * xmlns:oooc="http://openoffice.org/2004/calc"
 * xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
 * xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
 */

public class OOImageReplacer
{
	final static public Namespace NS_DRAW 	= Namespace.getNamespace("draw", "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0") ;
	final static public Namespace NS_XLINK 	= Namespace.getNamespace("xlink", "http://www.w3.org/1999/xlink") ;
	
	final static public String FRAME = "frame" ; // Tag
	final static public String FRAME_NAME = "name" ;
		final static public String IMAGE = "image" ; // Tag
		final static public String IMAGE_HREF = "href" ;
		
	final static public String PICTURES_LOCATION = "Pictures" ;
	
	final static public byte[] replace(byte[] odtb, Map<String,byte[]> imgs)
	throws Exception
	{
		// read jar file
		ByteArrayInputStream bais = new ByteArrayInputStream(odtb) ;
		
		Map<String,byte[]> jarm = readJAR(bais) ;
		
		try{ bais.close() ; } catch(Exception ex){}
		
		// parsing 'content.xml'
		byte[] cxml = jarm.get(ConstOOO.ODT_CONTENT_ENTRY) ;
		if(cxml==null) return odtb ;
		
		ByteArrayInputStream bis = new ByteArrayInputStream(cxml) ;
		
		Document doc = JDOMUtil.read(bis) ;
		
		try{ bis.close() ; } catch(Exception ex){}
		
		// replace
		String xqry = "//"+NS_DRAW.getPrefix()+":"+FRAME ;
		
		List<Element> lmens = JDOMUtil.query(doc, xqry) ;
		
		for(Element lmen : lmens)
		{
			String fnam = lmen.getAttributeValue(FRAME_NAME, NS_DRAW) ;
			if(fnam==null) continue ;
			
			Log.finest("Found image: "+fnam) ;
			
			if(!imgs.containsKey(fnam)) continue ;
			
			byte[] byts = imgs.get(fnam) ;
			if(byts==null) continue ;
			
			String href = PICTURES_LOCATION+"/"+fnam ;
			
			Element imag = lmen.getChild(IMAGE, NS_DRAW) ;
			imag.setAttribute(IMAGE_HREF, href, NS_XLINK) ;
			
			Log.finest("... replacing image with: "+href) ;
			jarm.put(href, byts) ;
		}
		
		// no replacement done !
		if(jarm.isEmpty()) return odtb ;
		
		// updating 'content.xml'
		ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
		
		JDOMUtil.xmlout.output(doc, baos) ;
		
		try{ baos.close() ; } catch(Exception ex){}
		
		jarm.put(ConstOOO.ODT_CONTENT_ENTRY, baos.toByteArray()) ;

		// write jar file
		baos = new ByteArrayOutputStream() ;
		
		writeJAR(jarm, baos) ;
		
		try{ baos.close() ; } catch(Exception ex){}
				
		return baos.toByteArray() ;
	}
	
	final static public byte[] read(InputStream is)
	throws Exception
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
		byte[] buf = new byte[4096] ;
		
		while(true)
		{
			int read = is.read(buf) ;
			if(read==-1) break ;
			
			baos.write(buf, 0, read) ;
		}
		
		baos.close() ;
		
		return baos.toByteArray() ;
	}
	
	final static public Map<String,byte[]> readJAR(InputStream is)
	throws Exception
	{
		Map<String,byte[]> map = new HashMap<String,byte[]>() ;
	
		JarInputStream jis = new JarInputStream(is) ;
		
		while(true)
		{
			JarEntry jety = jis.getNextJarEntry() ;
			if(jety==null) break ;
			
			String name = jety.getName() ;
			
			if(jety.isDirectory())
			{	map.put(name, null) ;
				continue ; }
			
			byte[] byts = read(jis) ;
			
			map.put(name, byts) ;
		}
		
		jis.close() ;
		
		return map ;
	}
	
	final static public void writeJAR(Map<String,byte[]> etys, OutputStream os)
	throws Exception
	{
		JarOutputStream jos = new JarOutputStream(os) ;
		
		for(Map.Entry<String,byte[]> ety : etys.entrySet())
		{
			String name = ety.getKey() ;
			byte[] byts = ety.getValue() ;
			
			JarEntry jety = new JarEntry(name) ;
			
			jos.putNextEntry(jety) ;
			
			if(byts!=null) jos.write(byts) ;
			
			jos.closeEntry() ;
		}
		
		jos.finish() ;
	}
}