/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.oracle.server.imp ;

import java.sql.*;

import org.azuki.bean.oracle.DBMSWaitoneBean;
import org.azuki.bean.oracle.shared.ConstDBMSAlert;
import org.azuki.framework.command.ICommand;
import org.azuki.framework.context.*;

public class JobWait<T> implements ICommand<T>
{
	// End signal
	final static private String SIGTERM = "SIGNAL(TERM)" ;
	
	// Signal
	final static private String SQLSIG  = "{ call dbms_alert.signal('" ;
	final static private String SQLMID  = "', '" ;
	final static private String SQLEND = "')}" ;
	
	// Register
	final static private String SQLREG  = "{ call dbms_alert.register('" ;
	final static private String SQLREND = "')}" ;

	// Wait
	final static private String SQLWAIT = "{ call dbms_alert.waitone('" ;
	final static private String SQLWEND = "', ?, ?)}" ;
	
	// Unregister
	final static private String SQLDREG = "{ call dbms_alert.remove('" ;
	final static private String SQLDEND = "')}" ;
	
	
	final private DBMSWaitoneBean bean ;
	final private String evnt ;
	
	private String info ;
	
	public JobWait(DBMSWaitoneBean bean, String evnt)
	{
		this.bean = bean ;
		//this.datasource = datasource ;
		this.evnt = evnt ;
		this.info = new String("WAITONE("+evnt+")") ;
	}
	
	public String getInfo()
	{
		return info ;
	}
	
	private boolean stop = false ;
	
	final public void mustStop()
	{
		stop = true ;
		
		if(cs2==null) return ;
		
		Connection cx = null ;
		
		try
		{
			// N'interrompt pas la session ORACLE !!
			//cs2.cancel() ;
			
			String sql = new StringBuffer(SQLSIG).append(evnt)
			.append(SQLMID).append(SIGTERM).append(SQLEND).toString() ;

			cx = (Connection)ThreadContext.lookup(IContext.OBJECT_CONNECTION) ;

			CallableStatement cs = cx.prepareCall(sql) ;
			cs.execute() ;
			cs.close() ;
			
			Log.info("DBMS Waitone ("+evnt+") request stop.") ;

			cx.commit() ; // Send
		}
		catch(Exception e)
		{
			Log.severe(Log.getStack(e)) ;
		}
		finally
		{
			try
			{ if(cx!=null) cx.close() ; }
			catch (SQLException e)
			{ Log.severe("Error trying to close connection : "+Log.getStack(e)) ; }
		}
	}
	
	private CallableStatement cs2 = null  ;
	
	final public T process()
	throws Exception
	{
		Connection cx = null ;
		CallableStatement cs1 = null  ;
		CallableStatement cs3 = null  ;
		try
		{
			// JNDI Database connection

			cx = (Connection)ThreadContext.lookup(IContext.OBJECT_CONNECTION) ;
			
			Log.info("retreive db connection:"+cx);
			// Register 
			String sql = new StringBuffer(SQLREG)
			.append(evnt).append(SQLREND).toString() ;
			cs1 = cx.prepareCall(sql) ;
			cs1.execute() ;
			
			// WAIT DB event
			sql = new StringBuffer(SQLWAIT)
			.append(evnt).append(SQLWEND).toString() ;
			cs2 = cx.prepareCall(sql) ;
			cs2.registerOutParameter(1, Types.VARCHAR) ;
			cs2.registerOutParameter(2, Types.VARCHAR) ;
						
			try
			{
				while(true)
				{
					Log.info("DBMS Waitone ("+evnt+")") ;
					cs2.execute() ;
					
					Log.info("Receive DBMS alert : generate an EBP") ;

					// EBP
					bean.publish.call(bean,ConstDBMSAlert.OUTPUT_NAME,cs2.getString(1)) ;
					
					Log.info("DBMS Waitone ("+evnt+")") ;
					cs2.execute() ;
					
					String sign = cs2.getString(1) ;
					if(stop /*&& SIGTERM.equals(sign)*/) break ;
					
					// EBP
					bean.publish.call(bean, ConstDBMSAlert.OUTPUT_NAME, sign) ;
				}
			}
			finally
			{
				// Unregister
				sql = new StringBuffer(SQLDREG).append(evnt).append(SQLDEND).toString() ;
				cs3 = cx.prepareCall(sql) ;
				cs3.execute() ;
			}
		}
		catch(Exception e)
		{
			try { if(cs3!=null) cs3.close() ; } catch(Exception ee) {} cs3=null;
			try { if(cs2!=null) cs2.close() ; } catch(Exception ee) {} cs2=null;
			try { if(cs1!=null) cs1.close() ; } catch(Exception ee) {} cs1=null;
			try { if(cx !=null)  cx.close() ; } catch(Exception ee) {}  cx=null;
		}
		finally
		{
			if (cx!=null)
				cx.close() ;
		}
		return null;
	}
}
