/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.request.server.imp;

import javax.servlet.*;
import javax.servlet.http.*;

import org.azuki.bean.request.RequestHandlerBean;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.Framework;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

//
// Bridge Servlet
//

@SuppressWarnings("serial")
public class BridgeServlet extends HttpServlet
{
    static private boolean firstime = true ;

	final static private String LOOKUP_NAME = RequestHandlerBean.NAME ;
	final static private String LOOKUP_LABL = RequestHandlerBean.LABL ;

    @Override
	final public void init(ServletConfig cfg)
    throws ServletException
    {
        super.init(cfg) ;

        if(firstime) firstime = false ;
        else return ; // Only once
      
        try
        {
        	String cpat = getContextPath(cfg) ;
        	Log.info("Azuki framework context path ("+cpat+")") ;
        	
        	RequestHandlerBean.setContextPath(cpat) ;
        	
	    	Framework.init(Thread.currentThread().getContextClassLoader()) ;
	    	
	    	mp = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, LOOKUP_NAME, LOOKUP_LABL, "service") ;
        }
        catch(Exception ex)
        {
            ex.printStackTrace() ;
            throw new ServletException("Error occured on servlet's init() method : ", ex) ;
        }
    }	
	
    @Override
    final public void destroy()
    {
    	Framework.stop() ;
    }
    
	private IMethodProxy mp = null ;
	
    @Override
	final public void service(HttpServletRequest req, HttpServletResponse res)
    throws ServletException
    {
    	try
        {
        	mp.call(this, req, res) ;
        }
        catch(ServletException ex)
        {
            throw ex ;
        }
        catch(Exception ex) // Autre exception
        {
            // ex.printStackTrace();
            throw new ServletException("Error occured on servlet's service() method : ", ex) ;
        }
		finally
		{
			ThreadContext.reset() ; // Clean thread context
		}
    }
    
    final static public String getContextPath(ServletConfig cfg)
    throws Exception
    {
    	ServletContext sctx = cfg.getServletContext() ;
    	
    	float spec = Float.parseFloat(sctx.getMajorVersion()+"."+sctx.getMinorVersion()) ;
    	Log.info("Using Servlets API ("+spec+")") ;
    	
    	if(spec>=2.5f) return cfg.getServletContext().getContextPath() ;
    	
    	String surl = cfg.getServletContext().getResource("/").toString() ;
    	surl = StringUtil.unslashed(surl) ;
    	
    	int idx = surl.lastIndexOf('/') ;
    	if(idx==-1) throw new Exception("Cannot resolve context path from: "+surl) ;
    	
    	return surl.substring(idx) ;
    }
}