/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.request.shared.contract ;

import java.lang.reflect.*;

import javax.servlet.http.*;

import org.azuki.common.reflect.ReflectUtil;
import org.azuki.framework.aop.IContract;

/**
* 
* @author Robert Bakic
*
* Contract : The contract is to have a method with this arguments:
* 
*    - (HttpServlet, HttpServletRequest, HttpServletResponse)  
*
*/

final public class RequestHandlerContract implements IContract
{
	final static public Class[] ARGTYPE = {HttpServlet.class, HttpServletRequest.class, HttpServletResponse.class} ;
	
	final public void checkBindable(Class beanclass)
	throws Exception
	{
		if(getMethodCandidates(beanclass)==null || getMethodCandidates(beanclass).length==0)
			throw new Exception("One method should contains arguments HttpServlet.class, HttpServletRequest.class, HttpServletResponse.class") ;
		
	}		

	final public Selection getMethodSelection() { return Selection.ONLY_ONE; }		
	final public Method[] getMethodCandidates(Class bclass)
	{
		return ReflectUtil.getMethodWithParamTypes(bclass, ARGTYPE);
	}

	final public Selection getFieldSelection() { return Selection.NONE; }
	final public Field[] getFieldCandidates(Class bclass) {	return null ; }		
}