/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.scheduler.editor.imp;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.jdom.Attribute;

@SuppressWarnings("serial")
public class DaySelectorPanel extends JPanel implements ItemListener, ActionListener
{
	final static public int WEEK = 0 ;
	final static public int MONTH = 1 ;
	
	final static private String[] DAYS = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"} ;
	
	final static private int COLS = 7 ;
	
	private Attribute attr ;
	
	final private int delay ;
	
	private JButton all, none ;
	private JPanel boxes, buttons ;
	
	// CONSTRUCTEUR
	public DaySelectorPanel(int delay)
	{
		this.delay = delay ; 
		
		boxes = new JPanel() ;
		
		all = new JButton("All") ;
		all.addActionListener(this) ;
		
		none = new JButton("None") ;
		none.addActionListener(this) ;
		
		buttons = new JPanel() ;
		buttons.setLayout(new FlowLayout(FlowLayout.RIGHT)) ;
		buttons.add(all) ;
		buttons.add(none) ;
		
		setLayout(new BorderLayout()) ;
		add(boxes, BorderLayout.NORTH) ;
		add(buttons, BorderLayout.SOUTH) ;
	}
	
	// selection globale
	final private void setOverallSelection(boolean sel)
	{
		for(int i=0 ; i<boxes.getComponentCount() ; i++)
			((JCheckBox) boxes.getComponent(i)).setSelected(sel) ;
	}
	
	final public void load(Attribute attr)
	{
		this.attr = attr ;
		
		char[] c = attr.getValue().toCharArray() ;
		
		boxes.removeAll() ;
		boxes.setLayout(new GridLayout(c.length/COLS+1, COLS)) ;
		for(int i=0 ; i<c.length ; i++)
		{
			JCheckBox box = new JCheckBox() ;
			String label = delay==WEEK ? DAYS[i] : String.valueOf(i+1) ;
			box.setText(label) ;
			box.setName(String.valueOf(i)) ;
			box.setSelected(c[i]=='1') ;
			box.addItemListener(this) ;
			boxes.add(box) ;
		}
		
		boxes.revalidate() ;
	}

	// ItemListener
	final public void itemStateChanged(ItemEvent e)
	{
		JCheckBox box = (JCheckBox) e.getSource() ;
	
		int idx = Integer.parseInt(box.getName()) ;
		
		StringBuffer sb = new StringBuffer(attr.getValue()) ;
		sb.setCharAt(idx, box.isSelected() ? '1' : '0') ;
		
		attr.setValue(sb.toString()) ;
	}

	// ActionListener
	final public void actionPerformed(ActionEvent e)
	{
		boolean select = false ;
		if(e.getSource()==all) select = true ;
			
		setOverallSelection(select) ;
	}
}