/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.scheduler.server.imp ;

import java.util.Calendar;


// TODO : add second
final public class ATimer
{
	final static private boolean[] template = new boolean[31] ;

	static
	{
		for(int i=0; i<template.length; i++) template[i] = true ;
	}

	private boolean[] days = new boolean[31] ;

	private boolean mon = true ; // default set to all day
	private boolean tue = true ;
	private boolean wed = true ;
	private boolean thu = true ;
	private boolean fri = true ;
	private boolean sat = true ;
	private boolean sun = true ;

	private long period = Times.DAY ; // default

	private int hour   = -1 ; // default set to current
	private int minute = -1 ;
	private int second = -1 ;
	
	public ATimer()
	{
		// default to all day
		System.arraycopy(template, 0, days, 0, template.length) ;
	}

	@Override
	final public String toString()
	{
		StringBuffer buf = new StringBuffer() ;
		buf.append("Period     : ");
		buf.append(periodAsTime(period));
		buf.append("\n");

		if(hour != -1)
		{
			buf.append("Hour       : ");
			buf.append(hour);
			buf.append("\n");
			buf.append("Minute     : ");
			buf.append(minute);
			buf.append("\n");
			// add second MJ
			buf.append("Second    : ");
			buf.append(second);
			buf.append("\n");
			
		}

		boolean all = true ;
		for(int i=0; i<days.length; i++)
		{
			all = all & days[i] ;
		}

		buf.append("Month Days : ") ;

		if(all)
		{
			buf.append("All");
		}
		else
		{
			for(int i=0; i<days.length; i++)
			{
				if(days[i])
				{
					buf.append(i+1) ;
					buf.append('.');
				}
			}
		}

		buf.append('\n');

		buf.append("Week Days  :") ;

		all = mon & tue & wed & thu & fri & sat & sun ;

		if(all)
		{
			buf.append(" All") ;
		}
		else
		{
			if(mon)  buf.append(" mon") ;
			if(tue)  buf.append(" tue") ;
			if(wed)  buf.append(" wed") ;
			if(thu)  buf.append(" thu") ;
			if(fri)  buf.append(" fri") ;
			if(sat)  buf.append(" sat") ;
			if(sun)  buf.append(" sun") ;
		}

		return buf.toString() ;
	}

	final static private String periodAsTime(long period)
	{
		int h = (int)(period/Times.HOUR) ;
		period -= h*Times.HOUR ;
		int m = (int)(period/Times.MINUTE) ;
		period -= m*Times.MINUTE;
		int s = (int)(period/Times.SECOND) ;

		StringBuffer sb = new StringBuffer();
		if(h<10) sb.append('0');
		sb.append(h);
		sb.append('h');
		if(m<10) sb.append('0');
		sb.append(m) ;
		sb.append('m');
		if(s<10) sb.append('0');
		sb.append(s) ;
		sb.append('s') ;

		return sb.toString() ;
	}

	// Prevent GTimer for further modification
	final public ATimer copy()
	{
		ATimer clone = new ATimer() ;
		clone.mon = mon ;
		clone.tue = tue ;
		clone.wed = wed ;
		clone.thu = thu ;
		clone.fri = fri ;
		clone.sat = sat ;
		clone.sun = sun ;
		System.arraycopy(days, 0, clone.days, 0, days.length) ;
		clone.period = period ;
		clone.hour = hour ;
		clone.minute = minute ;
		clone.second = second ;
		return clone ;
	}

	final public void check()
	throws IllegalArgumentException
	{
		boolean ok = mon | tue | wed | thu | fri | sat | sun ;

		for(int i=0; i<days.length; i++)
		{
			ok = ok | days[i] ;
		}

		if(!ok)
		throw new IllegalArgumentException("You must specify at least one day.") ;
	}

// PERIOD
	final public long getPeriod()
	{
		return period ;
	}

	final public void setPeriod(long period)
	{
		this.period = period ;
	}

	// TIME
	final public void setTime(int hour, int minute, int second)
	throws IllegalArgumentException
	{
		if(hour<0)  throw new IllegalArgumentException('('+hour+") Hour must be at least equal to 0.");
		if(hour>23) throw new IllegalArgumentException('('+hour+") Hour must be lower than 24.");

		if(minute<0)  throw new IllegalArgumentException('('+minute+") Minute must be at least equal to 0.");
		if(minute>59) throw new IllegalArgumentException('('+minute+") Minute must be lower than 60.");

		if(second<0)  throw new IllegalArgumentException('('+second+") Second must be at least equal to 0.");
		if(second>59) throw new IllegalArgumentException('('+second+") Second must be lower than 60.");

		this.hour = hour ;
		this.minute = minute ;
		this.second = second ;
	}

	final public int getHour()
	{
		return hour ;
	}

	final public int getMinute()
	{
		return minute ;
	}

	final public int getSecond()
	{
		return second;
	}
	
	// WEEK
	final public boolean isDayOfWeek(int day)
	throws IllegalArgumentException
	{
		switch(day)
		{
			case Calendar.MONDAY    : return mon ;
			case Calendar.TUESDAY   : return tue ;
			case Calendar.WEDNESDAY : return wed ;
			case Calendar.THURSDAY  : return thu ;
			case Calendar.FRIDAY    : return fri ;
			case Calendar.SATURDAY  : return sat ;
			case Calendar.SUNDAY    : return sun ;
			default :
			throw new IllegalArgumentException("Illegal number, see class Calendar.") ;
		}
	}

	final public void setAllDaysOfWeek(boolean enable)
	{
		mon = enable ;
		thu = enable ;
		wed = enable ;
		tue = enable ;
		fri = enable ;
		sat = enable ;
		sun = enable ;
	}

	/*
		 * Sets the timer for the specified day of the week.<br> 
		 * The processing handled by the timer will be executed only that day.
		 * @param day The day (Calendar.MONDAY, Calendar.TUESDAY, etc...)
		 * @param enable <true> or <false>
		 * @throws IllegalArgumentException
	 */
	
	final public void setDayOfWeek(int day, boolean enable)
	throws IllegalArgumentException
	{
		switch(day)
		{
			case Calendar.MONDAY :
				mon = enable ;
				break ;
			case Calendar.TUESDAY :
				tue = enable ;
				break ;
			case Calendar.WEDNESDAY :
				wed = enable ;
				break ;
			case Calendar.THURSDAY :
				thu = enable ;
				break ;
			case Calendar.FRIDAY :
				fri = enable ;
				break ;
			case Calendar.SATURDAY :
				sat = enable ;
				break ;
			case Calendar.SUNDAY :
				sun = enable ;
				break ;
			default :
				throw new IllegalArgumentException("Illegal day, see class Calendar.") ;
		}
	}

	// MONTH
	final public boolean isDayOfMonth(int number)
	throws IllegalArgumentException
	{
		if(number > 31)
			throw new IllegalArgumentException("Day number cannot exceed 31.") ;

		if(number < 1)
			throw new IllegalArgumentException("Day number must be greater than 0.") ;

		return days[number-1] ; // Zero based
	}

	final public void setAllDaysOfMonth(boolean enable)
	{
		for(int i=0; i<days.length; i++) days[i] = enable ;
	}

	/**
	 * Sets the timer for the specified day of the month.<br> 
	 * The processing handled by the timer will be executed only that day.
	 * @param number The month (Calendar.JANUARY, Calendar.FEBRUARY, etc...)
	 * @param enable <true> or <false>
	 * @throws IllegalArgumentException
	 */
	final public void setDayOfMonth(int number, boolean enable)
	throws IllegalArgumentException
	{
		if(number > 31)
			throw new IllegalArgumentException("Day number cannot exceed 31.") ;

		if(number < 1)
			throw new IllegalArgumentException("Day number must be greater than 0.") ;

		days[number-1] = enable ;
	}
}