/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.scheduler.server.imp ;

import java.util.*;

import org.azuki.bean.scheduler.shared.ConstSched;
import org.azuki.common.util.*;
import org.azuki.framework.command.ICommand;
import org.jdom.Element;

public class SchedulerImp 
{

	// Map of all scheduled jobs (running or pending)
	final private Hashtable<Integer, TT> all = new Hashtable<Integer, TT>() ;	
	
	
	// Execution cannot be handle by this Timer class
	// Because of thread management :
	// 1) There is only one timer.
	// 2) Job are sent to CPU.
	////////////////////////////////
	final private Timer timer = new Timer(true); // Already thread safe
	
	
	final public Integer scheduleJob(String jobName, ICommand icmd, Element para)
	throws IllegalArgumentException
	{
	    Element etype = para.getChild(ConstSched.TYPE) ;
        String type = etype.getAttributeValue(ConstSched.VALUE) ;
        
        int hh = Integer.parseInt(etype.getAttributeValue(ConstSched.HOUR)) ;
        int mm = Integer.parseInt(etype.getAttributeValue(ConstSched.MINU)) ;
        int ss = Integer.parseInt(etype.getAttributeValue(ConstSched.SECO)) ;

        long period = Times.DAY ; // AT

        ATimer timer = new ATimer() ;

        if(type.equalsIgnoreCase(ConstSched.PERIODIC))
            period = hh*Times.HOUR + mm*Times.MINUTE + ss*Times.SECOND ;
        else // AT
            timer.setTime(hh, mm, ss) ;
            
        timer.setPeriod(period) ;

        Element eweek = para.getChild(ConstSched.WEEK) ;
        boolean allw = Util.isOK(eweek.getAttributeValue(ConstSched.ALL)) ;
        timer.setAllDaysOfWeek(allw) ;
        if(!allw) calcWeek(eweek.getAttributeValue(ConstSched.DAYS), timer) ;

        Element emonth = para.getChild(ConstSched.MONTH) ;
        boolean allm = Util.isOK(emonth.getAttributeValue(ConstSched.ALL)) ;
        timer.setAllDaysOfMonth(allm) ;
        if(!allm) calcMonth(emonth.getAttributeValue(ConstSched.DAYS), timer) ;    	
		
		return scheduleJob(jobName, icmd, timer) ;
	}
	
    final static private void calcWeek(String src, ATimer timer)
    {
        char[] cs = src.toCharArray() ;
        if(cs[0]=='1') timer.setDayOfWeek(Calendar.MONDAY, true) ;
        if(cs[1]=='1') timer.setDayOfWeek(Calendar.TUESDAY, true) ;
        if(cs[2]=='1') timer.setDayOfWeek(Calendar.WEDNESDAY, true) ;
        if(cs[3]=='1') timer.setDayOfWeek(Calendar.THURSDAY, true) ;
        if(cs[4]=='1') timer.setDayOfWeek(Calendar.FRIDAY, true) ;
        if(cs[5]=='1') timer.setDayOfWeek(Calendar.SATURDAY, true) ;
        if(cs[6]=='1') timer.setDayOfWeek(Calendar.SUNDAY, true) ;
    }
    
    final static private void calcMonth(String src, ATimer timer)
    {
        char[] cs = src.toCharArray() ;
        for(int i=1; i<=31; i++) // 1 based
        {
            timer.setDayOfMonth(i, (cs[i]=='1')) ;
        }
    }	
	
	final public Integer scheduleJob(String jobName, ICommand icmd, long period, int hour, int minu, int sec,
			boolean[] weekdays, boolean[] monthdays)
	throws IllegalArgumentException
	{
		ATimer timer = new ATimer() ;
		
		// FIX BUG MJ : can't schedule a task to be execute juste one time at a fixed time
		if(period>=0) timer.setPeriod(period) ;
		
		if(hour!=-1)
			timer.setTime(hour, minu, sec) ;
		
		if(weekdays!=null)
		{
			// FIX BUG MJ : can't schedule a task to be execute juste one time at a fixed time : day start at 1 instead of 0 : generate scheduler error
			for(int i=0;i<weekdays.length;i++)
				timer.setDayOfWeek(i+1, weekdays[i]) ;
				
		}
		
		if(monthdays!=null)
		{
			// FIX BUG MJ : can't schedule a task to be execute juste one time at a fixed time : day start at 1 instead of 0 : generate scheduler error
			for(int i=0;i<monthdays.length;i++)
				timer.setDayOfMonth(i+1, monthdays[i]) ;
		}
	
		return scheduleJob(jobName,icmd, timer) ;
	}
	
	
	
	final private Integer scheduleJob(String jobName, ICommand icmd, ATimer time)
	throws IllegalArgumentException
	{		
		time.check();
		time = time.copy(); // prevent for further modifications

		// JTT encapsule le job pour verif / scheduling
		JTT jtt = new JTT(icmd, jobName, time, this) ; 
		
		long period = time.getPeriod();
		long delay ;

		if(time.getHour() == -1) // default
			delay = period ;
		else
			delay = calculDelay(time.getHour(), time.getMinute()) ;

		TT tt = new TT(jtt) ;
		
		Integer ID = jtt.getID() ;
		all.put(ID, tt) ;
		if (period==0)
			timer.schedule(tt, delay);
		else
			timer.schedule(tt, delay, period);
		return ID ;
	}

	final public Integer sendJobAfter(String jobName, ICommand icmd, long delai)
	{
        JTT jtt = new JTT(icmd, jobName, this) ;
		TT tt = new TT(jtt) ;
		Integer ID = jtt.getID() ;
		all.put(ID, tt) ;
		timer.schedule(tt, delai);
        return ID ;
	}

	final public void cancel(Integer ID)
	{
		TT tt = all.remove(ID) ;
		if(tt!=null)	tt.cancel() ;
	}

	final public void suspend(Integer ID, boolean suspend)
	{
		TT tt = all.get(ID);
		tt.getJTT().suspendJob(suspend);
	}
	
	final public List<Object[]> list()
	{
		ArrayList<Object[]> list = new ArrayList<Object[]>() ;
		Enumeration enu = all.elements() ;

		while(enu.hasMoreElements())
		{
			TT tt = (TT)enu.nextElement() ; 
			
			JTT jtt = tt.getJTT() ;
			
			Object[] os = new Object[5] ;
			os[0] = jtt.getID() ;
			os[1] = jtt.getJobName() ;

			ATimer timer = jtt.getTimer() ;
			if(timer != null) // null : Schedule once
			{
				if(timer.getHour() != -1)
					os[2] = ShellUtil.lpad(String.valueOf(timer.getHour()),2,'0') + ':'+ ShellUtil.lpad(String.valueOf(timer.getMinute()),2,'0') + ':' + ShellUtil.lpad(String.valueOf(timer.getSecond()),2,'0')   ;
				else
					os[2] = "Immediately" ;

				os[3] = new Long(timer.getPeriod()) ;
			}
			if (jtt.isSuspended())
				os[4] = "Idle";
			else
				os[4] = "Actif";
			
			list.add(os) ;
		}

		return list ;
	}
	
	// Calcul le delai en miliseconds entre maintenant et
	// l'heure passe en parametre (issu de GTimer)
	final static private long calculDelay(int rhour, int rminute)
	{
		Calendar dar = Calendar.getInstance() ;

		dar.set(Calendar.HOUR_OF_DAY, rhour);
		dar.set(Calendar.MINUTE, rminute);
		dar.set(Calendar.SECOND, 0);
		dar.set(Calendar.MILLISECOND, 0);

		// Target est l'heure de declenchement pour le jour meme
		long target = dar.getTime().getTime() ; // change this in 1.4 by getTimeInMillis()
		long now = System.currentTimeMillis() ;

		if(target > now) return target-now ;

		return target+Times.DAY-now ;
	}		
}