package org.azuki.bean.shell.server.imp ;

import java.net.ServerSocket;

import org.azuki.bean.shell.ShellBean;
import org.azuki.framework.command.*;
import org.azuki.framework.context.Log;

public class TcpListener implements ICommand
{
	final private ShellBean shel ;
	
	private ServerSocket lsnr ;
	
	public TcpListener(ShellBean shel)
	{
		this.shel = shel ;
	}
	
    final public boolean isRunning()
    {
    	return lsnr!=null ;
    }
    
    final public void start()
	{
		if(isRunning()) return ;
		
		int port = shel.getPort() ;
		
		Log.info("Shell starting on port "+port) ;
		
		try
		{
			lsnr = new ServerSocket(port) ;
			Command.invoke(this) ;
		}
		catch(Exception e)
		{
			Log.severe(Log.getStack(e)) ;
		}
	}
	
    final public void stop()
	{
		if(!isRunning()) return ;
		
		try
		{ 
			if(lsnr!=null) lsnr.close() ;
		}
		catch(Exception e)
		{
			Log.severe(Log.getStack(e)) ;
		}
		finally
		{
			lsnr = null ;	
		}
	}
    
    //
    // ICommand
    //
	final public Object process()
    throws Exception 
    {
		while(isRunning())
		{
			try
			{
				while(isRunning())
					Command.send(new TcpHandler(shel, lsnr.accept())) ;
			}
			catch(Exception e)
			{
				Log.severe(Log.getStack(e)) ;
				Thread.sleep(1000) ; // Wait, pour eviter un bouclage
			}
		}
		
		Log.info("Shell on port "+shel.getPort()+" stopped.") ;
		
		return null ;
	}
}