/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.shell.server.shell ;

import org.azuki.bean.shell.ShellBean;
import org.azuki.bean.shell.shared.ConstShell;
import org.azuki.common.bean.rpc.Service;
import org.azuki.common.beandoc.BeanDocumentation;
import org.azuki.common.framework.*;
import org.azuki.common.reflect.*;
import org.azuki.common.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Shell
{
	final static public String list(ShellBean shel)
	throws Exception
	{
		StringBuilder bdr = new StringBuilder() ;
		
		// headers
		StringBuilder hdr = new StringBuilder() ;
		hdr.append(ShellUtil.rpad("Service", ConstShell.WIDTH_SERVICE)) ;
		hdr.append(ShellUtil.rpad("Method", ConstShell.WIDTH_METHOD)) ;
		hdr.append(ShellUtil.rpad("Description", ConstShell.WIDTH_DESCRIPTION)) ;
		hdr.append("\n") ;
		hdr.append(ShellUtil.rpad("", hdr.length(), '-')) ;
		hdr.append("\n") ;
		
		bdr.append(hdr) ;
		
		for(Service serv : shel.getServices())
		{
			String desc = Annotations.getMethodDescription(serv.meth) ;
			
			bdr.append(ShellUtil.rpad(serv.serv, ConstShell.WIDTH_SERVICE)) ;
			bdr.append(ShellUtil.rpad(Util.oneString(serv.name, serv.labl)+"."+serv.meth.getName(), ConstShell.WIDTH_METHOD)) ;
			bdr.append(ShellUtil.rpad(desc, ConstShell.WIDTH_DESCRIPTION)) ;
			bdr.append("\n") ;
		}
		
		return bdr.toString() ;
	}
	
	final static public String usage(ShellBean shel, String name)
	throws Exception
	{
		Service serv = shel.getService(name) ;
		if(serv==null) return "Service not found: "+name ;
		
		Parameter[] ps = Annotations.getParameters(serv.meth) ;
		
		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append("usage: ") ;
		bdr.append(name) ;
		
		for(Parameter p : ps) bdr.append(" ").append(p.getName()) ;
		
		bdr.append("\n") ;
		bdr.append(Annotations.getMethodDescription(serv.meth)) ;
		bdr.append("\n") ;

		for(Parameter p : ps)
		{
			bdr.append("\t").append(p.getName()) ;
			bdr.append(": ").append(p.getDescription()) ;
			bdr.append(" (").append(p.getType().getSimpleName()).append(")") ;
		}
			
		return bdr.toString() ;
	}

	final static public String help(ShellBean shel, String name)
	throws Exception
	{
		Service serv = shel.getService(name) ;
		if(serv==null) return "Service not found: "+name ;
		
		BeanDefinition bdef = BeanRepository.instance().find(serv.name) ;
		if(bdef==null) return "Bean not found: "+serv.name ;
		
		return BeanDocumentation.asText(bdef) ;
	}
}