/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.tomcat.editor ;

import java.awt.*;

import javax.swing.*;

import org.azuki.bean.tomcat.server.imp.HttpConstants;
import org.azuki.bean.tomcat.shared.ConstTomcat;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.JDOMUtil;
import org.azuki.common.widget.JFormPanel;
import org.jdom.Element;

@SuppressWarnings("serial")
public class TomcatConnectorEditor extends AbstractEditor
{
	final private ListCellRenderer threadPriorityRenderer = new ThreadPriorityRenderer() ;
	
	JDOMComboField schmField ;
	JDOMNumberField portField ;
	
	JFormPanel connForm ;
	
	JDOMAttributeField kfilField ;
//	JDOMAttributeField kpasField ;
	JDOMPasswordField  kpasField ;
	
	JFormPanel secuForm ;
	
	JDOMNumberField maxtField ;
	JDOMNumberField minsField ;
	JDOMNumberField maxsField ;
	JDOMNumberUnlimitedField accpField ;
	JDOMComboField prioField ;
	
	JFormPanel poolForm ; 
	
	JPanel mainPanel ;
	
	/*
	 * AbstractEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element connLmen = JDOMUtil.getElement(para, ConstTomcat.CONNECTOR) ;
		Element secuLmen = JDOMUtil.getElement(connLmen, ConstTomcat.SECURITY) ;
		Element poolLmen = JDOMUtil.getElement(connLmen, ConstTomcat.POOL) ;
		
		// Connnector
		schmField = new JDOMComboField(HttpConstants.SCHEME_HTTP, HttpConstants.SCHEME_AJP, HttpConstants.SCHEME_HTTPS) ;
		schmField.setAttribute(JDOMUtil.getAttribute(connLmen, ConstTomcat.CONNECTOR_SCHM, ConstTomcat.defaultScheme)) ;
		
		portField = new JDOMNumberField(1, 65536, 1) ;
		portField.setAttribute(JDOMUtil.getAttribute(connLmen, ConstTomcat.CONNECTOR_PORT, ConstTomcat.defaultPort)) ;
		
		connForm = new JFormPanel() ;
		connForm.addField(schmField, "Scheme") ;
		connForm.addField(portField, "Port") ;
		
		// Security
		kfilField = new JDOMAttributeField(JDOMUtil.getAttribute(secuLmen, ConstTomcat.SECURITY_KFIL)) ;
//		kpasField = new JDOMAttributeField(JDOMUtil.getAttribute(secuLmen, ConstTomcat.SECURITY_KPAS)) ;
		kpasField = new JDOMPasswordField(JDOMUtil.getAttribute(secuLmen, ConstTomcat.SECURITY_KPAS)) ;
		
		secuForm = new JFormPanel() ;
		secuForm.addField(kfilField, "Keystore file") ;
		secuForm.addField(kpasField, "Keystore password") ;
		
		secuForm.setBorder(BorderFactory.createTitledBorder("Security")) ;
		
		// Pool
		maxtField = new JDOMNumberField(15, 999, 10) ;
		maxtField.setAttribute(JDOMUtil.getAttribute(poolLmen, ConstTomcat.POOL_MAXT, ConstTomcat.defaultMaxThreads)) ;
		
		minsField = new JDOMNumberField(2, 100, 1) ;
		minsField.setAttribute(JDOMUtil.getAttribute(poolLmen, ConstTomcat.POOL_MINS, ConstTomcat.defaultMinSpareThreads)) ;
		
		maxsField = new JDOMNumberField(25, 200, 5) ;
		maxsField.setAttribute(JDOMUtil.getAttribute(poolLmen, ConstTomcat.POOL_MAXS, ConstTomcat.defaultMaxSpareThreads)) ;
		
		accpField = new JDOMNumberUnlimitedField(1, 999, 10, -1) ;
		accpField.setAttribute(JDOMUtil.getAttribute(poolLmen, ConstTomcat.POOL_ACCP, ConstTomcat.defaultAcceptCount)) ;
		
		prioField = new JDOMComboField(ConstTomcat.MIN_PRIORITY, ConstTomcat.NORM_PRIORITY, ConstTomcat.MAX_PRIORITY) ;
		prioField.setRenderer(threadPriorityRenderer) ;
		prioField.setAttribute(JDOMUtil.getAttribute(poolLmen, ConstTomcat.POOL_PRIO, ConstTomcat.defaultThreadPriority)) ;
		
		poolForm = new JFormPanel() ;
		poolForm.addField(maxtField, "Max. threads") ;
		poolForm.addField(minsField, "Min. spare threads") ;
		poolForm.addField(maxsField, "Max. spare threads") ;
		poolForm.addField(accpField, "Accept count") ;
		poolForm.addField(prioField, "Priority") ;
		
		poolForm.setBorder(BorderFactory.createTitledBorder("Thread pool")) ;
		
		// Main
		mainPanel = new JPanel() ;
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS)) ;
		mainPanel.add(connForm) ;
		mainPanel.add(secuForm) ;
		mainPanel.add(poolForm) ;
		
		setDialogTitle("Tomcat connector configuration") ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(mainPanel, BorderLayout.NORTH) ;
	}
	
	/*
	 * Thread proprities renderer
	 */
	final class ThreadPriorityRenderer extends DefaultListCellRenderer
	{
		@Override
		final public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			if(ConstTomcat.MIN_PRIORITY.equals(value)) 			value = "MINIMUM" ;
			else if(ConstTomcat.NORM_PRIORITY.equals(value)) 	value = "NORMAL" ;
			else if(ConstTomcat.MAX_PRIORITY.equals(value)) 	value = "MAXIMUM" ;
			
			return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus) ;
		}
	}
}