/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.tomcat.editor ;

import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.logging.Level;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.UIResource;
import javax.swing.table.TableCellRenderer;

import org.azuki.bean.tomcat.shared.ConstTomcat;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.*;
import org.azuki.common.widget.JFormPanel;
import org.jdom.*;

@SuppressWarnings("serial")
final public class TomcatEngineEditor extends AbstractEditor
{
	final static private Map<String,String> servDefValues = new HashMap<String,String>() ;
	static
	{
		servDefValues.put(ConstTomcat.SERVLET_ENAB, Boolean.TRUE.toString()) ;
		servDefValues.put(ConstTomcat.SERVLET_NAME, "(undefined)") ;
	}
	
	final static private JDOMElementTableBinding servTableBinding
	= JDOMElementTableBinding.create(	ConstTomcat.SERVLET,
										ListUtil.asList(ConstTomcat.SERVLET_ENAB, ConstTomcat.SERVLET_NAME),
										ListUtil.asList("", "Name"),
										servDefValues) ;
	
	final static private JDOMElementTableBinding mimeTableBinding
	= JDOMElementTableBinding.create(	ConstTomcat.MIME,
										ListUtil.asList(ConstTomcat.MIME_EXTE, ConstTomcat.MIME_TYPE),
										ListUtil.asList("Extension", "Type")) ;
	
	final private ListSelectionListener defaultListSelectionAdapter = new ListSelectionAdapter() ;
	
	protected JDOMAttributeField bdirField ;
	protected JDOMAttributeField hrooField ;
	protected JDOMAttributeField wlcmField ;
	protected JDOMAttributeField jvmrField ;
	protected JFormPanel engnForm ;
	
	protected JDOMAttributeField ldirField ;
	protected JDOMComboField llvlField ;
	protected JFormPanel loggForm ;
	
	protected JDOMElementTablePanel servTablPanel ;
	protected JScrollPane servTablScroll ;
	protected JPanel servPanel ;
	
	protected JDOMElementTablePanel mimeTablePanel ;
	
	protected JTabbedPane tabbedPane ;
	
	/*
	 * AbstractEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element engnLmen = JDOMUtil.getElement(para, ConstTomcat.ENGINE) ;
		Element loggLmen = JDOMUtil.getElement(engnLmen, ConstTomcat.LOGGING) ;
		
		// engine
		bdirField = new JDOMAttributeField(JDOMUtil.getAttribute(engnLmen, ConstTomcat.ENGINE_BDIR)) ;
		hrooField = new JDOMAttributeField(JDOMUtil.getAttribute(engnLmen, ConstTomcat.ENGINE_HROO)) ;
		wlcmField = new JDOMAttributeField(JDOMUtil.getAttribute(engnLmen, ConstTomcat.ENGINE_WELC)) ;
		jvmrField = new JDOMAttributeField(JDOMUtil.getAttribute(engnLmen, ConstTomcat.ENGINE_JVMR)) ;
		
		engnForm = new JFormPanel() ;
		engnForm.addField(bdirField, "Base directory") ;
		engnForm.addField(hrooField, "Http root") ;
		engnForm.addField(wlcmField, "Welcome file") ;
		engnForm.addField(jvmrField, "JVM route") ;
		
		// logging
		ldirField = new JDOMAttributeField(JDOMUtil.getAttribute(loggLmen, ConstTomcat.LOGGING_DIR)) ;
		
		llvlField = new JDOMComboField(Level.OFF, Level.FINEST, Level.FINER, Level.FINE, Level.CONFIG, Level.INFO, Level.WARNING, Level.SEVERE, Level.ALL) ;
		llvlField.setAttribute(JDOMUtil.getAttribute(loggLmen, ConstTomcat.LOGGING_LEVL)) ;
		
		loggForm = new JFormPanel() ;
		loggForm.setBorder(BorderFactory.createTitledBorder("Logging")) ;
		loggForm.addField(ldirField, "Directory") ;
		loggForm.addField(llvlField, "Level") ;
		
		// servlets
		servTablPanel = new JDOMElementTablePanel(engnLmen, servTableBinding) ;
		servTablPanel.setPreferredSize(new Dimension(200, 100)) ;
		servTablPanel.getTable().getSelectionModel().addListSelectionListener(defaultListSelectionAdapter) ;
		servTablPanel.getTable().getColumnModel().getColumn(0).setMaxWidth(20) ;
		servTablPanel.getTable().getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(new JCheckBox())) ;
		servTablPanel.getTable().getColumnModel().getColumn(0).setCellRenderer(new StatusCellRenderer()) ;
		
		servTablScroll = new JScrollPane() ;
		servTablScroll.setBorder(BorderFactory.createEmptyBorder()) ;
		
		servPanel = new JPanel(new BorderLayout()) ;
		servPanel.add(servTablPanel, BorderLayout.WEST) ;
		servPanel.add(servTablScroll, BorderLayout.CENTER) ;
		
		// mime-types
		mimeTablePanel = new JDOMElementTablePanel(engnLmen, mimeTableBinding) ;
		
		// tabbed pane
		tabbedPane = new JTabbedPane(SwingConstants.BOTTOM) ;
		tabbedPane.addTab("Servlets", servPanel) ;
		tabbedPane.addTab("Mimetypes", mimeTablePanel) ;
		
		// top panel
		JPanel topPanel = new JPanel() ;
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.PAGE_AXIS)) ;
		topPanel.add(engnForm) ;
		topPanel.add(loggForm) ;
		
		// main
		setDialogTitle("Tomcat engine configuration") ;
		
		setLayout(new BorderLayout(5,5)) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(topPanel, BorderLayout.NORTH) ;
		add(tabbedPane, BorderLayout.CENTER) ;
		
		setPreferredSize(new Dimension(640, 480)) ;
	}
	
	final public class ListSelectionAdapter implements ListSelectionListener
	{
		final public void valueChanged(ListSelectionEvent e)
		{
			if(e.getValueIsAdjusting()) return ;
			
			ListSelectionModel selModel = servTablPanel.getTable().getSelectionModel() ;
			
			if(selModel.isSelectionEmpty())
			{	servTablScroll.setViewportView(null) ;
				return ; }
			
			if(servTablPanel.getTable().getSelectedRows().length!=1)
			{	servTablScroll.setViewportView(null) ;
				return ; }
			
			int selidx = servTablPanel.getTable().getSelectedRow() ;
			JDOMElementTableModel tablModel = servTablPanel.getTable().getModel() ;
			
			TomcatServletForm form = new TomcatServletForm(tablModel.getElementAt(selidx)) ;
			servTablScroll.setViewportView(form) ;
		}
	}
	
	final static class StatusCellEditor extends DefaultCellEditor
	{
		public StatusCellEditor()
		{
			super(new JCheckBox()) ;
			
			JCheckBox checkBox = (JCheckBox)getComponent() ;
			checkBox.setHorizontalAlignment(SwingConstants.CENTER) ;
		}
	}
	
	final static class StatusCellRenderer extends JCheckBox implements TableCellRenderer, UIResource
	{
		private static final Border noFocusBorder = new EmptyBorder(1, 1, 1, 1) ;
		
		public StatusCellRenderer()
		{
			super() ;
			
			setHorizontalAlignment(SwingConstants.CENTER) ;
			setBorderPainted(true) ;
		}
		
		//
		// TableCellRenderer
		//
		final public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			if(isSelected)
			{
				setForeground(table.getSelectionForeground()) ;
				super.setBackground(table.getSelectionBackground()) ;
			}
			else
			{
				setForeground(table.getForeground()) ;
				setBackground(table.getBackground()) ;
			}

			String strval = value==null ? Boolean.TRUE.toString() : value.toString() ;
			if(StringUtil.isEmpty(strval)) strval = Boolean.TRUE.toString() ;
			
			setSelected(Boolean.valueOf(strval).booleanValue()) ;

			if(hasFocus)	setBorder(UIManager.getBorder("Table.focusCellHighlightBorder")) ;
			else			setBorder(noFocusBorder) ;
			
			return this ;
		}
    }
	
	final static public class TomcatServletForm extends JPanel
	{
		final static private List<String> attrList = ListUtil.asList(ConstTomcat.SERVLET_PARA_NAME, ConstTomcat.SERVLET_PARA_VALU) ;
		final static private List<String> headList = ListUtil.asList("Key", "Value") ;
		
		final protected JDOMAttributeField clasField ;
		final protected JDOMAttributeField mapgField ;
		final protected JDOMNumberField loadField ;
		
		final protected JFormPanel form ;
		
		final protected JDOMElementTablePanel paraPanel ;		
		
		public TomcatServletForm(Element lmen)
		{
			Attribute clasAttr = JDOMUtil.getAttribute(lmen, ConstTomcat.SERVLET_CLAS) ;
			Attribute mapgAttr = JDOMUtil.getAttribute(lmen, ConstTomcat.SERVLET_MAPG) ;
			Attribute loadAttr = JDOMUtil.getAttribute(lmen, ConstTomcat.SERVLET_LOAD) ;
			
			// form
			clasField = new JDOMAttributeField(clasAttr) ;
			mapgField = new JDOMAttributeField(mapgAttr) ;
			
			loadField = new JDOMNumberField(0, 10, 1) ;
			loadField.setAttribute(loadAttr) ;
			
			form = new JFormPanel() ;
			form.addField(clasField, "Class") ;
			form.addField(mapgField, "Mapping") ;
			form.addField(loadField, "Load on startup") ;
			
			// parameters
			paraPanel = new JDOMElementTablePanel(lmen, ConstTomcat.SERVLET_PARA, attrList, headList) ;
			paraPanel.setBorder(BorderFactory.createTitledBorder(" Init parameters ")) ;
			paraPanel.setPreferredSize(new Dimension(200, 60)) ;
			
			// main
			setBorder(BorderFactory.createTitledBorder(" Configuration ")) ;
			setLayout(new BorderLayout()) ;
			add(form, BorderLayout.NORTH) ;
			add(paraPanel, BorderLayout.CENTER) ;
		}
	}
}