/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin
 */

package org.azuki.bean.tomcat.server.console ;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.tomcat.*;
import org.azuki.bean.tomcat.server.imp.TomcatServlet;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.util.*;
import org.azuki.framework.bean.*;

// Console view for Tomcat Bean

final public class TomcatEngineConsole
{
	final static public String[] servletTableHeaders = {"Name", "Class", "Mapping", "Load", "Parameters"} ;
	final static public String[] connectorTableHeaders = {"Scheme", "Port"} ;
	
	final static public void service(TomcatEngineBean teb, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		IMetadata meta = Bean.getMetadata(teb) ;
		
		// properties
		Map<String,Object> pmap = new HashMap<String,Object>() ;
		pmap.put("Base dir", teb.getBaseDirectory()) ;
		pmap.put("Web directory", teb.getHttpRoot()) ;
		pmap.put("Welcome file", teb.getWelcomeFile()) ;
		pmap.put("JVM route", teb.getJVMRoute()) ;
		
		Table ptab = HtmlFactory.propertyList("Properties", pmap) ;
		
		// logging
		Map<String,String> lmap = new HashMap<String,String>() ;
		lmap.put("Directory", teb.getLoggingDirectory()) ;
		lmap.put("Level", teb.getLoggingLevel()) ;
		
		Table ltab = HtmlFactory.propertyList("Logging", lmap) ;
		
		// mimetypes
		Table mtab = HtmlFactory.propertyList("Mimetypes", teb.getMimeMapping()) ;
		
		// servlets
		List<List> srvs = new ArrayList<List>() ;
		for(TomcatServlet ts : teb.getServlets())
			srvs.add(ListUtil.toList(ts.name, ts.clas, ts.mapg, ts.load, ts.para)) ;
		
		Table stab = HtmlFactory.sortedTable(req, srvs, servletTableHeaders) ;
		
		// connectors
		List<List> cons = new ArrayList<List>() ;
		for(TomcatConnectorBean tcb : teb.getConnectors())
			cons.add(ListUtil.toList(tcb.getScheme(), new Integer(tcb.getPort()))) ;
		
		Table ctab = HtmlFactory.sortedTable(req, cons, connectorTableHeaders) ;
		
		//--
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		bdr.print("<h2>").print(Util.oneString(meta.getName(), meta.getLabel())).print("</h2>") ;
		bdr.println("<br/>") ;
		bdr.println(ptab) ;
		bdr.println("<br/>") ;
		bdr.println(ltab) ;
		bdr.println("<br/>") ;
		bdr.println(mtab) ;
		bdr.println("<br/>") ;
		bdr.println("<h3>Servlets</h3>") ;
		bdr.println(stab) ;
		bdr.println("<h3>Connectors</h3>") ;
		bdr.println(ctab) ;
		
		HttpResponse.print(res, bdr) ;
	}
}