/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin
 */

package org.azuki.bean.tomcat.server.imp ;

import java.io.*;
import java.util.*;

import org.apache.catalina.*;
import org.apache.catalina.core.StandardWrapper;
import org.apache.catalina.startup.Embedded;
import org.azuki.bean.tomcat.shared.ConstTomcat;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.context.Log;
import org.jdom.*;

final public class PContext
{
	final static public Context initRootContext(Embedded embed, String appBase, String welcomefile, List<TomcatServlet> lts, Map<String,String> mimes)
	throws Exception
	{
		// Verifie l'existence du DOCBASE
		File docBase = new File(appBase) ;
		if(!docBase.exists())
			throw new Exception("Cannot configure context for application 'ROOT'.") ;
		
    	Context httpContext = embed.createContext("", docBase.getAbsolutePath()) ;
    	httpContext.setCrossContext(true) ;
    	httpContext.setOverride(true) ;
    	httpContext.setSwallowOutput(true) ;
    	httpContext.setReloadable(true) ;
		
		// Configured standard wrappers
		for(TomcatServlet ts : lts)
		{
			Wrapper wrap = newWrapper(ts.name, ts.mapg, ts.clas) ;
			wrap.setLoadOnStartup(ts.load.intValue()) ;
			
			for(Map.Entry<String,String> ety : ts.para.entrySet())
				wrap.addInitParameter(ety.getKey(), ety.getValue()) ;
			
			httpContext.addChild(wrap) ;
		}
		
		// Initialize Mime mappings
		initMimeMapping(httpContext, mimes) ;
		
		// Default HTML file: index.html
		httpContext.addWelcomeFile(welcomefile) ;
		
		return httpContext ;
	}

    final static private Wrapper newWrapper(String name, String mapping, String cla)
	{
		StandardWrapper wrap = new StandardWrapper() ;
		wrap.setName(name) ;
		wrap.setServletClass(cla) ;
		wrap.addMapping(mapping) ;
		
		return wrap ;
	}

    final static private void initMimeMapping(Context context, Map<String,String> mimes)
	{
		for(Map.Entry<String,String> ety : mimes.entrySet())
			context.addMimeMapping(ety.getKey(), ety.getValue()) ;
	}
    
    
    //
    // Default mime types
    //
    static private Map<String,String> mmap = null ;
    final static public Map<String,String> getDefaultMimeTypes()
    {
    	if(mmap!=null) return mmap ;
    	
    	mmap = new HashMap<String,String>() ;
    	
    	try
    	{
    		InputStream is = PContext.class.getResourceAsStream("mimes.xml") ;
        	
        	Document docu = JDOMUtil.read(is) ;
        	
        	Element root = docu.getRootElement() ;
        	
        	for(Element mime : JDOMUtil.listChildren(root, ConstTomcat.MIME))
        	{
        		String exte = mime.getChildText(ConstTomcat.MIME_EXTE) ;
        		String type = mime.getChildText(ConstTomcat.MIME_TYPE) ;
        		
        		mmap.put(exte, type) ;
        	}
    	}
    	catch(Exception ex)
    	{
    		Log.severe("Cannot load default mime types: "+ex.getMessage()) ;
    	}
    	
    	return mmap ;
    }
}