/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.action ;

import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.EventListenerList;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class ActionImpl extends AbstractAction
{	
	final protected EventListenerList listenerList = new EventListenerList() ;
	
	public ActionImpl(String action)
	{
		this(action, null) ;
	}
	
	public ActionImpl(String action, Icon icon)
	{
		this(action, action, icon) ;
	}
	
	public ActionImpl(String action, String label, Icon icon)
	{
		this(action, label, icon, null) ;
	}
	
	public ActionImpl(String action, String label, Icon icon, KeyStroke keyStroke)
	{
		this(action, label, icon, label, keyStroke) ;
	}
	
	public ActionImpl(String action, String label, Icon icon, String desc, KeyStroke keyStroke)
	{
		//INFO: putValue() checks if new value is null !
		
		putValue(Action.NAME, label) ;
		putValue(Action.ACTION_COMMAND_KEY, action) ;
		
		putValue(Action.SHORT_DESCRIPTION, desc) ;
		putValue(Action.SMALL_ICON, icon) ;
		
		putValue(Action.ACCELERATOR_KEY, keyStroke) ;
	}

	final public String getName()
	{
		return (String)getValue(Action.NAME);
	}
	
	final public String getActionCommand()
	{
		return (String)getValue(Action.ACTION_COMMAND_KEY);
	}
    
	final public String getShortDescription()
	{
		return (String)getValue(Action.SHORT_DESCRIPTION);
	}
	
	final public Icon getIcon()
	{
		return (Icon)getValue(Action.SMALL_ICON);		
	}
	
	final public KeyStroke getAcceleratorKey()
	{
		return (KeyStroke)getValue(Action.ACCELERATOR_KEY) ;
	}
	
	/* Should finish the implementation and add get/set methods for all the 
	 * javax.swing.Action keys:
	 * 	Action.MNEMONIC_KEY	//KeyEvent
	 *	Action.LONG_DESCRIPTION
	 * 	ActionConstants.LARGE_ICON
	 */
	
	final public void addActionListener(ActionListener al)
	{
		listenerList.add(ActionListener.class, al) ;
	}
	
	final public void removeActionListener(ActionListener al)
	{
		listenerList.remove(ActionListener.class, al) ;
	}
	
	/*
	 * AbstractAction
	 */
	public void actionPerformed(ActionEvent actionEvent)
	{
		ActionEvent delegate = new ActionEvent(actionEvent.getSource(), actionEvent.getID(), (String)getValue(Action.ACTION_COMMAND_KEY)) ;
		
		for(ActionListener al : listenerList.getListeners(ActionListener.class))
			al.actionPerformed(delegate) ;
	}
}