/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.beandoc ;

import java.util.List;

import org.azuki.common.framework.*;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.context.Log;
import org.jdom.*;
import org.jdom.transform.JDOMResult;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class BeanDocumentation
{
	static private Document htmlStylesheet ;
	static private Document textStylesheet ;
	static
	{
		try
		{
			htmlStylesheet = JDOMUtil.read(BeanDocumentation.class.getResourceAsStream("bean-doc-html.xsl")) ;
			textStylesheet = JDOMUtil.read(BeanDocumentation.class.getResourceAsStream("bean-doc-text.xsl")) ;
		}
		catch(Exception ex)
		{}
	}
	
	final static public String asHtml(String name)
	{
		BeanDefinition bdef = BeanRepository.instance().find(name) ;
		return bdef==null ? "Bean not found: "+name : asHtml(bdef) ;
	}
	
	final static public String asHtml(BeanDefinition bdef)
	{
		String html = null ;
		
		try
		{
			Document doc = BeanDocumentationParser.parse(bdef) ;
			
			html = htmlTransform(doc) ;
		}
		catch(Exception ex)
		{
			html = Log.getStack(ex) ;
		}
		
		System.out.println("HTML") ;
		System.out.println(html) ;
		
		return html ;
	}
	
	
	final static public String asText(String name)
	{
		BeanDefinition bdef = BeanRepository.instance().find(name) ;
		return bdef==null ? "Bean not found: "+name : asText(bdef) ;
	}
	
	final static public String asText(BeanDefinition bdef)
	{
		String text = null ;
		
		try
		{
			Document doc = BeanDocumentationParser.parse(bdef) ;
			
			text = textTransform(doc) ;
		}
		catch(Exception ex)
		{
			text = Log.getStack(ex) ;
		}
		
		return text ;
	}
	
	
	
	final static private String textTransform(Document xml)
	{
		String str = null ;
		
		try
		{
			JDOMResult jres = JDOMUtil.xslTransform(xml, textStylesheet) ;
			
			List list = jres.getResult() ;
			if(!list.isEmpty()) str = ((Text)list.get(0)).getText() ;
		}
		catch(Exception ex)
		{
			str = Log.getStack(ex) ;
		}
		
		return str ;
	}
	
	final static private String htmlTransform(Document xml)
	{
		String htmlString = null ;
		
		try
		{
			JDOMResult jres = JDOMUtil.xslTransform(xml, htmlStylesheet) ;
			Document doc = jres.getDocument() ;
			htmlString = JDOMUtil.xmlout.outputString(doc.getRootElement()) ;
		}
		catch(Exception ex)
		{
			htmlString = Log.getStack(ex) ;
		}
		
		return htmlString ;
	}
}