/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.html ;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Attribute implements Comparable<Attribute>
{
	final protected String name ;
	protected String valu ;
	
	public Attribute(String name)
	{
		this(name, null) ;
	}
	
	public Attribute(String name, String valu)
	{
		if(name==null) throw new IllegalArgumentException("Attribute name cannot be null") ;
		
		this.name = name ;
		this.valu = valu ;
	}
	
	final public String getName()
	{
		return name ;
	}
	
	final public String getValue()
	{
		return valu ;
	}
	
	final public void setValue(String valu)
	{
		this.valu = valu ;
	}
	
	@Override
	final public boolean equals(Object obj)
	{
		return obj instanceof Attribute ? compareTo((Attribute)obj)==0 : super.equals(obj) ;
	}
	
	@Override
	final public String toString()
	{
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(name).append("=\"") ;
		if(valu!=null) bdr.append(valu) ;
		bdr.append('"') ;
		return bdr.toString() ;
	}

	/*
	 * Comparable
	 */
	final public int compareTo(Attribute attr)
	{
		return name.compareTo(attr.name) ;
	}
}