/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.html.renderer ;

import org.azuki.common.util.StringUtil;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class TooltipCellRenderer implements TableCellRenderer
{
	final private int len ;
	final private char[] seps ;
	
	public TooltipCellRenderer()
	{
		this(-1) ;
	}
	
	public TooltipCellRenderer(int len)
	{
		this(len, null) ;
	}
	
	public TooltipCellRenderer(int len, char... seps)
	{
		this.len = len ;
		this.seps = seps ;
	}
	
	@Override
	public String render(Object t)
	{
		if(t==null) return null ;
		
		String full = String.valueOf(t) ;
		if(seps!=null) full = StringUtil.insertWhiteSpaceAfter(full, seps) ;
		
		String trim = String.valueOf(t) ;
		
		boolean istr = false ;
		if(trim.length()>len)
		{	istr = true ;
			trim = trim.substring(0, len) ; }
		
		StringBuilder bdr = new StringBuilder() ;
		bdr.append("<div") ;
		bdr.append(" width=\"100%\"") ;
		bdr.append(" onmouseover=\"aztooltip_show('aztooltip', this, 0, 20, '"+full+"');\"") ;
		bdr.append(" onmouseout=\"aztooltip_hide('aztooltip');\"") ;
		bdr.append(">") ;
		bdr.append(trim) ;
		if(istr) bdr.append("...") ;
		bdr.append("</div>") ;
		
		return bdr.toString() ;
	}
}