/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.http ;

import java.io.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.azuki.common.console.http.HttpHeader;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class HttpResponse
{
	// demande d'authentification
	final static public void setBasicAuth(HttpServletResponse res, String realm)
	{
		StringBuilder head = new StringBuilder()
		.append(HttpServletRequest.BASIC_AUTH)
		.append(" realm=\"").append(realm).append("\"") ;
		
		res.addHeader(HttpHeader.WWW_AUTHENTICATE, head.toString()) ;
		
		res.setStatus(HttpServletResponse.SC_UNAUTHORIZED) ;
	}
	
	// PROXIES: desactive la gestion du cache
	final static public void setNoCache(HttpServletResponse res)
	{
		res.addHeader(HttpHeader.PRAGMA, "no-cache") ;
		res.addHeader(HttpHeader.CACHE_CONTROL, "no-cache") ;
	}
	
	// ecriture du texte dans la reponse
	final static public void print(HttpServletResponse res, Object text)
	throws IOException
	{
		ServletOutputStream os = res.getOutputStream() ;
		os.print(String.valueOf(text)) ;
		os.flush() ;
	}
	final static public void print(HttpServletResponse res, Object text, String mime)
	throws IOException
	{
		res.setContentType(mime) ;
		print(res, text) ;
	}
	
	final static public void write(HttpServletResponse res, byte[] bytes)
	throws IOException
	{
		ServletOutputStream os = res.getOutputStream() ;
		os.write(bytes) ;
		os.flush() ;
	}
	
	final static public void write(HttpServletResponse res, byte[] bytes, String mime)
	throws IOException
	{
		res.setContentType(mime) ;
		write(res, bytes) ;
	}
}