/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.servlet ;

import java.util.*;
import java.util.logging.Logger;

import javax.servlet.http.*;

import org.azuki.common.console.http.HttpRequest;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class Dispatcher
{
	final static private String KEY = "clasid" ;
	final static private Logger logger = Logger.getLogger("org.azuki.common") ;
	
	final private Hashtable<String,DispatchServlet> hash = new Hashtable<String,DispatchServlet>() ;
	
	private DispatchServlet defaultServlet = null ;
	
	// Map a new (pseudo)servlet
	final public void put(DispatchServlet ds)
	{
		put(ds, false) ;
	}
	
	final public void put(DispatchServlet ds, boolean def)
	{
		if(hash.isEmpty() || def) defaultServlet = ds ;
		
		String dsid = id(ds) ;
		
		if(hash.containsKey(dsid))
		{	logger.fine("Already bind: "+dsid) ;
			return ; }
		
		hash.put(dsid, ds) ;
	}
		
	// Call the corresponding request servlet
	final public void call(HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String key = HttpRequest.get(req, KEY) ;
		
		DispatchServlet ds = key==null ? defaultServlet : hash.get(key) ;
		if(ds==null) ds = defaultServlet ;
		
		ds.service(svt, req, res) ;
	}
	
	/*
	 * STATIC
	 */
	final static public String id(DispatchServlet ds)
	{
		return id(ds.getClass()) ;
	}
	
	final static public String id(Class<? extends DispatchServlet> clas)
	{
		return clas.getName() ;
	}
	
	final static public Map<String,String> initMap(Class<? extends DispatchServlet> clas)
	{
		Map<String,String> map = new HashMap<String,String>() ;
		map.put(KEY, id(clas)) ;
		return map ;
	}
}