/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.List;

import javax.swing.ListModel;
import javax.swing.event.*;

import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JDOMElementListModel implements ListModel
{
	final protected EventListenerList listenerList = new EventListenerList() ;
	
	final private Element lmen ;
	final private String childName ;
	
	public JDOMElementListModel(Element lmen, String childName)
	{
		this.lmen = lmen ;
		this.childName = childName ;
	}
	
	final public Element getElement()
	{
		return (Element)lmen.clone() ;
	}
	
	final public String getChildName()
	{
		return childName ;
	}
	
	public int indexOf(Element e)
	{
		return getElements().indexOf(e) ;
	}
	
	@SuppressWarnings("unchecked")
	public List<Element> getElements()
	{
		return lmen.getChildren(childName) ;
	}
	
	public void addElement(int index, Element e)
	{
		getElements().add(index, e) ;
		fireIntervalAdded(this, index, index) ;
	}
	
	public void addElement(Element e)
	{
		getElements().add(e) ;
		
		int index = indexOf(e) ;
		fireIntervalAdded(this, index, index) ;
	}
	
	public Element removeElement(int index)
	{
		Element e = getElements().remove(index) ;
		fireIntervalRemoved(this, index, index) ;
		return e ;
	}
	
	public boolean removeElement(Element e)
	{
		return removeElement(indexOf(e))!=null ;
	}
	
	final protected void modelChanged()
	{
		fireContentsChanged(this, -1, -1) ;
	}
	
	final protected void fireContentsChanged(Object source, int index0, int index1)
    {
		ListDataEvent event = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, index0, index1) ;
		notifyChange(event) ;
    }
	
    final protected void fireIntervalAdded(Object source, int index0, int index1)
    {
    	ListDataEvent event = new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, index0, index1) ;
		notifyChange(event) ;
    }

    final protected void fireIntervalRemoved(Object source, int index0, int index1)
    {
    	ListDataEvent event = new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, index0, index1) ;
		notifyChange(event) ;
    }
    
    final private void notifyChange(ListDataEvent event)
    {
    	ListDataListener[] arr = listenerList.getListeners(ListDataListener.class) ;
		for(ListDataListener ldl : arr)
			ldl.contentsChanged(event) ;
    }
	
	/*
	 * ListModel
	 */
	final public int getSize()
	{
		return getElements().size() ;
	}
	
	final public Element getElementAt(int index)
	{
		return getElements().get(index) ;
	}

	final public void addListDataListener(ListDataListener l)
	{
		listenerList.add(ListDataListener.class, l) ;
	}
	
	final public void removeListDataListener(ListDataListener l)
	{
		listenerList.remove(ListDataListener.class, l) ;
	}
}