/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.*;

import org.azuki.common.editor.*;
import org.azuki.common.util.ListUtil;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JDOMElementTableBinding
{
	final static private String defaultChildName = TagCommon.ROW ;
	final static private List<String> defaultAttributeNames = ListUtil.asList(AttrCommon.NAME, AttrCommon.VALU) ;
	final static private List<String> defaultAttributeHeaders = ListUtil.asList("Name", "Value") ;
	
	final static private JDOMElementTableBinding defaulTableBinding = create(defaultChildName, defaultAttributeNames, defaultAttributeHeaders) ;
	
	final private String childName ;
	final private List<String> attrNames ;
	final private List<String> attrHeaders ;
	final private Map<String,String> defValues ;
	
	JDOMElementTableBinding(String childName, List<String> attrNames, List<String> attrHeaders, Map<String,String> defValues)
	{
		this.childName = childName ;
		
		this.attrNames = attrNames ;
		this.attrHeaders = attrHeaders ;
		
		this.defValues = defValues ;
	}
	
	final public String getChildName()
	{
		return childName ;
	}
	
	final public List<String> getAttributeNames()
	{
		return attrNames ;
	}
	
	final public List<String> getAttributeHeaders()
	{
		return attrHeaders ;
	}
	
	final public Map<String,String> getDefaultValues()
	{
		return defValues ;
	}

	final static public JDOMElementTableBinding getDefault()
	{
		return defaulTableBinding ;
	}
	
	final static public JDOMElementTableBinding create(String childName, List<String> attrNames)
	{
		return new JDOMElementTableBinding(childName, attrNames, attrNames, null) ;
	}
	
	final static public JDOMElementTableBinding create(String childName, List<String> attrNames, List<String> attrHeaders)
	{
		return new JDOMElementTableBinding(childName, attrNames, attrHeaders, null) ;
	}
	
	final static public JDOMElementTableBinding create(String childName, List<String> attrNames, List<String> attrHeaders, Map<String,String> defValues)
	{
		return new JDOMElementTableBinding(childName, attrNames, attrHeaders, defValues) ;
	}
}