/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.*;

import javax.swing.tree.*;

import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JDOMElementTreeNode implements MutableTreeNode
{
	protected JDOMElementTreeNode parent ;
	protected Element lmen ;
	
	final private List<MutableTreeNode> children ;
	
	@SuppressWarnings("unchecked")
	public JDOMElementTreeNode(JDOMElementTreeNode parent, Element lmen)
	{
		this.parent = parent ;
		this.lmen = lmen ;
		
		this.children = new ArrayList<MutableTreeNode>() ;
		
		List<Element> elementList = lmen.getChildren() ;
		for(Element e : elementList)
			children.add(new JDOMElementTreeNode(this, e)) ;
	}
	
	final public Element getElement()
	{
		return lmen ;
	}
	
	@Override
	final public String toString()
	{
		return lmen.getName() ;
	}
	
	/*
	 * TreeNode
	 */
	final public TreeNode getParent()
	{
		return parent ;
	}
	
	final public int getChildCount()
	{
		return children.size() ;
	}
	
	final public boolean getAllowsChildren()
	{
		return true ;
	}

	final public boolean isLeaf()
	{
		return children.isEmpty() ;
	}
	
	final public int getIndex(TreeNode node)
	{
		return children.indexOf(node) ;
	}
	
	final public TreeNode getChildAt(int childIndex)
	{
		return children.get(childIndex) ;
	}
	
	final public Enumeration children()
	{
		return Collections.enumeration(children) ;
	}
	
	/*
	 * MutableTreeNode
	 */
	final public void insert(MutableTreeNode child, int index)
	{
		MutableTreeNode oldParent = (MutableTreeNode)child.getParent();

	    if (oldParent!=null) oldParent.remove(child);
	    child.setParent(this);
	   
	    lmen.addContent(index, ((JDOMElementTreeNode)child).lmen) ; // peut-etre BOF ..
		children.add(index, child) ;
	}

	final public void remove(int index)
	{		
		JDOMElementTreeNode treeNode = (JDOMElementTreeNode)children.remove(index) ;
		lmen.removeContent(treeNode.lmen) ; 
	}

	final public void remove(MutableTreeNode node)
	{
		children.remove(node) ;
		lmen.removeContent(((JDOMElementTreeNode)node).lmen) ;
	}

	final public void removeFromParent()
	{
		parent.remove(this) ;
	}

	final public void setParent(MutableTreeNode newParent)
	{
		this.parent = (JDOMElementTreeNode)newParent ;
	}

	final public void setUserObject(Object object)
	{
		lmen.setName(String.valueOf(object)) ;
	}
}