/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMIntegerField extends JTextField
{
	final protected DocumentListener defaultDocumentAdapter = new DocumentAdapter() ;
	final protected KeyListener defaultKeyAdapter = new KeyAdapter() ;
	
	protected Attribute attr ;
	
	public JDOMIntegerField()
	{
		setHorizontalAlignment(SwingConstants.RIGHT) ;
		
		addKeyListener(defaultKeyAdapter) ;
	}
	
	public JDOMIntegerField(Attribute attr)
	{
		this() ;
		
		if(attr==null) throw new IllegalArgumentException("Attribute cannot be null") ;
		
		setAttribute(attr) ;
	}
	
	final public void setAttribute(Attribute attr)
	{
		this.attr = attr ;
		
		getDocument().removeDocumentListener(defaultDocumentAdapter) ;
		setText(attr.getValue()) ;
		getDocument().addDocumentListener(defaultDocumentAdapter) ;
	}
	
	/*
	 * Default DocumentListener
	 */
	final class DocumentAdapter implements DocumentListener
	{
		public void changedUpdate(DocumentEvent e)
		{
			attr.setValue(getText()) ;
		}
		
		public void insertUpdate(DocumentEvent e)
		{
			attr.setValue(getText()) ;
		}
		
		public void removeUpdate(DocumentEvent e)
		{
			attr.setValue(getText()) ;
		}
	}
	
	/*
	 * Default KeyListener
	 */
	final class KeyAdapter implements KeyListener
	{
		final public void keyPressed(KeyEvent e)
		{}

		final public void keyReleased(KeyEvent e)
		{}

		final public void keyTyped(KeyEvent e)
		{
			if(Character.isLetter(e.getKeyChar()))
				e.consume() ;
		}
	}
}
