/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom;

import java.awt.event.*;

import javax.swing.*;

import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMNumberUnlimitedField extends JComponent 
{
	final static public String defaultUnlimitedLabel = "unlimited" ;
	
	final protected ItemListener defaultItemAdapter = new ItemAdapter() ;
	
	final protected int ulm ;
	
	final protected JDOMNumberField spin ;
	final protected JCheckBox cbox ;
	
	// CONSTRUCTEUR
	public JDOMNumberUnlimitedField(int min, int max, int step, int ulm)
	{
		this(min, max, step, ulm, defaultUnlimitedLabel) ;
	}
	
	public JDOMNumberUnlimitedField(int min, int max, int step, int ulm, String ulbl)
	{
		this(min, max, step, min, ulm, ulbl) ;
	}
	
	public JDOMNumberUnlimitedField(int min, int max, int step, int def, int ulm)
	{
		this(min, max, step, def, ulm, defaultUnlimitedLabel) ;
	}
	
	public JDOMNumberUnlimitedField(int min, int max, int step, int def, int ulm, String ulbl)
	{
		this.ulm = ulm ;
		
		spin = new JDOMNumberField(min, max, step, def) ; 
		cbox = new JCheckBox(ulbl) ;

		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS)) ;
		add(spin) ;
		add(cbox) ; 
	}
	
	// formattage du spinner
	final public void setNumberFormat(String decimalFormat)
	{
		spin.setNumberFormat(decimalFormat) ;
	}
	
	// ajoute un ItemListener
	final public void addItemListener(ItemListener il)
	{
		cbox.addItemListener(il) ;
	}
	
	// supprime un ItemListener
	final public void removeItemListener(ItemListener il)
	{
		cbox.removeItemListener(il) ;
	}
	
	final public void setAttribute(Attribute attr)
	{
		cbox.removeItemListener(defaultItemAdapter) ;
		
		spin.setAttribute(attr) ;

		boolean unl = attr.getValue().equals(String.valueOf(ulm)) ;
		cbox.setSelected(unl) ;
		spin.setEnabled(!unl) ;

		cbox.addItemListener(defaultItemAdapter) ;
	}

	/*
	 * JComponent
	 */
	@Override
	final public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled) ;
		cbox.setEnabled(enabled) ;
		spin.setEnabled(enabled) ;
	}
	
	/*
	 * Default ItemListener
	 */
	final class ItemAdapter implements ItemListener
	{
		final public void itemStateChanged(ItemEvent e)
		{
			boolean sel = e.getStateChange()==ItemEvent.SELECTED ; 
			Integer val = new Integer(sel ? ulm : spin.getDefaultValue()) ;
			
			spin.setValue(val) ;
			spin.setEnabled(!sel) ;
		}	
	}		
}