/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import javax.swing.JPasswordField;
import javax.swing.event.*;

import org.azuki.common.util.Crypto;
import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMPasswordField extends JPasswordField implements DocumentListener
{
	private Attribute attr ;
	
	public JDOMPasswordField(Attribute attr)
	{
		setAttribute(attr) ;
	}
	
	final public void setAttribute(Attribute attr)
	{
		this.attr = attr ;
		
		getDocument().removeDocumentListener(this) ;
		if (attr.getValue()!=null)
			setText(attr.getValue()) ;
		else
			setText("");
		getDocument().addDocumentListener(this) ;
	}
	
	final public Attribute getAttribute()
	{
		return attr ;
	}
	
	final private String getEncryptedText()
	{
		String str = String.valueOf(getPassword()) ;
		try
		{ str = Crypto.cypher(str) ; }
		catch(Exception ex)
		{ ex.printStackTrace() ; }
		
		return str ;
	}
	
	/*
	 * DocumentListener
	 */
	final public void changedUpdate(DocumentEvent e)
	{
		attr.setValue(getEncryptedText()) ;
	}

	final public void insertUpdate(DocumentEvent e)
	{
		attr.setValue(getEncryptedText()) ;
	}

	final public void removeUpdate(DocumentEvent e)
	{
		attr.setValue(getEncryptedText()) ;
	}	
}