/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.pointcut.imp ;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.lang.reflect.Field;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.TableCellRenderer;

import org.azuki.common.consts.Attr;
import org.azuki.common.reflect.Annotations;
import org.azuki.common.widget.JItemChooser;
import org.azuki.common.widget.jdom.JDOMElementTableCellRenderer;
import org.azuki.common.widget.model.ItemChooserTableModel;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class FieldChooser extends JItemChooser<Element> implements ActionListener
{
	final static private String UF = "Undefined field" ;
	
	final protected TableCellRenderer defaultCellRenderer = new JDOMElementTableCellRenderer(Attr.NAME, "(undefined)") ;
	final protected ListSelectionListener defaultSelectionAdapter = new ListSelectionAdapter() ;
	final protected TableModelListener itemSelectionAdapter = new ItemSelectionAdapter() ;
	
	final protected Field[] fields ;
	
	protected JTextArea fieldDesc ;
	protected JScrollPane fieldDescScroll ;
	protected JLabel fieldDescLabel ;
	protected JPanel fieldDescPane ;
	
	public FieldChooser(Collection<Element> lmens, Field[] fields)
	{
		super(new ItemChooserTableModel<Element>("Field", new ArrayList<Element>(lmens), Boolean.FALSE)) ;
		
		this.fields = fields ;
		
		fieldDesc = new JTextArea() ;
		fieldDesc.setRows(4) ;
		fieldDesc.setEditable(false) ;
		fieldDesc.setLineWrap(true) ;
		
		fieldDescScroll = new JScrollPane(fieldDesc) ;
		
		fieldDescLabel = new JLabel("Description :") ;
		fieldDescLabel.setLabelFor(fieldDesc) ;
		
		fieldDescPane = new JPanel(new BorderLayout()) ;
		fieldDescPane.add(fieldDescLabel, BorderLayout.NORTH) ;
		fieldDescPane.add(fieldDescScroll, BorderLayout.SOUTH) ;
		
		setFieldCellRenderer(defaultCellRenderer) ;
		
		addItemSelectionListener(itemSelectionAdapter) ;
		addListSelectionListener(defaultSelectionAdapter) ;
		
		add(fieldDescPane, BorderLayout.SOUTH) ; //TODO: faire autrement
	}
	
	final public void setFieldCellRenderer(TableCellRenderer cellRenderer)
	{
		table.getColumnModel().getColumn(1).setCellRenderer(cellRenderer) ;
	}
	
	final protected Field getField(String name)
	{
		if(fields==null) return null ;
		
		for(Field field : fields)
			if(field.getName().equals(name)) return field ;
		
		return null ;
	}
	
	/*
	 * Default table selection adapter 
	 */
	final public class ListSelectionAdapter implements ListSelectionListener
	{
		/*
		 * ListSelectionListener
		 */
		final public void valueChanged(ListSelectionEvent e)
		{
			if(e.getValueIsAdjusting()) return ;
			
			int idx = table.getSelectedRow() ;
			
			if(idx==-1)
			{	fieldDesc.setText("") ;
				return ; }
			
			String fieldName = ((Element)table.getValueAt(idx, 1)).getAttributeValue(Attr.NAME) ;
			Field field = getField(fieldName) ;
			
			fieldDesc.setText(field==null ? UF : Annotations.getDependencyInjectionDescription(field)) ;
			fieldDesc.setCaretPosition(0) ;
		}
	}
	
	/*
	 * Item selection adapter
	 */
	final public class ItemSelectionAdapter implements TableModelListener
	{
		final public void tableChanged(TableModelEvent e)
		{
			if(dialogActions!=null)
				dialogActions.getAction(OK_ACTION).setEnabled(tableModel.isSelectionValid()) ;
		}
	}
}