/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.pointcut.imp.renderer ;

import java.awt.*;
import java.lang.reflect.Field;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import org.azuki.common.consts.Attr;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
final public class FieldDeprecatedCellRenderer extends DefaultTableCellRenderer
{
	final private Field[] fields ;
	
	public FieldDeprecatedCellRenderer(Field[] fields)
	{
		this.fields = fields ;
	}
	
	final public Field getField(String fieldName)
	{
		for(Field field : fields)
			if(field.getName().equals(fieldName)) return field ;
		
		return null ;
	}
	
	final private int count(String fieldName)
	{
		int count = 0 ;
		
		for(Field field : fields)
		{	if(field.getName().equals(fieldName))
				count++ ; }
		
		return count ;
	}
	
	/*
	 * DefaultTableCellRenderer
	 */
	@Override
	final public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		String fieldName = value instanceof Element ? ((Element)value).getAttributeValue(Attr.NAME) : value.toString() ;
		
		Component cr = super.getTableCellRendererComponent(table, fieldName, isSelected, hasFocus, row, column) ;
		
		int count = count(fieldName) ;
		if(count==1)		cr.setForeground(Color.black) ;	// OK
		else if(count==0)	cr.setForeground(Color.red) ;	// does not exists
		else				cr.setForeground(Color.orange) ;// more than one
		
		return cr ;
	}
}