/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.pointcut.imp.renderer ;

import java.awt.*;
import java.lang.reflect.Method;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import org.azuki.common.consts.Attr;
import org.azuki.common.reflect.Signature;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
final public class MethodDeprecatedCellRenderer extends DefaultTableCellRenderer
{
	final private Method[] methods ;
	
	public MethodDeprecatedCellRenderer(Method[] methods)
	{
		this.methods = methods ;
	}
	
	final public Method getMethod(String methodName)
	{
		for(Method method : methods)
			if(method.getName().equals(methodName)) return method ;
		
		return null ;
	}
	
	final private int count(String methodName)
	{
		int count = 0 ;
		
		for(Method method : methods)
		{	if(method.getName().equals(methodName))
				count++ ; }
		
		return count ;
	}
	
	/*
	 * DefaultTableCellRenderer
	 */
	@Override
	final public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		String methodName = value instanceof Element ? ((Element)value).getAttributeValue(Attr.NAME) : value.toString() ;
		
		Method method = getMethod(methodName) ;
		
		Signature stur = null ;
		try
		{ stur = method==null ? new Signature(methodName) : new Signature(method) ; }
		catch(Exception ex){}
		
		Component cr = super.getTableCellRendererComponent(table, stur, isSelected, hasFocus, row, column) ;
		
		int count = count(methodName) ;
		if(count==1)		cr.setForeground(Color.black) ;	// OK
		else if(count==0)	cr.setForeground(Color.red) ;	// does not exists
		else				cr.setForeground(Color.orange) ;// more than one
		
		return cr ;
	}
}