/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.framework ;

import org.azuki.common.reflect.Contract;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.aop.IContract;
import org.azuki.framework.context.Log;

//This attributes are immutable, does not depends on the 
//bean instance definition.

//Static metadata

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class AspectDefinition
{
	final private BeanDefinition bdef ;
	
	final private String ctrt ;
	final private String edit ;
	
	private IContract cobj = null ;
	
	protected AspectDefinition(BeanDefinition bdef, String ctrt, String edit) 
	{
		this.bdef = bdef ;
		this.ctrt = ctrt ;
		this.edit = edit ;		
	}
	
	final public BeanDefinition getBeanDefinition()	{ return bdef; }
	
	final public String getContract()	{ return ctrt; }
	
	final public String getEditor()		{ return edit; }
	
	final public IContract getContractObject()
	{
		if(cobj!=null) return cobj ;
		
		if(StringUtil.isEmpty(ctrt))
		{	cobj = Contract.getDefault() ;
			return cobj ; }
		
		ClassLoader cl = bdef.getBeanClass().getClassLoader() ;
		Class clas = null ;
		
		try
		{ clas = cl.loadClass(ctrt) ; }
		catch(Exception ex)
		{ Log.severe(ex) ; return null ; }
		
		if(!IContract.class.isAssignableFrom(clas))
		{	Log.severe("Contract class is not instance of IContract: "+ctrt) ;
			return null ; }
		
		try
		{ cobj = (IContract)clas.newInstance() ; }
		catch(Exception ex)
		{ Log.severe(ex) ; return null ; }
		
		return cobj ;
	}
}