/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.reflect ;

import java.lang.annotation.Annotation;
import java.lang.reflect.*;
import java.util.ArrayList;

import org.azuki.framework.annotation.*;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Annotations
{
	final static public String NO_DESC = "No description" ;
	
	final static public String getMethodDescription(Method meth)
	{
		MethodInfo mnfo = meth.getAnnotation(MethodInfo.class) ;
		return mnfo==null ? NO_DESC : mnfo.desc() ;
		//TODO: synchro with MethodInfo.desc default value ?
	}
	
	final static public String getMethodReturnType(Method meth)
	{
		MethodInfo mnfo = meth.getAnnotation(MethodInfo.class) ;
		return mnfo==null ? NO_DESC : mnfo.retu() ;
		//TODO: synchro with MethodInfo.desc default value ?
	}
	
	final static public Parameter[] getParameters(Method meth)
	{
		Class<?>[] pcls = meth.getParameterTypes() ;
		Annotation[][] pans = meth.getParameterAnnotations() ;
		
		ArrayList<Parameter> list = new ArrayList<Parameter>() ;
		
		int i=0 ;
		for(Class<?> clas : pcls)
		{
			ArgInfo apar = i<pans.length ? findAnnotation(pans[i], ArgInfo.class) : null ;
			
			String paraName = apar==null ? "arg"+i : apar.name() ;
			String paraDesc = apar==null ? NO_DESC : apar.desc() ;
			
			Parameter para = new Parameter(i, clas, paraName, paraDesc) ;
			
			list.add(para) ;
			
			++i ;
		}
		
		return list.toArray(new Parameter[list.size()]) ;
	}
	
	final static public ArgInfo getParameter(Method meth, int pos)
	{
		Annotation[][] pans = meth.getParameterAnnotations() ;
		if(pos>=pans.length) return null ; 
		
		Annotation[] anns = pans[pos] ;
		for(Annotation ann : anns)
			if(ann instanceof ArgInfo) return (ArgInfo)ann ;
		
		return null ; // not found !
	}
	
	final static private <A extends Annotation> A findAnnotation(Annotation[] annos, Class<A> acls)
	{
		for(Annotation anno : annos)
			if(acls.isInstance(anno)) return acls.cast(anno) ;
		
		return null ;
	}
	
	final static public String getDependencyInjectionDescription(Field field)
	{
		DependencyInjectionInfo fnfo = field.getAnnotation(DependencyInjectionInfo.class) ;
		return fnfo==null ? NO_DESC : fnfo.desc() ;
		//TODO: synchro with FieldInfo.value default value ?
	}
	
	final static public <T extends AnnotatedElement> boolean isDependencyInjectionMandatory(T t)
	{
		DependencyInjectionInfo di = t.getAnnotation(DependencyInjectionInfo.class) ;
		return di==null ? false : di.mandatory() ;
	}
}