/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.reflect ;

import java.lang.reflect.*;
import java.util.*;

import org.azuki.framework.annotation.Export;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class BeanReflect
{
	final static public boolean isCandidate(Signature base, Signature stur)
	{
		String[] btyps = base.getParametersType() ; 
		if(btyps.length==1 && btyps[0].equals(Signature.VARARGS))
			return true ;
		
		String[] styps = stur.getParametersType() ;
		if(btyps.length!=styps.length) return false ;
		
		for(int i=0 ; i<btyps.length ; i++)
		{
			String type = btyps[i] ;
			if(type.equals(Object.class.getSimpleName())) continue ;
			if(type.equals(styps[i])) continue ;
			
			return false ;
		}
		
		return true ; 
	}
	
	final static public List<Signature> getMethodCandidates(Class<?> clas, Signature stur)
	{
		Collection<Signature> sturs = ReflectUtil.getSignatures(clas.getMethods()) ;
		
		Iterator<Signature> it = sturs.iterator() ;
		while(it.hasNext())
			if(!isCandidate(stur, it.next())) it.remove() ;
		
		return new ArrayList<Signature>(sturs) ;
	}
	
	// retourne TRUE si possede l'annotation "MethodExport"
	final static public boolean isMethodExported(Method method)
	{
		return method.isAnnotationPresent(Export.class) ; 
	}
	
	// retourne la liste de toutes les methodes publique exportee
	final static public Collection<Method> getMethodsExported(Class beanClass)
	{
		Collection<Method> methods = new HashSet<Method>() ;
		
		for(Method method : beanClass.getMethods())
			if(isMethodExported(method)) methods.add(method) ;
		
		return methods ;
	}
	
	final static public String getMethodProxySignature(Field proxy)
	{
		Type proxyType = proxy.getGenericType() ;
		
		Class[] paramTypes = null ;
		
		DependencyInjectionInfo dnfo = proxy.getAnnotation(DependencyInjectionInfo.class) ;
		if(dnfo!=null) paramTypes = dnfo.param() ;
		
		StringBuilder sb = new StringBuilder()
		.append(proxy.getName())
		.append('(').append(paramTypes==null ? "?" : ReflectUtil.toString(paramTypes)).append("):")
		.append(ParameterizedType.class.isInstance(proxyType) 
					? ReflectUtil.toString(((ParameterizedType)proxyType).getActualTypeArguments())
					: "?") ;
		
		return sb.toString() ;
	}
	
	// retourne la liste de MethodProxy
	final static public Field[] getMethodProxyFields(Class clas)
	{
		return ReflectUtil.getFields(clas, IMethodProxy.class) ;
	}
}