/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.reflect;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Parameter
{
	final private int pos ;
	final private Class<?> clas ;
	final private String name ;
	final private String desc ;
	
	public Parameter(int pos, Class<?> clas)
	{
		this(pos, clas, null, null) ;
	}
	
	public Parameter(int pos, Class<?> clas, String name, String desc)
	{
		this.pos = pos ;
		this.clas = clas ;
		this.name = name==null ? "arg"+pos : name ;
		this.desc = desc==null ? Annotations.NO_DESC : desc ;
	}
	
	final public int getPosition()
	{
		return pos ;
	}
	
	final public Class<?> getType()
	{
		return clas ;
	}
	
	final public String getName()
	{
		return name ;
	}
	
	final public String getDescription()
	{
		return desc ;
	}
	
	/*
	 * Object
	 */
	@Override
	final public String toString()
	{
		StringBuilder bdr = new StringBuilder()
		.append(clas.getSimpleName())
		.append(' ').append(name)
		.append(" : ").append(desc) ;
		return bdr.toString() ;
	}
}