/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util;

import java.io.* ;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class FileTool
{
	final static String defaultBackupExtension = "bak" ;
		
	final static public String canonical(String fname)
	{
		File file = new File(fname) ;
		try						{ return file.getCanonicalPath() ; }
		catch(IOException ioe)	{ return file.getAbsolutePath() ; }
	}
	
	final static public byte[] readFile(File f)
	throws IOException
	{	
		byte[] bytes = new byte[(int)f.length()] ;

		DataInputStream is = new DataInputStream(new FileInputStream(f)) ;
		is.readFully(bytes, 0, bytes.length) ;
		is.close() ;

		return bytes;
	}

	final static public void writeFile(String filename, byte[] content)
	throws IOException
	{
		File file = new File(filename) ;
		file.createNewFile() ;
		
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file)) ;
		bos.write(content) ;
		bos.flush() ;
		bos.close() ;
	}   

	final static public void delete(File fdir)
	throws IOException
	{
		if(!fdir.isDirectory())
		{	fdir.delete() ;
			return ; }

		File[] fs = fdir.listFiles() ;
		if(fs==null) return ;

		for(File f : fs) delete(f) ;
		
		fdir.delete() ;
	}

	final static public boolean backup(File file)
	{
		return backup(file, null) ;
	}
	
	final static public boolean backup(File file, String ext)
	{
		if(!file.exists()) return false ;
		
		if(ext==null)	ext = defaultBackupExtension ; 
		else			ext = ext.trim() ;
		
		File fbak = new File(file.getPath()+(ext.startsWith(".") ? ext : "."+ext)) ;
		if(fbak.exists()) fbak.delete() ;
		
		file.renameTo(fbak) ;
		
		return true ;
	}
}