/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util;

import java.io.UnsupportedEncodingException;
import java.util.Arrays;

/**
 * @author Mathieu Jucker
 */

public class ShellUtil
{
	final public static byte[] rpadBytes(String src, int size, String charSet)
	throws UnsupportedEncodingException
	{
		return rpad(src, size, ' ').getBytes(charSet) ;	
	}
	
	final public static String rpad(String src, int size)
	{
		return rpad(src, size, ' ') ;	
	}
	
	final public static String rpad(String src, int size, char c)
	{
		char[] chars = new char[size] ;
		Arrays.fill(chars, c) ;
		
		if(src==null) return new String(chars) ;
		
//		src.trim(); //FIXME: modify source string ???
		
		char[] srcChars = src.toCharArray() ;
		int len = Math.min(srcChars.length, chars.length) ;
		
		System.arraycopy(srcChars, 0, chars, 0, len) ;
		
		return new String(chars) ;	
	}
	
	final public static byte[] lpadBytes(String src, int size, String charSet)
	throws UnsupportedEncodingException
	{
		return lpad(src, size, ' ').getBytes(charSet) ;	
	}
	
	final public static String lpad(String src, int size)
	{
		return lpad(src, size, ' ') ;
	}
	
	final public static String lpad(String src, int size, char c)
	{
		char[] chars = new char[size] ;
		Arrays.fill(chars, c) ;
		
		if(src==null) return new String(chars) ;
		
//		src.trim(); //FIXME: modify source string ???
		
		char[] srcChars = src.toCharArray() ;
		
		int len = Math.min(srcChars.length, chars.length) ;
		
		int dstIndex = size-srcChars.length ;
		if(dstIndex<0) dstIndex = 0 ;
		
		System.arraycopy(srcChars, 0, chars, dstIndex, len) ;
		
		return new String(chars) ;
	}
}
