/*
 *   Created the 2 October 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.util.*;

import org.azuki.common.consts.Const;
import org.azuki.framework.context.Log;

/**
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */
public class Topology
{
	final static private String separator = "," ;
	
	final static private String TOPOLOGY_PROPERTY_FILE = "azuki-topology.properties" ;
	
	final static private String POINTCUT_PRIORITY    = "pointcut.priority" ;
	final static private String UNCHECKED_DEPENDENCY = "unchecked.dependency" ;
	
	static private boolean inited = false ;

	final static private List<String> ptys = new ArrayList<String>() ;
	final static private Map<String,List<String>> ruls = new HashMap<String,List<String>>() ;
	
	static { init() ; }
	
	final static private void init()
	{
		if(inited) return ;
		
		inited = true ;
		
		Log.info("==================") ;
		Log.info("= TOPOLOGY RULES =") ;
		Log.info("==================") ;
		
		Properties props = new Properties() ;
		try
		{ props.load(Topology.class.getClassLoader().getResourceAsStream(TOPOLOGY_PROPERTY_FILE)) ; }
		catch(Exception ex)
		{ Log.severe("Cannot load topology: "+ex.getMessage()) ; return ; }
		
		// priorities
		String spty = props.getProperty(POINTCUT_PRIORITY) ;
		List<String> list = StringUtil.asList(spty, separator);
		
		ptys.addAll(list) ;
		
		Log.info("\nPointcuts priorities : "+ptys+"\n") ;
		
		// rules 
		for(Map.Entry<?,?> ety : props.entrySet())
		{
			String key = (String)ety.getKey() ;
			String val = (String)ety.getValue() ;
			
			if(key.startsWith(UNCHECKED_DEPENDENCY)) uncheck(val) ;
		}
		Log.info("") ;
	}
	
	final static private void uncheck(String rule)
	{
		int indx = rule.indexOf(separator) ;
		if(indx==-1) return ;
		
		String pare = rule.substring(0, indx).trim() ;
		String chil = rule.substring(indx+1).trim() ;
		
		Log.info(".unchecked dependency for aspect ("+pare+") applied on ("+chil+")") ;
		
		List<String> list = ruls.get(pare) ;
		if(list==null)
		{	list = new ArrayList<String>() ;
			ruls.put(pare, list) ; }
		
		list.add(chil) ;
	}
	
	final static public boolean isUnchecked(String pare, String chil)
	{
		List<String> list = ruls.get(pare) ;
		return list==null ? false : list.contains(chil) ;
	}
	
	final static public int getPriority(String name)
	{
		int idx = ptys.indexOf(name) ;
		return idx==-1 ? Const.I_DEF_RUN_LEVEL : idx  ;
	}
}