/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.util.HashMap;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JFormPanel extends JDialogPanel
{
	final protected HashMap<JComponent,String> fields = new HashMap<JComponent,String>() ; // Map<field,label>
	
	public JFormPanel()
	{
		super(new GridBagLayout()) ;
		
		this.dispositionConstraints = BorderLayout.NORTH ;
	}
	
	public void addField(JComponent component)
	{
		addField(component,null) ;
	}
	
	public void addField(JComponent component, String label)
	{
		GridBagConstraints c = new GridBagConstraints() ;
		
		c.gridy = fields.size() ;
		c.anchor = GridBagConstraints.LINE_START ;
		c.insets = new Insets(2, 2, 2, 2) ;
		
		// label
		c.gridx = 0 ;
		JLabel jlabel = new JLabel(label) ;
		add(jlabel, c) ;
		
		// separator
		c.gridx = 1 ;
		add(new JLabel(label!=null && !label.trim().isEmpty() ? " : " : ""), c) ;
		
		// field
		c.gridx = 2 ;
		c.fill = GridBagConstraints.HORIZONTAL ;
		c.weightx = 1.0 ;
		add(component, c) ;
		
		fields.put(component, label) ;
	}
	
	/*
	 * Container
	 */
	@Override
	public void remove(Component comp)
	{
		fields.remove(comp) ;
		super.remove(comp) ;
	}
	
	@Override
	public void remove(int index)
	{
		fields.remove(getComponent(index)) ;
		super.remove(index) ;
	}
	
	@Override
	public void removeAll()
	{
		fields.clear() ;
		super.removeAll() ;
	}
	
	@Override
	public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled) ;
		
		for(JComponent comp : fields.keySet())
			comp.setEnabled(enabled) ;
	}
}