/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JIcon implements Icon
{
	final static public int ROTATE_LEFT = -1 ;
	final static public int ROTATE_NONE = 0 ;
	final static public int ROTATE_RIGHT = 1 ;
	
	final private Icon icon ;
	
	final private int rotation ;
	final private boolean isRotating ;
	
	private int width = 0 ;
	private int height = 0 ;

	public JIcon(Icon icon)
	{
		this(icon, ROTATE_NONE) ;
	}
	
	public JIcon(Icon icon, int rotation)
	{
		this.icon = icon ;
		this.isRotating = rotation==ROTATE_LEFT || rotation==ROTATE_RIGHT ;
		this.rotation = isRotating ? rotation : ROTATE_NONE ;
		
		computeUnits() ;
	}
	
	final public boolean isRotating()
	{
		return isRotating ;
	}
	
	final private void computeUnits()
	{
		int iconWidth = icon.getIconWidth() ;
		int iconHeight = icon.getIconHeight() ;
		
		width = isRotating ? iconHeight : iconWidth ;
		height = isRotating ? iconWidth : iconHeight ;
	}
	
	/*
	 * Icon
	 */
	final public int getIconHeight()
	{
		return height ;
	}

	final public int getIconWidth()
	{
		return width ;
	}

	final public void paintIcon(Component c, Graphics g, int x, int y)
	{
		Graphics2D g2 = (Graphics2D)g ;
		
		if(isRotating)
		{
			double rotationAngle = (Math.PI/2)*rotation ;
			
			int rotationX = rotation==ROTATE_LEFT ? x+width : x ;
			int rotationY = rotation==ROTATE_LEFT ? y+height : y ;
			
			g2.translate(rotationX, rotationY) ;
			g2.rotate(rotationAngle) ;
			icon.paintIcon(c, g2, 0, -icon.getIconHeight()) ;
			g2.rotate(-rotationAngle) ;
			g.translate(-rotationX, -rotationY) ;
		}
		else
		{
			icon.paintIcon(c, g, x, y) ;
		}
	}
}