/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.common.widget.JFilteredList.Filter;
import org.azuki.common.widget.model.ItemChooserTableModel;
import org.azuki.common.widget.model.ItemChooserTableModel.ItemSelection;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JItemChooser<T> extends JDialogPanel implements ActionListener
{
	protected ItemChooserTableModel<T> tableModel ;
	protected JTable table ;
	protected JScrollPane tableScroll ;
	protected JPanel tablePanel ;
	
	protected JButton filterButton ;
	protected JButton selectAllButton ;
	protected JPanel tableActions ;
	
	private Filter<T> filter = null ;
	private ListCellRenderer filterCellRenderer = null ;
	
	public JItemChooser()
	{
		this(new ItemChooserTableModel<T>("Empty", new ArrayList<T>())) ;
	}
	
	public JItemChooser(String label, List<T> list)
	{
		this(new ItemChooserTableModel<T>(label, list)) ;
	}
	
	public JItemChooser(String label, List<T> list, Boolean defaultSelected)
	{
		this(new ItemChooserTableModel<T>(label, list, defaultSelected)) ;
	}
	
	public JItemChooser(ItemChooserTableModel<T> tableModel)
	{
		super(new BorderLayout(5,5)) ;
		
		this.tableModel = tableModel ;
		
		table = new JTable(tableModel) ;
		table.getTableHeader().setReorderingAllowed(false) ;
		table.getTableHeader().setResizingAllowed(false) ;
		table.getColumnModel().getColumn(0).setMaxWidth(40) ;
		
		tableScroll = new JScrollPane(table) ;
		tableScroll.setPreferredSize(new Dimension(320,200)) ;
		
		filterButton = new JButton("Filter") ;
		filterButton.addActionListener(this) ;
		
		selectAllButton = new JButton("Select all") ;
		selectAllButton.addActionListener(this) ;
		
		tableActions = new JPanel(new FlowLayout(FlowLayout.RIGHT)) ;
		tableActions.add(filterButton) ;
		tableActions.add(selectAllButton) ;
		
		tablePanel = new JPanel(new BorderLayout()) ;
		tablePanel.add(tableScroll, BorderLayout.CENTER) ;
		tablePanel.add(tableActions, BorderLayout.SOUTH) ;
		
		add(tablePanel, BorderLayout.CENTER) ;
		
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
	}

	final public JTable getTable()
	{
		return table ;
	}
	
	final public void setItemSelectionMode(ItemSelection mode)
	{
		tableModel.setSelectionMode(mode) ;
	}
	
	final public boolean isSelectionValid()
	{
		return tableModel.isSelectionValid() ;
	}
	
	final public T getItem(int index)
	{
		return tableModel.getItem(index) ;
	}
	
	final public List<T> getItems()
	{
		return tableModel.getItems() ;
	}
	
	final public List<T> getSelectedItems()
	{
		return tableModel.getSelectedItems() ;
	}
	
	final public boolean isSelected(T item)
	{
		return tableModel.isSelected(item) ;
	}
	
	final public boolean isSelected(int index)
	{
		return tableModel.isSelected(index) ;
	}
	
	final public void setSelected(Collection<T> items, Boolean b)
	{
		tableModel.setSelected(items, b) ;
	}
	
	final public boolean isFilteringAllowed()
	{
		return filterButton.isVisible() ;
	}
	
	final public void setFilteringAllowed(boolean b)
	{
		filterButton.setVisible(false) ;
	}
	
	final public void setFilter(Filter<T> filter)
	{
		this.filter = filter ;
	}
	
	final public void setFilterCellRenderer(ListCellRenderer filterCellRenderer)
	{
		this.filterCellRenderer = filterCellRenderer ;
	}
	
	final public void addItemSelectionListener(TableModelListener l)
	{
		tableModel.addTableModelListener(l) ;
	}
	
	final public void removeItemSelectionListener(TableModelListener l)
	{
		tableModel.removeTableModelListener(l) ;
	}
	
	final public void addListSelectionListener(ListSelectionListener l)
	{
		table.getSelectionModel().addListSelectionListener(l) ;
	}
	
	final public void removeListSelectionListener(ListSelectionListener l)
	{
		table.getSelectionModel().removeListSelectionListener(l) ;
	}
		
	/*
	 * ActionListener
	 */
	final public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource() ; 
		
		if(source==selectAllButton)
		{
			setSelected(tableModel.getItems(), Boolean.TRUE) ;
		}
		else if(source==filterButton)
		{
			JFilteredList<T> filteredList = new JFilteredList<T>(tableModel.getItems()) ;
			if(filter!=null) 				filteredList.setFilter(filter) ;
			if(filterCellRenderer!=null) 	filteredList.setCellRenderer(filterCellRenderer) ;
			
			if(filteredList.showDialog(this, "Filtering list"))
				setSelected(filteredList.getSelectedItems(), Boolean.TRUE) ;
		}
	}
}