/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JTextIcon implements Icon
{
	final static private Font defaultFont = UIManager.getFont("Button.font") ;
	
	final static private boolean antialiased = Boolean.getBoolean("swing.aatext") ;
	
	final private Component component = new Container() ;
	private Font font ;
	
	private Color foreground = UIManager.getColor("Button.foreground") ;
	private Color background = UIManager.getColor("Button.background") ;
	
	final protected String text ;
	protected FontMetrics fontMetrics ;
	
	private boolean paintBackground = false ;
	
	private int width = 0 ;
	private int height = 0 ;

	public JTextIcon(String text)
	{
		this(text, defaultFont) ;
	}
	
	public JTextIcon(String text, Font font)
	{
		this.text = text ;
		this.font = font ;
		
		computeUnits() ;
	}
	
	final private void computeUnits()
	{
		fontMetrics = component.getFontMetrics(font) ;
		
		width = fontMetrics.stringWidth(text) ;
		height = fontMetrics.getAscent()+fontMetrics.getDescent() ;
	}
	
	final public Font getFont()
	{
		return this.font ;
	}
	
	final public void setFont(Font font)
	{
		this.font = font ;
		computeUnits() ;
	}
	
	final public Color getForeground()
	{
		return this.foreground ;
	}
	
	final public void setForeground(Color foreground)
	{
		this.foreground = foreground ;
	}
	
	final public Color getBackground()
	{
		return this.background ;
	}
	
	final public void setBackground(Color background)
	{
		this.background = background ;
	}
	
	final public void setBackgroundPainted(boolean b)
	{
		this.paintBackground = b ;
	}
	
	/*
	 * Icon
	 */
	final public int getIconHeight()
	{
		return height ;
	}

	final public int getIconWidth()
	{
		return width ;
	}

	final public void paintIcon(Component c, Graphics g, int x, int y)
	{
		Graphics2D g2 = (Graphics2D)g ;
		
		if(antialiased)
		{	g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON) ;
			g2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY) ; }
        
		Font oldFont = g2.getFont() ;
		Color oldColor = g2.getColor() ;
	
		if(paintBackground)
		{
			g2.setColor(background) ;
			g2.fillRect(x, y, width, height) ;
		}
		
		g2.setFont(font) ;
		g2.setColor(foreground) ;
		
		g2.drawString(text, x, y+fontMetrics.getAscent()) ;
		
		g2.setFont(oldFont) ;
		g2.setColor(oldColor) ;
	}
}