/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.field ;

import java.util.Vector;

import javax.swing.*;

import org.azuki.common.util.Crypto;
import org.azuki.common.widget.field.event.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JPasswordField extends javax.swing.JPasswordField implements IField
{
	final static private int defaultColumns = 15 ;
	
	protected String fieldName ;
	protected String initialValue ;
	
	private Vector<IFieldListener> listeners = new Vector<IFieldListener>() ;
	
	public JPasswordField()
	{
		this(null) ;
	}
	
	public JPasswordField(String text)
	{
		this(text, defaultColumns) ;
	}
	
	public JPasswordField(String text, int column)
	{
		super(null, text, column) ;
		
		this.initialValue = text ;
		
		setInputVerifier(new Verifier()) ;
		setEchoChar('*') ;
	}
	
	//FIXME: pas top --> on devrait passer par un listener ... quoique !!
	final public boolean hasValueChanged()
	{
		return !getFieldValue().equals(initialValue) ;
	}
	
	final public void notifyValueChanged()
	{
		fireValueChanged(new FieldEvent(this)) ;
		initialValue = getFieldValue() ;
	}
	
	final private void fireValueChanged(FieldEvent e)
	{
		for(IFieldListener l : listeners)
			l.valueChanged(e) ;
	}
	
	/*
	 * IField
	 */
	final public JComponent getUIComponent()
	{
		return this ;
	}
	
	final public void setInitialValue(String initialValue)
	{
		this.initialValue = initialValue ;
		setFieldValue(initialValue) ;
	}
	
	final public String getInitialValue()
	{
		return initialValue ;
	}
	
	final public void setFieldName(String fieldName)
	{
		this.fieldName = fieldName ;
	}

	final public String getFieldName()
	{
		return fieldName ;
	}

	final public void setFieldValue(String fieldValue)
	{
		setText(fieldValue) ;
	}
	
	final public String getFieldValue()
	{
//		return String.valueOf(getPassword()) ;
		
		String str = String.valueOf(getPassword()) ;
		try
		{ str = Crypto.cypher(str) ; }
		catch(Exception ex)
		{ ex.printStackTrace() ; }
		
		return str ;
	}

	final public void addFieldListener(IFieldListener l)
	{
		listeners.add(l) ;
	}
	
	final public void removeFieldListener(IFieldListener l)
	{
		listeners.removeElement(l) ;
	}
	
	/*
	 * Inner class
	 */
	protected class Verifier extends InputVerifier
	{
		@Override
		final public boolean verify(JComponent input)
		{
			JPasswordField field = (JPasswordField)input ;
			if(field.hasValueChanged())
				field.notifyValueChanged() ;
			return true ;
		}
	}
}