/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.model ;

import java.util.*;

import javax.swing.ListModel;
import javax.swing.event.*;

import org.azuki.common.widget.JFilteredList.Filter;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class FilteredListModel<T> implements ListModel
{
	final protected EventListenerList listenerList = new EventListenerList() ;
	
	final private List<T> originalList ;
	final private List<T> filteredList ;
	
	private String filterText = null ;
	
	private Filter<T> filter = null ;
	
	public FilteredListModel(List<T> list)
	{
		this.originalList = new ArrayList<T>(list) ;
		this.filteredList = new ArrayList<T>(list) ;
	}

	final public List<T> getFilteredItems()
	{
		return Collections.unmodifiableList(filteredList) ;
	}
	
	final public String getFilterText()
	{
		return filterText ;
	}
	
	final public void setFilter(Filter<T> filter)
	{
		this.filter = filter ;
	}
	
	final public void setFilterText(String filterText)
	{
		this.filterText = filterText ;
		
		filteredList.clear() ;
		
		if(filterText==null || filterText.trim().isEmpty())
		{
			filteredList.addAll(originalList) ;
		}
		else
		{
			for(T t : originalList)
			{
				if(filter!=null && filter.match(t, filterText))	filteredList.add(t) ;
				else if(t.toString().indexOf(filterText)!=-1) 	filteredList.add(t) ;
			}
		}
		
		modelChanged() ;
	}
	
	final protected void modelChanged()
	{
		ListDataEvent event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, -1, -1) ;
		
		ListDataListener[] arr = listenerList.getListeners(ListDataListener.class) ;
		for(ListDataListener ldl : arr)
			ldl.contentsChanged(event) ;
	}
	
	/*
	 * ListModel
	 */
	public int getSize()
	{
		return filteredList.size() ;
	}
	
	public T getElementAt(int index)
	{
		return filteredList.get(index) ;
	}

	final public void addListDataListener(ListDataListener l)
	{
		listenerList.add(ListDataListener.class, l) ;
	}
	
	final public void removeListDataListener(ListDataListener l)
	{
		listenerList.remove(ListDataListener.class, l) ;
	}
}