/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework;

import org.azuki.framework.bean.Bean;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class Framework
{
	final static public int MAJOR_VERSION = 1 ;
	final static public int MINOR_VERSION = 2 ;
	final static public int BUILD_VERSION = 0 ;

	final static public String getVersion()
	{
		return MAJOR_VERSION+"."+MINOR_VERSION+"."+BUILD_VERSION ;
	}

	static private Impl ifwk ;

	static boolean init = false ;
	final static public boolean inited() { return init ; }

	static
	{
		String fimp = null ;
		try
		{
			fimp = System.getenv("org.azuki.framework.implementation") ;
		}
		catch(Throwable t) { /* Maybe a security exception, don't care */}

		if(fimp==null)
			fimp = "org.azuki.kernel.framework.FwrkImpl" ;

		try
		{
			ifwk = (Impl)Class.forName(fimp).newInstance() ;
		}
		catch(Exception e)
		{
			throw new RuntimeException("Azuki Framework implementation instanciation error : "+Log.getStack(e)) ;
		}
	}

	public interface Impl
	{
		public void init(ClassLoader ccl)
		throws Exception ;

		public Bean.Impl     getBeanImpl() ;
		public Locator.Impl  getLocatorImpl() ;
	}

	final static public void init(ClassLoader ccl)
	throws Exception
	{
		if(init) return ;

		System.out.println();
		System.out.println("Starting Azuki framework/"+getVersion());
		System.out.println();

		try		{ ifwk.init(ccl) ; }
		finally { init = true ; }

		sendEvent(EVENT_AZUKI_STARTED) ;

		System.out.println("Azuki framework started.") ;
	}

	final static public void stop()
	{
		init = false ;
		sendEvent(EVENT_AZUKI_STOPPED) ;

		try
		{ Thread.sleep(2000) ; }
		catch(Exception ex)	{}

		System.out.println("Azuki framework stopped.") ;
	}

	final static private void sendEvent(String event)
	{
		try
		{
			IMethodProxy mp = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, "EventBasedProcessing", null, "publish") ;
			mp.call(event, new Long(System.currentTimeMillis())) ;
		}
		catch(Exception e)
		{
			e.printStackTrace()  ;
		}
	}

	final static public Bean.Impl     getBeanImpl()    { return ifwk.getBeanImpl(); }
	final static public Locator.Impl  getLocatorImpl() { return ifwk.getLocatorImpl(); }

	final static public String EVENT_AZUKI_STARTED = "AZUKI_STARTED" ;
	final static public String EVENT_AZUKI_STOPPED = "AZUKI_STOPPED" ;
	final static public String[] EVENTS = { EVENT_AZUKI_STARTED, EVENT_AZUKI_STOPPED } ;
}
