/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.context ;

import org.azuki.framework.object.ILifecycle;

/**
 * Interface for the command object in the Command Design Pattern.
 *      
 * @author Robert Bakic (robert.bakic@gmail.com)
 */
public interface IContext
{
	final static public String OBJECT_LOGGER 		= "logger" ;
	final static public String OBJECT_GLOBAL 		= "global" ;
	final static public String OBJECT_ANALYZER 		= "analyzer" ;
	final static public String OBJECT_LDAP			= "ldap" ;

	final static public String OBJECT_DATASOURCE 	= "datasource" ;
	final static public String OBJECT_CONNECTION 	= "connection" ;
	
	final static public String OBJECT_PERSISTENCEUNITNAME = "persistenceunitname" ;
	final static public String OBJECT_ENTITYMANAGER       = "entitymanager" ;
	
	final static public String OBJECT_HTTPREQUEST   = "httprequest" ;
	
	final static public String ATTR_USER 			= "user" ; // String
	
	public int getID() ;
	public int getProcessID() ;
	
	public boolean isBound(String key) ;
	
	// Simple Getters
	//no exeption, returns null
	// if no object bound	
	public Object getObject(String key) ;
	
	//
	// Getters with object creation if not bound
	public Object lookup(String key, Object... args)
	throws Exception ;

	// Returns the previously binded object or null.
	// ILifecicle parameter can be null (in this case the new context does not inherite)
	public IStub bind(String key, Object object, ILifecycle life) ;  
	
	public void unbind(String key) ;

	// Unbind the object (calling lifecycle end() method) and restore previously saved object if any.
	public void unbind(String key, IStub saved) ;
	
	public IContext fork() ;
	
	public interface IStub
	{
		public Object     getObject() ; 
		public ILifecycle getLifecycle() ;
	}
}