/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.context;

import org.azuki.framework.object.Locator;

/**
 *  Manage Thread's context attached object.
 * 
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */


final public class ThreadContext
{
	static private Impl imp = null;
	final static private Impl imp()
	{ 
		if(imp!=null) return imp ;

		try
		{
			imp = (Impl)Locator.create(Locator.OBJECT_CONTEXT) ;
			if(imp==null)
				throw new Exception("Mandatory context management implementation is missing : azuki.xml may be not found or does not contains the kernel universe.") ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
		}
		return imp ;
	}
	
	
	final static public IContext get()
	{
		return imp().getContext() ;
	}

	
	final static public void release(String key)
	throws Exception
	{
		imp().getContext().unbind(key) ;
	}
	
	// short for ThreadContext.get().lookup() !
	final static public Object lookup(String key, Object... args)
	throws Exception
	{
		return imp().getContext().lookup(key, args) ;
	}

	final static public void set(IContext ictx)
	{
		imp().setContext(ictx) ;
	}
	
	final static public void startThread(Thread thread)
	{
		imp().startThread(thread) ;
	}
	
	final static public void setStatus(boolean state, String info) // todo : enum
	{
		imp().setStatus(state, info) ;
	}

	// Call this method when we do not manage thread lifecycle
	final static public void reset()
	{
		imp().reset();
	}	

	public interface Impl
	{
		public IContext getContext() ;
		public void setContext(IContext ictx) ;
		
		public void startThread(Thread thread) ;
		public void setStatus(boolean state, String info) ;		
		
		public void reset() ;
	}		
}