/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.injection;


/**
 *  Allows a code advice to intercept a method call when the caller use the 
 *  MethodProxy class.
 *
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

public interface IController<T>
{
	/**
	 * This method is called by the framework before the targeted bean's method.
	 * Developpers can place here code in order to change the arguments of the method,
	 * to control the bean acces or to log the action.
	 * Multiple code advice applied to the targeted bean can implements this interface.
	 * In this case all the <code>intercept</code> method will be call in turn, folowing the 
	 *  chain of control design pattern.
	 *  
	 * @return The result of the targeted method invocation.
	 */	
	public T intercept(InvocationTarget<T> target)
	throws Exception ;
}
