/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.metadata ;

import java.lang.reflect.*;
import java.util.*;

import org.azuki.common.consts.*;
import org.azuki.common.util.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

// Point Cut

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

public class PC implements IPointcut, Comparable<PC>
{	
	// PC
	final private Integer pcid  ; // Poincut ID
	final public  Integer getPointcutID() { return pcid  ; } 

	final Element lmen ;
		
	final public IMetadata getAspectMetadata()
	{
		try
		{
			return getAspMeta() ;
		}
		catch(Exception e) {} // Undefined
		return null ;
	}
	
	private IMetadata ameta ; // Use only this getter to acces this field.
	final private IMetadata getAspMeta() 
	throws Exception
	{
		if(ameta!=null) return ameta ;
		
		// Initialized later cause of wrapper creation
		ameta  = Bean.searchFirst(aspname, asplabl) ;
		if(ameta==null)
		{
			StringBuilder sb = new StringBuilder("[ERROR] Poincut init (").append(pcid).append(',')
			.append(Util.oneString(aspname, asplabl)).append(") for bean : ")
			.append(Util.oneString(tmeta.getName(), tmeta.getLabel())) ;

			throw new Exception(sb.toString()) ;
		}
		return ameta  ;
	}

	final private IMetadata tmeta ;
	final public  IMetadata getTargetMetadata() { return tmeta ; }
	 	
	private Object advice ;
	final public Object getAdvice() { return advice; } 
	
	private Field[]  fds ;
	private Method[] mts ;

	final public Method[] getSelectedMethods() { return mts; }
	final public Field[]  getSelectedFields()  { return fds; }
	
	final private String aspname ;
	final private String asplabl ;

	public PC(int pcid, String aspname, String asplabl, IMetadata tmeta, Element lmen)
	{
		this.pcid       = new Integer(pcid) ;
		this.aspname    = aspname ;
		this.asplabl    = asplabl ;
		this.tmeta      = tmeta ;
		this.lmen       = lmen ;
	}

	final static private Element LAZY = new Element("LAZY") ;


	final public Element getElement()
	{ 
		return lmen ; 
	}	
	
	private Element para = LAZY ;	
	final public  Element getParameter()
	{ 
		if(para==LAZY) para = lmen.getChild(Tag.PARA) ;
		return para ; 
	}	

	private Map<String, Element> fdslmen = null ;		
	final public Element getElement(Field field)
	{
		if(fdslmen==null) initFieldsElement() ;
		return fdslmen.get(field.getName()) ;
	}
	
	@SuppressWarnings("unchecked")
	final private void initFieldsElement()
	{
		fdslmen = new HashMap<String,Element>() ;
		
		List<Element> lmt = lmen.getChildren(Tag.FIELD) ;
		for(Element mt : lmt)
		{
			String m = mt.getAttributeValue(Attr.NAME) ;
			Log.info("\t\t on field ["+m+"]") ;
			fdslmen.put(m, mt);
		}
	}
	
	private Map<String, Element> mtslmen = null ;		
	final public Element getElement(Method method)
	{
		if(mtslmen==null) initMethodsElement() ;
		return mtslmen.get(method.getName()) ;
	}
	
	@SuppressWarnings("unchecked")
	final private void initMethodsElement()
	{
		mtslmen = new HashMap<String,Element>() ;
		
		List<Element> lmt = lmen.getChildren(Tag.METHOD) ;
		for(Element mt : lmt)
		{
			String m = mt.getAttributeValue(Attr.NAME) ;
			Log.info("\t\t on method ["+m+"]") ;
			mtslmen.put(m, mt);
		}		
	}
	

	final public Object createAdv() 
	{
		try
		{	
			getAspMeta() ; // Initialisation, do not move
			
			StringBuilder sb = new StringBuilder("  +----(").append(pcid).append(',')
			.append(Util.oneString(aspname, asplabl)).append(")");
			
			Log.info(sb.toString()) ;
	
			Object aspect = ameta.getBean() ;
			
			if(aspect==null)
			{	Log.info("Aspect implementation not found ("+ameta+")");
				return null ; }	
			
			IAspect aspt = null ;

			try
			{
				aspt = (IAspect)aspect ;
			}
			catch(ClassCastException cce)
			{
				throw new Exception("Bean ("+ameta+") is not a aspect") ;
			}
			
			Class bc = tmeta.getBean().getClass() ;

			if(fdslmen==null) initFieldsElement() ;
			if(mtslmen==null) initMethodsElement() ;
			
			fds = getSelectedFields(bc, fdslmen.keySet()) ;
			mts = getSelectedMethods(bc.getMethods(), mtslmen.keySet()) ;

			advice =  aspt.createAdvice(this) ;		
		}
		catch(Throwable e)
		{
			Log.severe("[ERROR] Poincut ("+pcid+") cannot be created for bean : "+getTargetMetadata().getID()) ;
			Log.severe(Log.getStack(e)) ;
		}
		
		return advice ; // Advice can be null, if problem occured
	}
	
	final static public Field[] getSelectedFields(Class c, Collection<String> fds)
	{
		if(c==null)   return new Field[0] ;
		if(fds==null) return new Field[0] ;
							
		ArrayList<Field> alf = new ArrayList<Field>() ;
			
		for(String fd : fds)
		{
			try
			{
				Field field = c.getField(fd) ;

				if(!Modifier.isPublic(field.getModifiers()))
					continue ;
				
				alf.add(field) ;
			}
			catch(Exception e)
			{ 
				// Non present
			}
		}
			
		Field[] fields = new Field[alf.size()] ;
		alf.toArray(fields) ;
		return fields ;
	}
	
	final static public Method[] getSelectedMethods(Method[] meths, Collection<String> mts)
	{
		if(meths==null) return new Method[0] ;
		if(mts  ==null) return new Method[0] ; 
		
		List<Method> sel = new ArrayList<Method>() ;

		for(String mt : mts)
		{
			for(Method meth : meths)
			{
				if(mt.equals(meth.getName()))
				{
					if(!Modifier.isPublic(meth.getModifiers())) continue ;	
					sel.add(meth) ;
					break ;
				}
			}
		}

		Method[] methods = new Method[sel.size()] ;
		sel.toArray(methods) ;
		return methods ;
	}
	
	//
	// Comparable
	//
	final public int compareTo(PC pc)
	{
		if(aspname.equals(pc.aspname)) return 0 ;
		
		int pty1 = Topology.getPriority(aspname) ;
		int pty2 = Topology.getPriority(pc.aspname) ;
		
		return pty1-pty2 ;
	}		
}