/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.metadata;

import java.util.*;

import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.injection.IController;

// PointCut Definition Container

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class PCC
{
	// All into map for overloading
	final private HashMap<Integer, PC> pcs = new HashMap<Integer, PC>(); 
	
	// Controllers only
	private IController[] ctrls = new IController[0] ;
		
	
	final public void addPointcut(PC ipc)
	{
		pcs.put(ipc.getPointcutID(), ipc) ;
	}
	
	// IMetadata
	final public IPointcut getPointcut(Integer pcid)	{ return pcs.get(pcid); }
	
	final public List<IPointcut> getPointcuts()
	{ 
		List<IPointcut> lpc = new ArrayList<IPointcut>() ;
		
		for(IPointcut ipc : pcs.values())
			lpc.add(ipc) ;
	
		return lpc;
	}
	

	@SuppressWarnings("unchecked")
	final public List<IPointcut> getPointcuts(Integer aspid)
	{
		List lpc = new ArrayList<PC>() ;
		
		for(PC ipc : pcs.values())
		{
			IMetadata ameta  = ipc.getAspectMetadata() ;
			if(ameta==null) continue ;
			if(ameta.getID().equals(aspid)) 
				lpc.add(ipc) ;
		}
		
		return lpc;
	}

	final public List<IPointcut> getPointcuts(String name)
	{ 
		List<IPointcut> lpc = new ArrayList<IPointcut>() ;
		
		for(IPointcut ipc : pcs.values())
		{			
			IMetadata ameta  = ipc.getAspectMetadata() ;
			if(ameta==null) continue ;
			if(name.equals(ameta.getName()))
				lpc.add(ipc) ;
		}
		
		return lpc;
	}
	
	public IController[] getControllers()
	{
		return ctrls ;
	}
		
	// Only after bean instances has been created
	public void createAdvices()
	{
		// PointCuts must be sorted first !
		
		ArrayList<PC> spcs = new ArrayList<PC>() ; 
		for(PC pC : pcs.values())
			spcs.add(pC) ;
		
		Collections.sort(spcs) ;
		
		ArrayList<IController> lc = new ArrayList<IController>() ;

		for(PC pC : spcs)
		{
			Object advice = pC.createAdv() ;

			if(advice==null) 
			{
				// Le composant ne rempli pas le contrat 
				// TODO : Mettre un statut pour signifier que l'ASPECT
				// n'a pas ete appliqué ?
				continue ;
			}
			
			if(advice instanceof IController)
			{
				lc.add((IController)advice) ;
				
				// Dans la boucle, pour contrtoller dispo pour autre advice
				ctrls = lc.toArray(new IController[lc.size()]) ;
			}
		}
	}
}