/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.metadata ;

import java.util.*;

import org.azuki.common.dag.*;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

// Table of bean wrappers 
// provides lookup services for finding bean 

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class TO
{
	// Bean / Wrapper
	final static private ArrayList<BW> lball = new ArrayList<BW>();	// List of all bean wrapper
	final static public Collection<BW> getAll() { return lball; }
	
	final static public List<BW> getBeanFromUniverse(String univ)
	{
		DAG<BW>  dag    = new DAG<BW>() ;
		List<BW> lbuniv = new ArrayList<BW>() ; // List of universe's bean 
		
		// les dependances liées à l'AOP
		addAspectDependencies(dag, univ, lbuniv) ;
		
     	 // Pour tester la cycling exception
		 //
//		 tp.addEdge("AppPortal:Console", null, "RequestDispatcher:main", null) ;
//		 tp.addEdge("AppPortal:Console", null, "AppPortal:Console", null) ;	 
		
		if(dag.hasCycle()) 
		{
			Log.severe("============================");
			Log.severe("= TOPOLOGICAL SORT FAILED ==");
			Log.severe("============================");
			Log.severe("") ;
			Log.severe("== PROBLEM MUST BE SOLVED IN THE LINK ABOVE ==");
			List<Node<BW>> ll = dag.getCycle() ; 
			StringBuilder sb = new StringBuilder() ;
			boolean one = false ;
			for(Node<BW> node : ll)
			{
				if(!one) one = true ;
				else	 sb.append(" --> ") ;
				sb.append(node.getKey()) ;
			}
			Log.severe(sb.toString()) ;
			
			Log.severe("=============================================");
			Log.severe("  OBJECT INITIALISATION IS NOT SORTED !!!!   ");
			Log.severe("  THIS MAY CAUSE INITIALIZATION PROBLEM      ");
			Log.severe("=============================================");
		}
		else
		{
			// OK pour les aspect, on ajoute les dependances liees à l'injection 
			addInjectionDependencies(dag, univ, lbuniv) ;
			
			try
			{
				Log.info("=====================");
				Log.info("= TOPOLOGICAL SORT ==");
				Log.info("=====================");
				List<Node<BW>> nodes = dag.sort() ;
			
				List<BW> lbsorted = new ArrayList<BW>() ; // list of bean, sorted
				
				for(Node<BW> node : nodes)
				{
					Log.info(node.getKey());
					lbsorted.add(node.getObject());
				}

				// Adding not sorted BW (because not in the dag)
				for(BW bw : lbuniv)
					if(!lbsorted.contains(bw)) lbsorted.add(bw) ;
						
				return lbsorted ;
			}
			catch(Exception e)
			{
				Log.severe(e) ;
			}
		}
		
		return lbuniv ; // Unsorted
	}
	
	@SuppressWarnings("unchecked")
	final static private void addAspectDependencies(DAG dag, String univ, List<BW> lbuniv)
	{
		for(BW bw : lball)
		{
			if(!univ.equals(bw.getUniverse())) continue ;
				
			lbuniv.add(bw) ;

			List<IPointcut> lpc = bw.getPointcuts() ;
			 
			 for(IPointcut pc : lpc)
			 {
				 BW bwa = (BW)pc.getAspectMetadata() ;
	    		 if(bwa==null) continue ; //instance sans doute disable
				 
				 if(!univ.equals(bwa.getUniverse())) continue ;  // Verifier l'univers
				 
				 String parent = Util.oneString(bwa.getName(), bwa.getLabel()) ;
				 String child = Util.oneString(bw.getName(), bw.getLabel()) ;
				 
				 if(Topology.isUnchecked(parent, child)) continue ;
				 
				 dag.addEdge(parent, bwa, child, bw) ;
			 }
		}		
	}
	
	@SuppressWarnings("unchecked")
	final static private void addInjectionDependencies(DAG dag, String univ, List<BW> lbuniv)
	{		
		// Dependances liees a l'injection de methodes
    	/*
    	<pointcut name="MethodProxy">
        	<field name="addConnector">
        	    <para>
            	     <proxy name="TomcatEngine" labl="Engine" meth="addConnector" />
            	</para>
           	</field>
         </pointcut>
    	 */
		for(BW bw : lbuniv)
		{
			List<IPointcut> lpc = bw.getPointcuts("MethodProxy") ;

		    String child = Util.oneString(bw.getName(), bw.getLabel()) ;
			
		    for(IPointcut pc : lpc)
			{
		    	Element lmen = pc.getElement() ;
		    	if(lmen==null) continue ;
		    	
		    	List<Element> ls = lmen.getChildren("field") ;
		    	for(Element ef : ls)
		    	{
		    		ef = ef.getChild("para") ;
		    		if(ef==null) continue ;
		    	
		    		ef = ef.getChild("proxy") ;
		    		if(ef==null) continue ;
		    		
		    		String name = ef.getAttributeValue("name") ;
		    		String labl = ef.getAttributeValue("labl") ;

		    		String parent = Util.oneString(name, labl) ;
					
		    		try
		    		{
		    			BW bsrc = (BW)Bean.searchFirst(name, labl) ;
		    			if(bsrc==null) continue ; //instance sans doute disable
		    				
		    			if(!univ.equals(bsrc.getUniverse())) continue ;  // Important : Verifier l'univers
		    			
		    			boolean added = dag.addEdgeIfNoCycle(parent, bsrc, child, bw) ;
		    			
		    			if(added) Log.info("Adding injection dependency : " +child+ " <-- " + parent) ;
		    			else      Log.info("Skip   injection dependency : " +child+ " <-- " + parent) ;
		    		}
		    		catch(Exception e)
					{
						Log.severe(e) ; 
					}
		    	}	
			}
		}			
		
		Log.info(""); // newline			
	}
	
	
	
	final static public void add(BW bw)
	{
		lball.add(bw);
	}

	// Returns null instead of BeanNotFoundException
	final static public BW getWrapper(Object bean)
	{
		if(bean==null) return null ;
		
		for(BW bw : lball)
		{
			Object b = bw.getBean() ;
			if(b==null) continue; // Instanciation failure
			if(bean.equals(b)) return bw ;		
		}
		return null ;
	}
	
	final static public BW lookup(Integer bid)
	{
		if(bid==null) return null ;

		for(BW bw : lball)
		{
			if(bid.equals(bw.getID()))
				return bw ;
		}		
		return null ;
	}

	// remarque : labl can be null (search by name only), "" is a valid labl value
	@SuppressWarnings("unchecked")
	final static public Collection<IMetadata> search(String name, String labl) 
	{
		if(name==null) return Collections.EMPTY_LIST ;

		Collection<IMetadata> res = new ArrayList<IMetadata>() ;
		
		for(IMetadata bw : lball)
		{
			if(name.equals(bw.getName()))
			{
				if(labl==null) res.add(bw) ; // search by name only
				else
				{
					if(labl.equals(bw.getLabel()))
						res.add(bw) ;
				}
			}
		}
		
		return res;	
	}

	// remarque : labl can be null (search by name only), "" is a valid labl value
	final static public IMetadata searchFirst(String name, String labl) 
	{
		if(name==null) return null ;

		for(IMetadata bw : lball)
		{
			if(name.equals(bw.getName()))
			{
				if(labl==null) return bw ; 
				
				if(labl.equals(bw.getLabel()))
					return bw ;
			}
		}
		
		return null;	
	}
}