/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.weaving ;

import java.io.InputStream;
import java.util.*;

import org.azuki.common.consts.*;
import org.azuki.common.util.Util;
import org.azuki.framework.context.Log;
import org.azuki.kernel.metadata.*;
import org.jdom.*;
import org.jdom.input.SAXBuilder;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class P1Universe
{
	final static public void init(InputStream in)
	{
		try
		{
			Document doc = new SAXBuilder().build(in);		
			
			Element root = doc.getRootElement() ; // Tag.WEAVING
		
			// Sorted
			Collection<List<Element>> lms = selectUniverse(root) ;

			for(List<Element> llm : lms) // run-level
			{
				for(Element lm : llm) // universe
					initUniverse(lm) ;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
		}		
	}
	
	@SuppressWarnings("unchecked")
	final static private Collection<List<Element>> selectUniverse(Element root)
	{
		TreeMap<Integer, List<Element>> tms = new TreeMap<Integer, List<Element>>() ; // To sort by run-level

		// Get valid universe and sort into tree map
		List<Element> list = root.getChildren(Tag.UNIVERSE) ;
		for(Element lm : list)
		{
			String univ = lm.getAttributeValue(Attr.NAME) ;
			String enab = lm.getAttributeValue(Attr.ENAB, Const.YES) ; // Default = ok
			boolean uena = Util.isOK(enab) ;

			if(!uena)
			{	Log.info("Found universe ["+univ+"] ....... disable.") ;
				continue ; }
			
			String slvl = lm.getAttributeValue(Attr.RLVL) ;
			
			int i = Const.I_DEF_RUN_LEVEL ;
			if(slvl!=null && !slvl.isEmpty())
				i = Integer.parseInt(slvl) ;
			
			Integer rlvl = new Integer(i) ;
			Log.info("Found universe ["+univ+"] at run level : "+rlvl) ;
			
			List<Element> tmp = tms.get(rlvl) ;
			if(tmp==null)
			{
				tmp = new ArrayList<Element>() ;
				tms.put(rlvl, tmp) ;
			}
			else
				Log.warning("<<<<<<< WARNING : Two universes have the same run-level : "+rlvl+" >>>>>>>") ;

			
			tmp.add(lm) ;
		}		
		
		return tms.values() ;
	}
	
	final static private void initUniverse(Element lm)
	{
		String univ = lm.getAttributeValue(Attr.NAME) ;

		Log.info("\n") ;
		printu(univ) ;

		printe("DEFINITION") ;

		P2InstanceDefinition.init(lm, univ) ;
			
		List<BW> lwrap = TO.getBeanFromUniverse(univ) ;
		
		printe(" INSTANCE ") ;
		
		for(BW owrap : lwrap) owrap.createInstance() ;
		
// 		printe(lgr, "POSTCREATE") ;
//		for(BW owrap : lwrap) owrap.postCreate(lgr) ;
		
		printe(" POINTCUT ") ;

		// Create advices instances...
		for(BW owrap : lwrap) owrap.createAdvices() ;
	}
	
	final static private void printu(String univ)
	{
		Log.info("=================================================================== ") ;
		Log.info("=========================== START Universe ("+univ+")") ;
		Log.info("=================================================================== ") ;
	}

	final static private void printe(String txt)
	{
		Log.info("\n") ;
		Log.info("==============") ;
		Log.info(new StringBuilder("= ").append(txt).append(" =").toString()) ;
		Log.info("==============\n") ;
	}	
}