/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.weaving ;

import java.util.*;

import org.azuki.common.consts.*;
import org.azuki.common.framework.*;
import org.azuki.common.util.Util;
import org.azuki.framework.context.Log;
import org.azuki.kernel.metadata.*;
import org.jdom.Element;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class P2InstanceDefinition
{
	@SuppressWarnings("unchecked")
	final static public void init(Element lm, String univ)
	{
		List<Element> list = lm.getChildren(Tag.SCHEMA) ;
		for(Element lm2 : list)
			initSchema(univ, lm2) ;
	}
	
	@SuppressWarnings("unchecked")
	final static private void initSchema(String univ, Element lmen)
	{
		String sche = lmen.getAttributeValue(Attr.NAME) ;
		
		String enab  = lmen.getAttributeValue(Attr.ENAB, Const.YES) ; // Default = ok		
		boolean bena = Util.isOK(enab) ; 

		if(!bena)
		{
			Log.info("~~ Schema ("+sche+") ~~") ;
			Log.info("   ...... disable.\n") ;
			return ;
		}
 
		Log.info("~~ Schema ("+sche+") ~~\n") ;
		
		List<Element> list = lmen.getChildren(Tag.INSTANCE) ;

		for(Element lm : list)
			initBean(univ, sche, lm) ;		
	}
		
	static private int countbid = 0 ;
	
	final static public void initBean(String univ, String sche, Element lmen)
	{
		try
		{
			Map<String, String> attrs = Util.attrToMap(lmen) ;
					
			String name = attrs.get(Attr.NAME) ;
			String labl = attrs.get(Attr.LABL) ;
			if(labl==null) labl = "" ; // No label
			
			Integer bid = new Integer(++countbid) ;
			
			String vers = attrs.get(Attr.VERS) ;
			
			StringBuilder sb = new StringBuilder("XML instance [").append(bid).append(',')
			.append(Util.oneString(name, labl)).append("]") ;

			Log.info(sb.toString()) ;
	
			String sena = attrs.get(Attr.ENAB) ;

			boolean bena = true ; // Default = ok
			if(sena!=null)
				bena = Util.isOK(sena) ;

			if(!bena)
			{
				Log.info("     DISABLE ["+bid+"]\n") ;
				return ;
			}
			
			BeanDefinition bdef = BeanRepository.instance().find(name) ;
	
			if(bdef==null)
				throw new Exception("Undefined bean : " + name) ;
	
			if(!vers.equals(bdef.getVersion()))
				Log.warning("Warning not the same version") ;
			
			BW owrap = new BW(bid, name, labl, bdef, univ, sche/*, rlvl*/) ;
				
			// PointCuts
			PPointCut.definePointcuts(owrap, lmen) ;
			
			Log.info("          OK ["+bid+"]\n") ;
				
			TO.add(owrap) ; 
		}
		catch(Exception e)
		{
			Log.severe("initBean() "+Log.getStack(e)) ;
		}
	}			
}